// This loads defines a clipping plane according to the wand position
//
// DIVERSE graphics interface -- Performer ---- version 2.2.0
// released: Fri Oct 18 09:27:29 EDT 2002
// The DIVERSE HOMPAGE is at: www.diverse.vt.edu
// 
// Copyright (C) 2000, 2001, 2002  John Kelso, Lance Arsenault and Ron Kriz
// 
// This software, DIVERSE graphics interface -- Performer (DGIPF),
// is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License (LGPL) as
// published by the Free Software Foundation; either version 2.1 of
// the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this software, in the top level source directory in
// a file named "COPYING"; if not, see it at:
// http://www.gnu.org/copyleft/lesser.html or write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
//#include <fstream.h>

static double* planeEq;
static double* planeEq2;
static double* planeEq3;
static double* planeEq4;
static double* planeEq5;
static double* planeEq6;
// shared memory for plane normals (from wand) - 6 directions
dtkSharedMem* shmSlice;
// shared memory for place distance from wand
dtkSharedMem* shmDist6out;
dtkSharedMem* shmDist6in;
//  Shared memory segment for the on-off button
dtkSharedMem* shm2;
//  Shared memory segment for the spin rate
dtkInLocator *wand ;
// Shared memory for wandPointer
dtkSharedMem* shm4;
// Shared memory for which planes are on
dtkSharedMem* shmPlanesOn;
unsigned char selected;
float valD,distC;
float vals[18];
float dist6in[6];
float dist6out[6];
unsigned char pon[6];
unsigned char onoff;
int c;

class clipWand : public dtkAugment {
public:

  clipWand(dpf *app) ;
  int preConfig(void);
  int postConfig(void);
  int preFrame(void);
  int postFrame(void);
  ~clipWand();

private:
  dpf* app;
  dpfDisplay *display;
  float wandData[6];
  pfCoord wandCoord;
  pfCoord wandCoord2;
  float planePos[6];
  float distcheck;
  float distcheck2;
  pfVec3 check;
  pfVec3 check2;
  pfVec3 upVector;
} ;

//  Enable the clipWand plane in pre draw
int preDraw (pfTraverser *trav, void *data) {

  // first see if the on button is set to on
  if (shm2) {
    shm2->read(&onoff);
  }

  if (onoff == 1) {
    if (shmSlice)
      shmSlice->read(vals);
    if (shmDist6out)
      shmDist6out->read(dist6out);
    if (shmPlanesOn)
      shmPlanesOn->read(&pon);
    
    static void* firstChan = NULL;
    if (firstChan == NULL) 
      firstChan = trav->getChan();
    
    if (firstChan == trav->getChan()) {
      //  Set up the coefficients in the plane equation
      //  Ax + By + Cz = D
      
      //  A,B,C correspond to the normal vector coefficients
      for (c=0; c<3; c++) {
	planeEq[c] = -vals[c];
	planeEq2[c] = -vals[c+3];
	planeEq3[c] = vals[c+6];
	// plane 2 points out in the opposite direction
	planeEq4[c] = -vals[c+6];
	planeEq5[c] = -vals[c+12];
	planeEq6[c] = -vals[c+15];
      }

      //  Plug in position to solve for D
      planeEq[3] = dist6out[0];
      planeEq2[3] = dist6out[1];
      planeEq3[3] = -dist6out[2];
      planeEq4[3] = dist6out[3];
      planeEq5[3] = dist6out[4];
      planeEq6[3] = dist6out[5];
    }

    //printf ("enabling clipWand plane, channel %x\n",trav->getChan());
    // plane 0=+x; plane 1=-x; 2=+y; 3=-y; 4=+z; 5=-z
    glClipPlane(GL_CLIP_PLANE0, planeEq);
    glClipPlane(GL_CLIP_PLANE1, planeEq2);
    glClipPlane(GL_CLIP_PLANE2, planeEq3);
    glClipPlane(GL_CLIP_PLANE3, planeEq4);
    glClipPlane(GL_CLIP_PLANE4, planeEq5);
    glClipPlane(GL_CLIP_PLANE5, planeEq6);
    if (pon[0] == 1)
      glEnable(GL_CLIP_PLANE0);
    if (pon[1] == 1)
      glEnable(GL_CLIP_PLANE1);
    if (pon[2] == 1)
      glEnable(GL_CLIP_PLANE2);
    if (pon[3] == 1)
      glEnable(GL_CLIP_PLANE3);
    if (pon[4] == 1)
      glEnable(GL_CLIP_PLANE4);
    if (pon[5] == 1)
      glEnable(GL_CLIP_PLANE5);
  }
  else {
    glDisable(GL_CLIP_PLANE0);
    glDisable(GL_CLIP_PLANE1);
    glDisable(GL_CLIP_PLANE2);
    glDisable(GL_CLIP_PLANE3);
    glDisable(GL_CLIP_PLANE4);
    glDisable(GL_CLIP_PLANE5);
  }

  return PFTRAV_CONT;
}

//  Disable the clip plane in post draw
int postDraw (pfTraverser *trav, void *data) {
  //printf ("disabling clip plane, channel %x\n", trav->getChan());
  glDisable(GL_CLIP_PLANE0);
  glDisable(GL_CLIP_PLANE1);
  glDisable(GL_CLIP_PLANE2);
  glDisable(GL_CLIP_PLANE3);
  glDisable(GL_CLIP_PLANE4);
  glDisable(GL_CLIP_PLANE5);

  return PFTRAV_CONT;
}

// create object and set description.
// be sure to validate it!
clipWand::clipWand(dpf *app_in) :
  dtkAugment("clipWand")
{
  app = app_in;
  setDescription("clipping plane") ;
  validate();
  planePos[0] = planePos[1] = planePos[2] = planePos[3] = 
    planePos[4] = planePos[5] = 0.0f ;
} ;


clipWand::~clipWand()
{
   delete shmSlice;
   delete shmDist6in;
   delete shmDist6out;
   delete shmPlanesOn;
   delete shm2;
   delete shm4;
}

// called only once by dpf, after pfInit() and dpfDisplay:preConfig()
// but before pfConfig()
int clipWand::preConfig(void) {
  //  Allocate plane equation out of Performer shared memory
  planeEq = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());
  planeEq2 = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());
  planeEq3 = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());
  planeEq4 = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());
  planeEq5 = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());
  planeEq6 = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());

  shmSlice = new dtkSharedMem(18 * sizeof(float), "vectors6");
  // distances from wand to plane for the 6 planes - incoming data
  shmDist6in = new dtkSharedMem( 6*sizeof(float), "dval6");
  // space for plane d values
  shmDist6out = new dtkSharedMem( 6*sizeof(float), "dval-out6");
  shmPlanesOn = new dtkSharedMem (6*sizeof(unsigned char), "planeson");

  //  Check if there is a segment "doclip" the on-off button
  shm2 = new dtkSharedMem(sizeof(unsigned char), "doclip");
  if(!shm2 || shm2->isInvalid()) {
      shm2 = NULL;
  }

  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand || wand->isInvalid())
    wand = NULL ;

  shm4 = new dtkSharedMem(sizeof(unsigned char), "pointerOn");
  if(!shm4 || shm4->isInvalid()) {
    shm4 = NULL;
  }

  // turn button off to start
  selected = 0;
  if (shm2)
     shm2->write(&selected);
  
  // start out by hiding the wandPointer
  if (shm4) {
    selected = 1;
    shm4->write(&selected);
  }

  // start out initializing all planes off, and all values = 0;  
  for ( int i = 0; i < 6; ++i ) {
    pon[i] = 0;
    dist6in[i] = 0.0;
  }

  // read in current pon settings from wandClip if available
  if (shmPlanesOn)
    shmPlanesOn->write(&pon);

  if (shmDist6in)
    shmDist6in->write(&dist6in);

  return DTK_CONTINUE ;
}


int clipWand::preFrame(void)
{
  display = app->display();
  if(!display) return DTK_CALLBACK_ERROR;

  return DTK_REMOVE_CALLBACK;
}

int clipWand::postConfig(void) {

  //  Set the preDraw and postDraw functions set up the clipping planes
  app->display()->world()->setTravFuncs (PFTRAV_DRAW, preDraw, postDraw);
  return DTK_REMOVE_CALLBACK ;
}

int clipWand::postFrame(void) {
  
  // read the wand direction
  if (wand)
    wand->read(wandData);

  // define the wand data in world coordinates
  wandCoord.xyz.set(wandData[0], wandData[1], wandData[2]);
  wandCoord.hpr.set(wandData[3], wandData[4], wandData[5]);
  display->dpfToWorld(&wandCoord);

  // create a matrix from the HPR position in world coordinates
  pfMatrix wandWorld;
  wandWorld.makeEuler(wandCoord.hpr[0],wandCoord.hpr[1],wandCoord.hpr[2]);
  
  // read in all the distances from the wand to all of the planes
  if (shmDist6in)
      shmDist6in->read(&dist6in);

  // set up each plane by extending a vector in each direction

  // create plane 1 - +x direction
  // create a vector (check) in the +x direction 
  upVector.set(1.f, 0.f, 0.f);
  check.xformVec(upVector, wandWorld);

  // find the absolute distance of the direction vectors
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);
  vals[0] = check[0];
  vals[1] = check[1];
  vals[2] = check[2];
  valD = dist6in[0];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[0] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  // create plane 2 in the -x direction
  upVector.set(-1.f, 0.f, 0.f);
  check.xformVec(upVector, wandWorld);

  // find the absolute distance of the direction vectors
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);
  vals[3] = check[0];
  vals[4] = check[1];
  vals[5] = check[2];
  valD = dist6in[1];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[1] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  // plane 3 is the +y direction
  // create a vector in the +y direction and move according to the wand
  upVector.set(0.f, 1.f, 0.f);
  check.xformVec(upVector, wandWorld);

  // find the absolute distance of the direction vectors
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);

  //write check values to shmSlice for plane 3 normal
  vals[6] = check[0];
  vals[7] = check[1];
  vals[8] = check[2];
  valD = dist6in[2];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[2] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  // create plane 4 (-y)  
  // create a vector in the -y direction for the plane normal
  // find the point in the (0,1,0) direction that is plane 4 distance
  valD = dist6in[3];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[3] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];
  upVector.set(0.f, -1.f, 0.f);
  check.xformVec(upVector, wandWorld);
  vals[9] = check[0];
  vals[10] = check[1];
  vals[11] = check[2]; 

  // plane 5 is the +z direction
  // create a vector in the +y direction and move according to the wand
  upVector.set(0.f, 0.f, 1.f);
  check.xformVec(upVector, wandWorld);

  // find the absolute distance of the direction vectors
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);

  //write check values to shmSlice for plane 3 normal
  vals[12] = check[0];
  vals[13] = check[1];
  vals[14] = check[2];
  valD = dist6in[4];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[4] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  // plane 6 is the -z direction
  // create a vector in the +y direction and move according to the wand
  upVector.set(0.f, 0.f, -1.f);
  check.xformVec(upVector, wandWorld);

  // find the absolute distance of the direction vectors
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);

  //write check values to shmSlice for plane 3 normal
  vals[15] = check[0];
  vals[16] = check[1];
  vals[17] = check[2];
  valD = dist6in[5];
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];
  dist6out[5] = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  if (shmDist6out) {
    shmDist6out->write(&dist6out);
  }
  if (shmSlice)
    shmSlice->write(vals);

  return DTK_CONTINUE ; 
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new clipWand(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
