// This loads defines a clipping plane according to the wand position
//
// DIVERSE graphics interface -- Performer ---- version 2.2.0
// released: Fri Oct 18 09:27:29 EDT 2002
// The DIVERSE HOMPAGE is at: www.diverse.vt.edu
// 
// Copyright (C) 2000, 2001, 2002  John Kelso, Lance Arsenault and Ron Kriz 
// 
// This software, DIVERSE graphics interface -- Performer (DGIPF),
// is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License (LGPL) as
// published by the Free Software Foundation; either version 2.1 of
// the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this software, in the top level source directory in
// a file named "COPYING"; if not, see it at:
// http://www.gnu.org/copyleft/lesser.html or write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
//#include <fstream.h>

static double* planeEq;
// shared memory for plane normal (from wand)
dtkSharedMem* shmSlice;
// shared memory for place distance from wand
dtkSharedMem* shmDist;
dtkSharedMem* shmDist2;
//  Shared memory segment for the on-off button
dtkSharedMem* shm2;
//  Shared memory segment for the spin rate
dtkInLocator *wand ;
// Shared memory for wandPointer
dtkSharedMem* shm4;
unsigned char selected;
float wandData[6];
float valD,distC;
float vals[3];
unsigned char onoff;

class clipWandPlane : public dtkAugment {
public:

  clipWandPlane(dpf *app) ;
  int preConfig(void);
  int postConfig(void);
  int preFrame(void);
  int postFrame(void);
  ~clipWandPlane();

private:
  dpf* app;
  dpfDisplay *display;
  float saveWand[3];
  float wandPos[6];
  float planePos[6];
  pfCoord wandCoord;
  pfCoord wandCoord2;
  bool wandStart;
  bool use_wireframe;
  char* nameOfWire;
  float wandEnd;
  float distcheck;
  float distcheck2;
  pfVec3 check;
  pfVec3 check2;
} ;

//  Enable the clipWandPlane plane in pre draw
int preDraw (pfTraverser *trav, void *data) {

  // first see if the on button is set to on
  if (shm2) {
    shm2->read(&onoff);
  }

  if (onoff == 1) {
    if (shmSlice)
      shmSlice->read(vals);
    if (shmDist)
      shmDist->read(&valD);
    if (shmDist2)
      shmDist2->read(&distC);
    
    static void* firstChan = NULL;
    if (firstChan == NULL) 
      firstChan = trav->getChan();
    
    if (firstChan == trav->getChan()) {
      // printf ("calculating plane EQ\n");
      
      //  Set up the coefficients in the plane equation
      //  Ax + By + Cz = D
      
      //  A,B,C correspond to the normal vector coefficients
      int c;
      for (c=0; c<3; c++) {
	planeEq[c] = vals[c];
      }
      
      //  Plug in position to solve for D
      planeEq[3] = - distC;
    }

    //printf ("enabling clipWandPlane plane, channel %x\n",trav->getChan());
    //cout << "current coordinates of clipping plane: " << planeEq[0] << " " <<planeEq[1] << " " <<planeEq[2] << " " <<planeEq[3] << endl;
    glClipPlane(GL_CLIP_PLANE0, planeEq);
    glEnable(GL_CLIP_PLANE0);
  }

  return PFTRAV_CONT;
}

//  Disable the clip plane in post draw
int postDraw (pfTraverser *trav, void *data) {
  //printf ("disabling clip plane, channel %x\n", trav->getChan());
  glDisable(GL_CLIP_PLANE0);

  return PFTRAV_CONT;
}

// create object and set description.
// be sure to validate it!
clipWandPlane::clipWandPlane(dpf *app_in) :
  dtkAugment("clipWandPlane")
{
  app = app_in;
  setDescription("clipping plane") ;
  validate();
  saveWand[0] = 0.0;
  saveWand[1] = 0.0;
  saveWand[2] = 0.0;
  wandStart = false;
  wandPos[0] = wandPos[1] = wandPos[2] = wandPos[3] = 
    wandPos[4] = wandPos[5] = 0.0f ;
  planePos[0] = planePos[1] = planePos[2] = planePos[3] = 
    planePos[4] = planePos[5] = 0.0f ;
  wandEnd = 0.5;
  use_wireframe = false;
} ;


clipWandPlane::~clipWandPlane()
{
   delete shmSlice;
   delete shmDist;
   delete shmDist2;
   delete shm2;
   delete shm4;
}

// called only once by dpf, after pfInit() and dpfDisplay:preConfig()
// but before pfConfig()
int clipWandPlane::preConfig(void) {
  //  Allocate plane equation out of Performer shared memory
  planeEq = (double*) pfMalloc (4 * sizeof(double), pfGetSharedArena());

  shmSlice = new dtkSharedMem(3 * sizeof(float), "slice");
  shmDist = new dtkSharedMem( sizeof(float), "dval");
  shmDist2 = new dtkSharedMem( sizeof(float), "dval2");

  //  Check if there is a segment "doclip" the on-off button
  shm2 = new dtkSharedMem(sizeof(unsigned char), "doclip");
  if(!shm2 || shm2->isInvalid()) {
      shm2 = NULL;
  }

  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand || wand->isInvalid())
    wand = NULL ;

  shm4 = new dtkSharedMem(sizeof(unsigned char), "pointerOn");
  if(!shm4 || shm4->isInvalid()) {
    shm4 = NULL;
  }

  // turn button off to start
  selected = 0;
  if (shm2)
     shm2->write(&selected);
  
  // start out by hiding the wandPointer
  if (shm4) {
    selected = 1;
    shm4->write(&selected);
  }

  return DTK_CONTINUE ;
}


int clipWandPlane::preFrame(void)
{
  display = app->display();
  if(!display) return DTK_CALLBACK_ERROR;

  return DTK_REMOVE_CALLBACK;
}

int clipWandPlane::postConfig(void) {

  //  Set the preDraw and postDraw functions set up the clipping planes
  app->display()->world()->setTravFuncs (PFTRAV_DRAW, preDraw, postDraw);

  // see if the user wants to use a wireframe object
  char *env = getenv("wireObject");
  if (env) {
    nameOfWire = strdup(env) ;
  }
  else
    nameOfWire = "line10.savg";

  char *env2 = getenv("useWire");

  if (env2) {
    char* useDup = strdup(env2);
    if (strcasecmp(useDup,"true")==0)
      use_wireframe = true;
  }

  // set up wireframe if it is needed
  if (use_wireframe) {
    pfGroup *dcs =
    new dpfDCS(wandPos, /* position */
	       1.0f, /* overall scale */
	       /* file       scale xyz     position xyz  rotation hpr */
	       nameOfWire, DPF_IDENTITY,
	       NULL);
    display->scene()->addChild(dcs);
  }

  return DTK_REMOVE_CALLBACK ;
}

int clipWandPlane::postFrame(void) {
  
  // read the wand direction
  if (wand)
    wand->read(wandData);

  // define the wand data in world coordinates
  wandCoord.xyz.set(wandData[0], wandData[1], wandData[2]);
  wandCoord.hpr.set(wandData[3], wandData[4], wandData[5]);
  display->dpfToWorld(&wandCoord);

  wandCoord2.xyz.set(wandData[0], wandData[1], wandData[2]);
  wandCoord2.hpr.set(wandData[3], wandData[4], wandData[5]);
  display->dpfToEther(&wandCoord2);
  
  if (!wandStart) {
    saveWand[0] = wandCoord2.xyz[0];
    saveWand[1] = wandCoord2.xyz[1];
    saveWand[2] = wandCoord2.xyz[2];
    wandStart = true;
  }
  else {
    saveWand[0] = wandCoord2.xyz[0];
    saveWand[1] = wandCoord2.xyz[1];
    saveWand[2] = wandCoord2.xyz[2];
  }

  // create a matrix from the HPR position in world coordinates
  pfMatrix wandWorld;
  wandWorld.makeEuler(wandCoord.hpr[0],wandCoord.hpr[1],wandCoord.hpr[2]);
  
  // create a vector in the +y direction and move according to the wand
  pfVec3 upVector(0.f, 1.f, 0.f);
  check.xformVec(upVector, wandWorld);
  
  // create a matrix from the HPR position in world coordinates
  pfMatrix wandWorld2;
  wandWorld2.makeEuler(wandCoord2.hpr[0],wandCoord2.hpr[1],wandCoord2.hpr[2]);
  
  // create a vector in the +y direction and move according to the wand
  pfVec3 upVector2(0.f, 1.f, 0.f);
  check2.xformVec(upVector2, wandWorld2);

  // reset the handPointer position based on the direction of the wand
  distcheck = sqrt(check[0]*check[0] + 
		   check[1]*check[1] + 
		   check[2]*check[2]);
  distcheck2 = sqrt(check2[0]*check2[0] + 
		   check2[1]*check2[1] + 
		   check2[2]*check2[2]);

  // set the wand position for the wireframe
  wandPos[0] = wandData[0] + wandEnd*check2[0]/distcheck;;
  wandPos[1] = wandData[1] + wandEnd*check2[1]/distcheck;
  wandPos[2] = wandData[2] + wandEnd*check2[2]/distcheck;;
  wandPos[3] = wandData[3];
  wandPos[4] = wandData[4];
  wandPos[5] = wandData[5];

  
  //write check values to shmSlice to use as the plane normal
  vals[0] = check[0];
  vals[1] = check[1];
  vals[2] = check[2];
  if (shmSlice)
    shmSlice->write(vals);

  // find the point along check2 that is the correct distance from the wand
  if (shmDist)
      shmDist->read(&valD);
  /*planePos[0] = wandData[0] + valD*check[0]/distcheck;;
  planePos[1] = wandData[1] + valD*check[1]/distcheck;
  planePos[2] = wandData[2] + valD*check[2]/distcheck;;
  planePos[3] = wandData[3];
  planePos[4] = wandData[4];
  planePos[5] = wandData[5];*/
  planePos[0] = wandCoord.xyz[0] + valD*check[0]/distcheck;;
  planePos[1] = wandCoord.xyz[1] + valD*check[1]/distcheck;
  planePos[2] = wandCoord.xyz[2] + valD*check[2]/distcheck;;
  planePos[3] = wandCoord.hpr[0];
  planePos[4] = wandCoord.hpr[1];
  planePos[5] = wandCoord.hpr[2];

  // create a plane with direction = check2, at this point
  distC = check[0]*planePos[0] + check[1]*planePos[1] + check[2]*planePos[2];

  if (shmDist2) {
    shmDist2->write(&distC);
  }

  return DTK_CONTINUE ; 
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new clipWandPlane(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
