//#define SPEW
#define NIST
#define SHM_TEXT_NAME "shmHeadsUpText" 

#include <Performer/pf/pfCullProgram.h>

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class desktopWandHPR : public dtkAugment {
public:

  desktopWandHPR(dpf *app) ;
  ~desktopWandHPR(void) ;
  int preFrame(void);
  int postFrame(void);

private:
  dtkInLocator *head;      // dummy head
  dtkInLocator *wand;      // simulated wand tracker hpr

  dtkSharedMem *headShm, *wandShm ;

  dtkInValuator *pointer;  // X mouse pointer in
  dtkInXKeyboard *keyboard; // keyboard in

  dtkDequeuer *dequeuer;

  KeyCode xkc_shift_l, xkc_shift_r ;

  float head_xyzhpr[6];
  float wand_xyzhpr[6];

  dpf *app;
  dpfDisplay *display ;
  dpfScreen *masterScreen ;
  pfPipeWindow *masterPipeWindow ;
  pfChannel *chan ;
  float left, right, bottom, top ; //viewport
  float l2, b2, rld, tbd ; // constants based on viewport
  pfDCS *wandCursor ;
  pfNode *cursor ;
  bool cursorOn ;

  dtkSharedMem *shmText ;
  bool visible ;

#ifdef NIST
  dtkNavList *navList ;
  bool navOn ;
  dtkAugment *picker ;
  bool pickerOn ;
#endif

} ;

desktopWandHPR::desktopWandHPR(dpf *app_in) :
  dtkAugment("desktopWandHPR")
{
  head = NULL ;
  wand = NULL;

  pointer = NULL;
  keyboard = NULL;

  dequeuer = NULL;

  wandCursor = NULL ;
  cursor = NULL ;
  cursorOn = false ;

  shmText = NULL ;
  visible = false ;

  app = app_in;
  display = app->display();
  head_xyzhpr[0] =  head_xyzhpr[1] =  head_xyzhpr[2] =  
    head_xyzhpr[3] =  head_xyzhpr[4] =  head_xyzhpr[5] = 0.f ;

  wand_xyzhpr[0] =  wand_xyzhpr[1] =  wand_xyzhpr[2] =  
    wand_xyzhpr[3] =  wand_xyzhpr[4] =  wand_xyzhpr[5] = 0.f ;

  // try to load shmHeadsUpText if it is not already loaded
  if (!app->check("shmHeadsUpText") && (app->load("shmHeadsUpText"))) 
    return ;

  if(!(head = static_cast<dtkInLocator *>
       (app->check("head", DTKINLOCATOR_TYPE))))
    {
      head = new dtkInLocator(app->record(), "head");
      if(!head || head->isInvalid()) 
	return ;
      head->setDescription("a dtkInLocator for head tracker");
      if(app->add(head)) 
	return ;
    }

  if(!(wand = static_cast<dtkInLocator *>
       (app->check("wand", DTKINLOCATOR_TYPE))))
    {
      wand = new dtkInLocator(app->record(), "wand");
      if(!wand || wand->isInvalid()) 
	return ;
      wand->setDescription("a dtkInLocator for wand tracker");
      if(app->add(wand)) 
	return ;
    }

  headShm = new dtkSharedMem(sizeof(float)*6, "head");
  if (headShm->isInvalid()) 
    return ;

  // just initialize to all zeros
  head->write(head_xyzhpr);
  headShm->write(head_xyzhpr);

  wandShm = new dtkSharedMem(sizeof(float)*6, "wand");
  if (wandShm->isInvalid())
    return ;

#ifdef NIST
  navList = NULL ;
  navOn = false ;
  picker = NULL ;
  pickerOn = true ;
#endif

  setDescription("DSO to set wand HPR and wand buttons based on mouse position and buttons") ;
  validate();
} ;

desktopWandHPR::~desktopWandHPR(void) {
}

int desktopWandHPR::preFrame(void) {
  if(!(keyboard = static_cast<dtkInXKeyboard *>
       (app->getNext(DTKINXKEYBOARD_TYPE))))
    return dtkAugment::ERROR_;
  keyboard->queue();

  if((!(xkc_shift_l = keyboard->getKeyCode(XK_Shift_L))) ||
     (!(xkc_shift_r = keyboard->getKeyCode(XK_Shift_R)))) {
    dtkMsg.add(DTKMSG_ERROR,
	       "desktopWandHPR::preFrame()- no Keycap to "
	       "match the shift keys in file %s",
	       __FILE__);
    return dtkAugment::ERROR_;
  }

  if(!(pointer = (dtkInValuator *) 
       app->get("pointer", DTKINVALUATOR_TYPE)) ||
     pointer->number() < 2) {
    dtkMsg.add(DTKMSG_ERROR,"desktopWandHPR::preFrame()"
	       " dtkInValuator named \"pointer\" with"
	       " two or more values not found.\n");
    return dtkAugment::ERROR_;
  }

  if (!((masterScreen = display->masterScreen()) &&
	(chan = masterScreen->pfchan(0))))
    return dtkAugment::ERROR_ ; 

  if (!masterScreen->simpleFrustum()) {
    dtkMsg.add(DTKMSG_ERROR,"desktopWandHPR::preFrame()"
	       " master screen does not have a simple frustum/\n") ;
    return dtkAugment::ERROR_;
  }
  masterPipeWindow = masterScreen->pwin()->pfpwin() ;

  chan->getViewport(&left, &right, &bottom, &top) ;
  l2 = left*2.f ;
  b2 = bottom*2.f ;
  rld = right - left ;
  tbd = top - bottom ;

  wandCursor = new pfDCS ;
  display->addGhostNode(wandCursor) ;
  display->scene()->addChild(wandCursor) ;
  cursor=pfdLoadFile("cursor.savg") ;
  
  // try to connect to shared memory created by shmHeadsUpText
  shmText =  new dtkSharedMem(SHM_TEXT_NAME) ;
  if (!shmText)
    dtkMsg.add(DTKMSG_ERROR,
	       "desktopWandHPR::preFrame() failed:"
	       " couldn't open shared memory segment %s\n",SHM_TEXT_NAME) ; 
  
#ifdef NIST
  picker = app->check("sgPicker") ;
  navList = (dtkNavList *) app->check("dtkNavList", DTKNAVLIST_TYPE) ;
  if (!navList) {
    navList = new dtkNavList ;
    if (app->add(navList)) {
      delete navList ;
      navList = NULL ;
      return ERROR_ ;
    }
  }
#endif
  
  return dtkAugment::REMOVE_CALLBACK ; 
}

int desktopWandHPR::postFrame(void) {
  
  // only do if shift is pressed
  if ( (keyboard->read(xkc_shift_l)) || (keyboard->read(xkc_shift_r))) {
#ifdef SPEW
    printf("shift at frame %d\n",pfGetFrameCount()) ;
#endif    
#ifdef NIST
    
    if (!cursorOn) {
      wandCursor->addChild(cursor) ;
      //pfuLoadPWinCursor(masterPipeWindow,PFU_CURSOR_OFF) ;
      cursorOn = true ;
    }
    
    if (picker && !pickerOn) {
      app->addCallback(picker, DTK_POSTFRAME) ;
      pickerOn = true ;
    }
    if (navOn) {
      dtkNav *nav = navList->current() ;
      if (nav) {
	app->removeCallback(nav, DTK_POSTFRAME) ;
	navOn = false ;
      }
    }
#endif
    static float oldxy[2] = {0.0f, 0.0f};
    float xy[2] = {0.0f, 0.0f};
    pointer->read(xy);
    // only process wand hpr if mouse has moved
    if ((xy[0]!=oldxy[0]) || (xy[1]!=oldxy[1])) {
      float vxy[2] = {(((xy[0]+1.f)-l2)/rld)-1.f, (((xy[1]+1.f)-b2)/tbd)-1.f} ;
      if (vxy[0]<-1.f)
	vxy[0]=-1.f ;
      else if  (vxy[0]>1.f)
	vxy[0]=1.f ;
      
      if (vxy[1]<-1.f)
	vxy[1]=-1.f ;
      else if  (vxy[1]>1.f)
	vxy[1]=1.f ;
      
      float h, v ;
      chan->getFOV(&h, &v) ;
      
      // for x, assume a triangle a, b, c.  
      // a=adjacent side, distance from eye to cursor, always 1
      // b=opposite side, distance to cursor, based on fov
      // c=hypot

      float b = vxy[0]*tan((h/2.f)*(M_PI/180.f)) ;
      wand_xyzhpr[3] = -atan(b)*180.f/M_PI ;

      float c = sqrt(1.f+(b*b)) ;

      // for y, assume a triangle with sides c, d, e
      // c is the same c as above triangle, but is not adjacent side
      // d is oppostite side, distance to cursor, based on fov 
      // e is the hypot, but we don't need it
      float d = vxy[1]*tan((v/2.f)*(M_PI/180.f)) ;

      wand_xyzhpr[4] = atan(d/c)*180.f/M_PI ;

#ifdef SPEW
      printf("b = %f\n",b) ;
      printf("c = %f\n",c) ;
      printf("d = %f\n",b) ;
      printf("  pointer: x = %3.3f y = %3.3f\n",xy[0],xy[1]) ;
      printf("  fov: h = %3.3f v = %3.3f\n",h,v) ;
      printf("  viewport: l/r = (%3.3f %3.3f) b/t = (%3.3f %3.3f)\n",
	     left, right, bottom, top) ;
      printf("  vmouse: x = %3.3f y =  %3.3f\n",vxy[0], vxy[1]) ;
      printf("  wand: h = %3.3f p =  %3.3f\n",wand_xyzhpr[3], wand_xyzhpr[4]) ;
#endif
      if (wandCursor) {
	wandCursor->setRot(wand_xyzhpr[3], wand_xyzhpr[4],  wand_xyzhpr[5]) ;
      }
      wand->write(wand_xyzhpr);
      wandShm->write(wand_xyzhpr);
      
      if (shmText && !visible) {
	shmText->write("               VIRTUAL WAND ENABLED       NAVIGATION DISABLED") ;
	visible = true ;
      }
      
    }//if ((xy[0]!=oldxy[0]) || (xy[1]!=oldxy[1])) {
  } else {  // shift up up
    if (shmText && visible) {
      shmText->write("") ;
      visible = false ;
    }
#ifdef NIST
    if (picker && pickerOn) {
      app->removeCallback(picker, DTK_POSTFRAME) ;
      pickerOn = false ;
    }
    if (!navOn) {
      dtkNav *nav = navList->current() ;
      if (nav) {
	app->addCallback(nav, DTK_POSTFRAME) ;
	navOn = true ;
      }
    }
#endif
    if (cursorOn) {
      wandCursor->removeChild(cursor) ;
      //pfuLoadPWinCursor(masterPipeWindow,PFU_CURSOR_left_ptr) ;
      cursorOn = false ;
    }
  } //if ( (keyboard->read(xkc_shift_l)) || (keyboard->read(xkc_shift_r)))
  return dtkAugment::CONTINUE ; 
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new desktopWandHPR(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
