#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

int shmCheck(int key, int size, 
	     unsigned int seconds, unsigned int tries, int verbose) {
  static unsigned int tried = 0 ;
  int id ;
  struct shmid_ds buf ;
  extern int errno;

  if (verbose)
    fprintf(stderr, "shmCheck(%d, %d, %d, %d, %d)\n",
	   key, size, seconds, tries, verbose) ;
  while (tried<tries || tries==0) {
    
    if (verbose)
      fprintf(stderr, "\nchecking shared memory ID %d, size %d\n",
	     key, size) ;
    
    id = shmget((key_t) key, (size_t) size, 0);
    if (id>=0) {
      if (verbose)
	fprintf(stderr, "id returned from shmget = %d\n",id) ;
      if (shmctl(id, IPC_STAT, &buf))
	fprintf(stderr, "shmctl failed, will try again\n") ;
      else
	if (size == buf.shm_segsz) 
	  return 1 ;
        else if (verbose)
	  fprintf(stderr, "shmctl says size of shared memory is %d, but %d was requested\n",
		 size, buf.shm_segsz) ; 
    } else if (verbose) {
      fprintf(stderr, "shmget failed, will try again\n") ;
    }

    tried++ ;

    if (seconds>0)
      sleep(seconds) ;
    
  }
  return 0 ;
}

#ifdef shmCheckMain

main(int argc, char **argv) {
  fprintf(stderr, "\nshmcheck returns %d\n",shmCheck(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]))) ;
}

#endif
