/* This reads data from system V shared memory with the form from VRCO
 * inc. trackd. 
 */


/* local hack version to just read the dat and print it.  It doesn't
   put it in shared memory or do any conversions */

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/stat.h>
#include <math.h>
#include <dtk.h>


#define PID_FILE    "/tmp/dtk-rawTrackd_PID"

static int run = 1;

static void Exit_catcher(int sig)
{
  dtkMsg.add(DTKMSG_NOTICE,
	     "caught signal %d .. exiting.\n",
	     sig);
  run = 0;
}

int main(int argc, char **argv)
{
  dtkMsg.setPreMessage("%s%s%s", dtkMsg.color.vil,
                       "dtk-getTrackd", dtkMsg.color.end);

  {
    unlink(PID_FILE);
    umask(00);
    FILE *pid_file = fopen(PID_FILE, "w");
    if(!pid_file)
      {
	dtkMsg.add(DTKMSG_ERROR, 1,
		   "fopen(\""
		   PID_FILE
		   "\", \"w\") failed.\n");
	return 1;
      }
    else
      {
	fprintf(pid_file,"%d\n",getpid());
	fclose(pid_file);
      }
  }

  signal(SIGINT,Exit_catcher);
  signal(SIGTERM,Exit_catcher);

  dtkVRCOSharedMem sysV;
  if(sysV.isInvalid()) return 1;

  // run at 100 Hz => 1/100 second period
  dtkRealtime_init(0.01);


  dtkTime t ;

  t.reset(0.f, 1) ;

  for(;run;)
    {
      
      float ft = (float)t.get() ;
      printf("%f head: %f %f %f %f %f %f\n",ft,
	     sysV.head->x, sysV.head->y, sysV.head->z,
	     sysV.head->h, sysV.head->p, sysV.head->r) ;
      
      printf("%f wand: %f %f %f %f %f %f\n",ft,
	     sysV.wand->x, sysV.wand->y, sysV.wand->z,
	     sysV.wand->h, sysV.wand->p, sysV.wand->r) ;
      
      // sleep until next cycle.
      dtkRealtime_wait();
    }
  dtkRealtime_close();
  unlink(PID_FILE);
  
  return 0;
}
