/* zeros a shared memory segment, creating it if it doesn't exist

   John Kelso, 9/04, kelso@nist.gov

 */

#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <dtk.h>

static int Usage(char *name) {
  fprintf(stderr,
	  " Usage: %s SHM_FILE [ SIZE ]\n\n"
	  "  Zeros a DTK shared memory file named SHM_FILE\n"
	  "  whose size is SIZE bytes.\n"
	  "  SIZE can be give as \"-\", or omitted, if the shared\n"
	  "  memory segment already exists.\n",
	  name) ;
  return 1 ;
}

int main(int argc, char **argv) {
  
  dtkMsg.setPreMessage(argv[0]) ;
  
  if (argc > 3 || argc < 2) 
    return Usage(argv[0]) ;
  
  char *name = argv[1] ;

  int size ;
  bool useExistingSize = false ;
  if ((argc==2) || (!(strcmp(argv[2],"-")))) {
    useExistingSize = true ;
  } else if ((1 != sscanf(argv[2],"%d",&size) || size<1)) {
    dtkMsg.add(DTKMSG_ERROR, "Malformed SIZE\n") ;
    return 1 ;
  }

  dtkSharedMem *shm = NULL ;
  if (useExistingSize) {
    shm = new dtkSharedMem(name) ;
    if(shm->isInvalid()) 
      return 1 ;
    size = shm->getSize() ;
    //printf("using existing size of %d bytes\n",size) ;
  } else {
    shm = new dtkSharedMem(size, name) ;
    if(shm->isInvalid()) 
      return 1 ;
  }
  
  char *zero = (char *) dtk_malloc(size) ;
  bzero(zero, size) ;

  if(shm->write(zero)) 
    return 1 ;
  
  return 0 ;
}
