#include <stdio.h>
#include <stdlib.h>
#include <dtk.h>
//#include <fstream.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>


void usage() {
  printf("usage: \nhev-clipwand on (-x,+x,-y,+y,-z,+z) float (1-6 entries) or\nnhev-clipwand off (-x,+x,-y,+y,-z,+z)\n");
}

int findPlane(char* xyz) {
  int planeNum = 0;
  char* xplus = new char[3];
  char* xminus = new char[3];
  char* yplus = new char[3];
  char* yminus = new char[3];
  char* zplus = new char[3];
  char* zminus = new char[3];
  xplus = "+x";;
  xminus = "-x";
  yplus = "+y";
  yminus = "-y";
  zplus = "+z";
  zminus = "-z";
  if (strcasecmp(xyz,xplus) == 0) planeNum = 1;
  if (strcasecmp(xyz,xminus) == 0) planeNum = 2;
  if (strcasecmp(xyz,yplus) == 0) planeNum = 3;
  if (strcasecmp(xyz,yminus) == 0) planeNum = 4;
  if (strcasecmp(xyz,zplus) == 0) planeNum = 5;
  if (strcasecmp(xyz,zminus) == 0) planeNum = 6;
  return planeNum;
}

main(int argc, char **argv) {
  // set up shared memory for clipWand (6 floats)
  dtkSharedMem* shmBox;
  dtkSharedMem* shmPlanesOn;
  dtkSharedMem* shm2;

  shmBox = new dtkSharedMem( 6*sizeof(float), "dval6");
  shmPlanesOn = new dtkSharedMem (6*sizeof(unsigned char), "planeson");
  shm2 = new dtkSharedMem(sizeof(unsigned char), "doclip");

  if(!shmBox || shmBox->isInvalid())
      shmBox = NULL ;

  if(!shmPlanesOn || shmPlanesOn->isInvalid())
      shmPlanesOn = NULL ;

  if(!shm2 || shm2->isInvalid()) {
      shm2 = NULL;
  }

  char* onchar = new char[3];
  onchar = "on";
  char* offchar = new char[4];
  offchar = "off";
  char* allchar = new char[4];
  allchar = "all";
  unsigned char selected;

  float box[6];
  int planesWanted = 0;
  int planeNum;
  unsigned char pon[6];

  for ( int i = 0; i < 6; ++i ) {
    pon[i] = 0;
    box[i] = 0.0;
  }
  int numParameters = 1;

  // read in current pon settings from wandClip if available
  if (shmPlanesOn)
    shmPlanesOn->read(&pon);
 
  // read in current plane distances from wandClip
  if (shmBox)
      shmBox->read(&box);

  // first parse the input
  if (argc < 3) {
    usage();
  }
  else {
    // there must be an on or off parameter
    selected = 2;
    if (strcasecmp(onchar, argv[1]) == 0) {
      // turn on mode
      selected = 1;
      ++numParameters;
    }
    else if (strcasecmp(offchar, argv[1]) == 0) {
      // turn off mode
      selected = 0;
      ++numParameters;
    }

    if (selected == 2) {
      // error - need an on or off
      usage();
    }
    else if (selected == 1) {
      // deal with all the plane definitions
      for ( int j = numParameters; j < argc; ++j ) {
	planeNum = findPlane(argv[j]);
	if (planeNum == 0 || argc < (j+2)) {
	  usage();
	  break;
	}
	box[planeNum-1] = atof(argv[j+1]);
	pon[planeNum-1] = 1;
	++planesWanted;
	++j;
      }
    }
    else {
      // either turn them all off or
      if (strcasecmp(allchar,argv[2])==0) {
	for ( int j = 0; j < 6; ++j ) {
	  pon[j] = 0;
	}
      }
      else {
	// collect planes to turn off
	for ( int j = numParameters; j < argc; ++j ) {
	  planeNum = findPlane(argv[j]);
	  pon[planeNum-1] = 0;
	  ++planesWanted;
	}
      }
    }
  }

  // send to clipWand DSO
  if (shmBox && shmBox->isValid()) {
    shmBox->write(&box);
  }
  if (shmPlanesOn && shmPlanesOn->isValid()) {
    shmPlanesOn->write(&pon);
  }

  // if any pon values are 1, set the on button
  selected = 0;
  for ( int j = 0; j < 6; ++j ) {
    if (pon[j] == 1) {
      selected = 1;
      break;
    }
  }
  if (shm2)
    shm2->write(&selected);

  delete shmBox;
  delete shmPlanesOn;
  return 0;
}
