/*

 This adds a light node that follows the wand- parameters for the light can
 be set using dtk shared memory- two floats containing intensity and spread

try a command like this to set it:
  dtk-floatSliders hev-flashlight -N 2 -s 0 0 1 .8 -s 1 0 90 10 -l intensity spread

*/

#include <stdlib.h>
#include <math.h>
#include <Performer/pf/pfLightSource.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class wandLight : public dtkAugment
{
public:

  wandLight(dpf *app) ;
  ~wandLight(void) ;
  int postConfig(void) ;   
  int postFrame(void) ;   

private:

  dpf *app ;
  dpfDisplay *display ;
  pfLightSource *l ;
  dtkInLocator *wand;
  float wandPos[6] ;

  float cone[2] ;
  dtkSharedMem* coneShm ;
} ; 


wandLight::wandLight(dpf *app_in) : dtkAugment("wandLight")
{
  app = app_in ;
  l = NULL ;

  coneShm = NULL ;
  coneShm = new dtkSharedMem(sizeof(cone), "hev-flashlight") ;
  if(!coneShm || coneShm->isInvalid()) {
    return ;
  }

  setDescription("add a light node to at the "
		 "positon and direction of the wand") ;
  // dtkAugment starts out invalid.
  validate() ; 
}


wandLight::~wandLight(void)
{
  if (l) {
    delete l ;
    l = NULL ;
  }
}

int wandLight::postConfig(void)
{
  display = app->display() ;
  if (!display->scene()) 
    return dtkAugment::ERROR_ ;
  
  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand) return dtkAugment::ERROR_ ;

  l = new pfLightSource ;
  if(!display->scene()->addChild(l)) {
    return dtkAugment::ERROR_ ;
    
  }
  return dtkAugment::REMOVE_CALLBACK ;
}


// OK to have a little lag for better overall performance
int wandLight::postFrame(void)
{

  coneShm->read(cone) ;
  l->setColor(PFLT_AMBIENT, cone[0], cone[0], cone[0]) ;
  l->setColor(PFLT_DIFFUSE, cone[0], cone[0], cone[0]) ;
  l->setColor(PFLT_SPECULAR, cone[0], cone[0], cone[0]) ;
  l->setSpotCone(0.f, cone[1]) ;

  wand->read(wandPos);

  pfMatrix mat ;
  mat.makeEuler(wandPos[3],
		wandPos[4],
		wandPos[5]) ;

  pfVec3 dir = pfVec3(0.0f, 1.0f, 0.0f) ;
  dir = dir*mat ;

  l->setPos(wandPos[0],
            wandPos[1],
            wandPos[2],
	    1.0f ) ;
  l->setSpotDir(dir[0], dir[1], dir[2]) ;

  return dtkAugment::CONTINUE ;
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new wandLight(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


