// fltk program to display a jpeg image

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <dtk.h>

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Shared_Image.H>

////////////////////////////////////////////////////////////////////////
// evil global variables
static bool noborder = false ;
static bool fullscreen = false ;

////////////////////////////////////////////////////////////////////////
static void usage(char *name) {
  fprintf(stderr,"Usage: %s [FLTK options] [--noborder] [--fullscreen] file\n", name) ;
}

////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv) {
  
  fl_register_images() ;
  Fl::visual(FL_RGB);

  int fltk_argc;
  char **fltk_argv;
  int other_argc;
  char **other_argv;
  // dtkFLTKOptions_get will pull out FLTK options into its return pointer.
  if(dtkFLTKOptions_get(argc, (const char **) argv,
                        &fltk_argc,  &fltk_argv,
                        &other_argc, &other_argv)) 
    return 1;
  
  char *prog = other_argv[0] ;
  // parse other args

  printf("other_argc=%d\n",other_argc) ; 
  if (other_argc<2) {
    usage(prog) ;
    return 1 ;
  }
  int o = 1 ; // skip argv[0]
  while (o<(other_argc-1)) {
    size_t l = strlen(argv[o]) ;
    if (l<3) l=3 ;

    if (!strncasecmp(other_argv[o],"--noborder",l)) {
      noborder = true ;
    } else if (!strncasecmp(other_argv[o],"--fullscreen",l)) {
      fullscreen = true ;
    } else {
      fprintf(stderr,"%s: invalid parameter \"%s\"\n", prog, other_argv[o]) ;
      usage(prog) ;
      return 1 ;
    }
    o++ ;
  }
  
  Fl_Shared_Image *img;
  img = Fl_Shared_Image::get(other_argv[other_argc-1]) ;
  if (!img) {
    fprintf(stderr, "%s: file or image file format not recognized!\n", prog);
    return 1 ;
  }

  Fl_Window *window = new Fl_Window(0, 0, img->w(), img->h(), NULL); 
  window->color(FL_BLACK) ;
  if (noborder) {
    window->border(0) ;
  }
  if (fullscreen) {
    window->fullscreen() ;
  }
  {
    Fl_Box *box = new Fl_Box(0, 0, img->w(), img->h()) ;
    box->image(img) ;
    box->redraw() ;
  }
  window->end();
  window->show(fltk_argc, fltk_argv);

  while (Fl::wait()) {
  }
  return 0 ;
}
