#include <stdio.h>
#include <stdlib.h>
#include <dtk.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string>

using namespace std ;

void usage();
bool isCommandCorrect(char* command);

main(int argc, char **argv) {
 
  bool goodCommand = false;

  // first check for a good command
  if (argc < 2)
    usage();

  char *command ;
  command = argv[1] ;
  goodCommand = isCommandCorrect(command);
  if (!goodCommand)
    usage();

  unsigned char mmode;  
  mmode = 0;
  dtkSharedMem* shm_measure;

  // define shared memory for measurement1D mode
  shm_measure = new dtkSharedMem(sizeof(unsigned char), "measureMode");
  if(!shm_measure || shm_measure->isInvalid())
    shm_measure = NULL ;
  else
    shm_measure->flush();

  //  Shared memory segment used to enter a file name
  dtkSharedMem* shm_filename;
  char filename[1024];

  // define shared memory for a filename to be defined
  shm_filename = new dtkSharedMem(sizeof(char)*1024, "filesname");
  if(!shm_filename || shm_filename->isInvalid())
    shm_filename = NULL ;
  else
    shm_filename->flush();

  //  Shared memory segment used to pass min/max values
  dtkSharedMem* shm_minmax;
  float minmax[2];

  // define shared memory for a filename to be defined
  shm_minmax = new dtkSharedMem(sizeof(float)*2, "minmax");
  if(!shm_minmax || shm_minmax->isInvalid())
    shm_minmax = NULL ;
  else
    shm_minmax->flush();

  // issue a dtk-writeShm command for each mode
  if (strcasecmp(command, "create") == 0) {  
    mmode = 1;
  }
  else if (strcasecmp(command, "edit") == 0) {  
    mmode = 2;
  }
  else if (strcasecmp(command, "delete") == 0) {  
    mmode = 3;
    //printf("argc = %d, argv[2] = %s\n",argc, argv[2]) ;
    if (argc==3 && !strcasecmp("all", argv[2]))
      mmode = 10 ;
  }
  else if (strcasecmp(command, "inactive") == 0) {
    mmode = 4;
  }
  else if (strcasecmp(command, "undo") == 0) {  
    mmode = 5;
  }
  else if (strcasecmp(command, "save") == 0) {  
    mmode = 6;
    // take in a file name and write it to shared memory
    if (argc < 3) {
      usage();
      exit(1);
    }
    strcpy(filename,argv[2]);
    printf("got the filename in hev-measure: %s\n",filename);
    if (shm_filename) {
      shm_filename->write(filename);
      printf("wrote filename: %s\n",filename);
    }
  }
  else if (strcasecmp(command, "restore") == 0) {  
    mmode = 7;
    // take in a file name and write it to shared memory
    if (argc < 3) {
      usage();
      exit(1);
    }
    strcpy(filename,argv[2]);
    if (shm_filename)
      shm_filename->write(filename);
  }
  else if (strcasecmp(command, "highlight") == 0) {
    // determine if this is on or off mode
    
    // pass the min max values
    if (argc < 3) {
      usage();
      exit(1);
    }
    if (argc == 3) {
      // off mode
      if (strcasecmp(argv[2],"off")==0) {
	mmode = 9;
      }
      else {
	usage();
	exit(1);
      }
    }
    else {
      // on mode
      mmode = 8;
      minmax[0] = atof(argv[2]);
      minmax[1] = atof(argv[3]);
      //printf("min: %f  and max: %f\n",minmax[0],minmax[1]);
      
      // otherwise pass the values
      if (shm_minmax) {
	shm_minmax->write(minmax);
      }
    }
  }

  //printf("writing mmode: %d\n",mmode);
  if (shm_measure)
    shm_measure->write(&mmode);

  //printf("exit\n");
}

void usage() {
  printf("usage: hev-measurement1D function\n examples: \n hev-measurement1D create\n hev-measurement1D edit\n hev-measurement1D delete [all]\n hev-measurement1D inactive\n hev-measurement1D undo\n hev-measurement1D save\n hev-measurement1D restore\n hev-measurement1D highlight min max\n hev-measurement1D highlight off\n");
  exit(1);
}

bool isCommandCorrect(char* command) {
   bool isit = false;
   if (strcasecmp(command, "create") == 0) isit = true;
   else if (strcasecmp(command, "edit") == 0) isit = true;
   else if (strcasecmp(command, "delete") == 0) isit = true;
   else if (strcasecmp(command, "inactive") == 0) isit = true;
   else if (strcasecmp(command, "undo") == 0) isit = true;
   else if (strcasecmp(command, "reset") == 0) isit = true;
   else if (strcasecmp(command, "save") == 0) isit = true;
   else if (strcasecmp(command, "restore") == 0) isit = true;
   else if (strcasecmp(command, "highlight") == 0) isit = true;
   return isit;
}
