// hev-sgEditor command:
//
// hev-sgEditor load nodename filename
// hev-sgEditor unload nodename
// hev-sgEditor on nodename branchname
// hev-sgEditor off nodename
// hev-sgEditor group groupname
//   (defines a group node)
// hev-sgEditor dcs nodename [xyz] [hpr] [sxsysz]
//   (defines a pfDCS node for a dynamic coordinate system)
// hev-sgEditor scs nodename [xyz] [hpr] [sxsysz]
//   (defines a pfSCS node, for a static coordinate system)
// hev-sgEditor makedcs nodename
//   (defines a dynamic coordinate system from selected points)
// hev-sgEditor clone oldnode newnode
//   (copies an existing object)

#include <stdio.h>
#include <stdlib.h>
#include <dtk.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ptrace.h>
#include <string>

using namespace std ;

void usage();
bool isCommandCorrect(char* command);

main(int argc, char **argv) {
  //  Shared memory segment to put the call
  dtkSharedMem* shm_call;

  // shared memory to hold an input line
  shm_call = new dtkSharedMem(sizeof(char)*1000, "fileChars");
  if(!shm_call || shm_call->isInvalid())
      shm_call = NULL ;

  char* nextchar = new char[4];
  nextchar = "next";
  char* line = new char[1000];
  const char* line2 = new char[1000];
  bool goodCommand = false;
  
  // first parse the input
  if (argc == 2 ) {
    if (strcasecmp(nextchar, argv[1]) == 0) {
      // check for "next" and do nothing
    }
    else
      usage();
  }
  else if (argc > 2) {
    // first check the command
    goodCommand = isCommandCorrect(argv[1]);

    if (!goodCommand)
      usage();
    else {
      // load command into shared memory array
      string wholeLine = argv[1];
      string somemore;
      for ( int i = 2; i < argc; ++i ) {
	somemore = argv[i];
	wholeLine += "  " + somemore;
      } 
      printf("print the line from hev-sgEditor: %s\n",wholeLine.c_str());
      line2 = wholeLine.c_str();
      strcpy(line, line2);
      printf("check the line: %s\n",line);

      // send line to DSO for processing
      if (shm_call && shm_call->isValid()) {
	// write the line to shared memory
	printf("writing to shmcall\n");
	shm_call->write(line);
      }
    }
  }
  else
    usage();

  delete shm_call;
  return 0;
}

void usage() {
  printf("usage: hev-sgEditor function objectname [parameter]\n examples: \n hev-sgEditor load bluesqare blueSquare.savg\n hev-sgEditor unload bluesquare\n hev-sgEditor on bluesquare\n hev-sgEditor off bluesquare\n hev-sgEditor clone bluesquare blsquare2\n hev-sgEditor dcs dcsname xyz hpr sxsysz\n hev-sgEditor scs scsname xyz hpr sxsysz\n hev-sgEditor group groupname\n");
  exit(1);
}

bool isCommandCorrect(char* command) {
   bool isit = false;
   if (strcasecmp(command, "load") == 0) isit = true;
   if (strcasecmp(command, "unload") == 0) isit = true;
   if (strcasecmp(command, "makedcs") == 0) isit = true;
   if (strcasecmp(command, "on") == 0) isit = true;
   if (strcasecmp(command, "off") == 0) isit = true;
   if (strcasecmp(command, "clone") == 0) isit = true;
   if (strcasecmp(command, "group") == 0) isit = true;
   if (strcasecmp(command, "dcs") == 0) isit = true;
   if (strcasecmp(command, "scs") == 0) isit = true;
   return isit;
}
