// hev-sgEditor command:
//
// hev-sgEditor load nodename filename
// hev-sgEditor unload nodename
// hev-sgEditor on nodename branchname
// hev-sgEditor off nodename
// hev-sgEditor group groupname
//   (defines a group node)
// hev-sgEditor dcs nodename [xyz] [hpr] [sxsysz]
//   (defines a pfDCS node for a dynamic coordinate system)
// hev-sgEditor scs nodename [xyz] [hpr] [sxsysz]
//   (defines a pfSCS node, for a static coordinate system)
// hev-sgEditor makedcs nodename
//   (defines a dynamic coordinate system from selected points)
// hev-sgEditor clone oldnode newnode
//   (copies an existing object)
// hev-sgEditor seq sequenceName [time]
//   (creates a sequence node)
// hev-sgEditor updateSeqTime
//   (updates a sequence node)


#include <stdio.h>
#include <stdlib.h>
#include <dtk.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string>
#include "parse.C"

using namespace std ;

void usage();
bool isCommandCorrect(char* command);
int getLine(char * pline);
int atEOF();

main(int argc, char **argv) {

  char pline[1024];
  int pargc = 0;
  char **pargv  = NULL;

  //  Shared memory segment to put the call
  dtkSharedMem* shm_call;

  // shared memory to hold an input line
  shm_call = new dtkSharedMem(sizeof(char)*1000, "fileChars");
  if(!shm_call || shm_call->isInvalid())
      shm_call = NULL ;
  shm_call->flush();

  char* nextchar = new char[4];
  nextchar = "next";
  char* line = new char[1000];
  const char* line2 = new char[1000];
  bool goodCommand = false;
 
  // first check for a good command
  if (argc < 2)
    usage();
  goodCommand = isCommandCorrect(argv[1]);
  if (!goodCommand)
    usage();

  // if input from stdin: cycle through; otherwise send 1 message
  int cycle = 0;
  if (strcmp(argv[1],"-")==0)
    cycle = 1;

  int done = 0;
  string wholeLine;
  string somemore;

  while (done == 0) {

    if (cycle == 0) {    
      // load command into shared memory array
      wholeLine = argv[1];

      for ( int i = 2; i < argc; ++i ) {
	somemore = argv[i];
	wholeLine += "  " + somemore;
      } 
      
      //printf("print the line from hev-sgEditor: %s\n",wholeLine.c_str());
      line2 = wholeLine.c_str();
      strcpy(line, line2);
      //printf("check the line: %s\n",line);
      
      // send line to DSO for processing
      if (shm_call && shm_call->isValid()) {
	// write the line to shared memory
	//printf("writing to shmcall in hev-sgEditor: %s\n",line);
	shm_call->write(line);
	//printf("back from writing to shmcall in hev-sgEditor\n");
	shm_call->flush();
      }
      done = 1;
    }

    else {
      // read from stdin
      if( !getLine(pline) )
	done = 1;
      //printf("next line to process: %s\n",pline);
      
      // parse the new line
      dpf_charsToArgcArgv(pline, &pargc, &pargv);
      //printf("pargc: %d\n",pargc);
      
      // load command into shared memory array
      wholeLine = pargv[1];

      for ( int i = 2; i < pargc; ++i ) {
	somemore = pargv[i];
	wholeLine += "  " + somemore;
      } 
      
      //printf("print the line from hev-sgEditor: %s\n",wholeLine.c_str());
      line2 = wholeLine.c_str();
      strcpy(line, line2);
      //printf("check the line: %s\n",line);
      
      // send line to DSO for processing
      //if (shm_call && shm_call->isValid()) {
      // write the line to shared memory
      //printf("writing to shmcall in hev-sgEditor: %s\n",line);
      shm_call->write(line);
      //printf("back from writing to shmcall in hev-sgEditor\n");
      //}
    }
  }
}

void usage() {
  printf("usage: hev-sgEditor function objectname [parameter]\n examples: \n hev-sgEditor load bluesqare blueSquare.savg\n hev-sgEditor unload bluesquare\n hev-sgEditor reload bluesquare blueSquare.savg\n hev-sgEditor on bluesquare\n hev-sgEditor off bluesquare\n hev-sgEditor clone bluesquare blsquare2\n hev-sgEditor dcs dcsname xyz hpr sxsysz\n hev-sgEditor scs scsname xyz hpr sxsysz\n hev-sgEditor updateDCS dcsname xyz hpr sxsysz\n hev-sgEditor group groupname\n hev-sgEditor rubberband dcsname\n hev-sgEditor updateSeqTime time seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor pause seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor resume timestep seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor next seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor previous seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor jump frame seqnode1 [seqnode2] [seqnode3]...\n hev-sgEditor - (read from stdin)\n");
  exit(1);
}

bool isCommandCorrect(char* command) {
   bool isit = false;
   if (strcasecmp(command, "load") == 0) isit = true;
   else if (strcasecmp(command, "unload") == 0) isit = true;
   else if (strcasecmp(command, "reload") == 0) isit = true;
   else if (strcasecmp(command, "makedcs") == 0) isit = true;
   else if (strcasecmp(command, "on") == 0) isit = true;
   else if (strcasecmp(command, "off") == 0) isit = true;
   else if (strcasecmp(command, "clone") == 0) isit = true;
   else if (strcasecmp(command, "group") == 0) isit = true;
   else if (strcasecmp(command, "dcs") == 0) isit = true;
   else if (strcasecmp(command, "scs") == 0) isit = true;
   else if (strcasecmp(command, "rubberband") == 0) isit = true;
   else if (strcasecmp(command, "updateDCS") == 0) isit = true;
   else if (strcasecmp(command, "updateSeqTime") == 0) isit = true;
   else if (strcasecmp(command, "pause") == 0) isit = true;
   else if (strcasecmp(command, "resume") == 0) isit = true;
   else if (strcasecmp(command, "next") == 0) isit = true;
   else if (strcasecmp(command, "previous") == 0) isit = true;
   else if (strcasecmp(command, "jump") == 0) isit = true;
   else if (strcasecmp(command, "-") == 0) isit = true;
   return isit;
}

int getLine(char * pline)
/*  
 * Reads a line from stdin and prints it to pline. 
 * First does a test to see that not at end of file.  
 * If at end of file returns 0 indicating
 * false.  If not EOF The line is placed into "string" and 1
 * is returned indicating true.  This function eats the line.
 */
{
  if( !atEOF() )
  {
    gets(pline);
    return 1; // successful, not at EOF
  }
  else
  {
    return 0; //not successful
  }
}

int atEOF()
/*
 * Returns 1 if at end of file (EOF) and 0 otherwise
 */ 
{
  int c;
  if( (c = getchar()) != EOF)
  {
    /*
     * The if statement takes the first character: put it back 
     * before reading in the line 
     */ 
    ungetc(c, stdin);   
    return 0; // not at EOF
  }
  else
  {
    ungetc(c, stdin);
    return 1; // at EOF
  }
} 
