#include <stdio.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <Performer/pf/pfSequence.h>
#include <Performer/pr/pfString.h>
#include <dtk.h>

class hydrationtimeline : public dtkAugment {
public:
  hydrationtimeline(dpf *app) ;
  ~hydrationtimeline(void) ;
  int preFrame(void);
  int postConfig(void);
  int postFrame(void);
  
private:
  dpf* app;
  dpfDisplay *display;
  dtkSharedMem* shm_mode;
  dtkSharedMem* shm_seq;
  dtkSharedMem* shm_highlight;

  // counter for the toggleNodes
  int cnt;
  pfDCS** boxDCSList;
  pfNode** plotnodes;
  string plotnames[10];
  string plotnodenames[10];
  int numBars;
  pfNode* backgroundnode;
  pfDCS* backgroundDCS;
  pfNode* highlightnode;
  pfDCS* highlightDCS;
  pfCoord syscoord;
  pfMatrix coordMatrix;
  int i;
  int mmode;
  char seqnode[100];
  int highlight;
  pfSequence *sq;
  pfSequence* getSeqNode(char* nodename);
  pfSequence *seq_node;
  pfGroup *seqParent;
  int numChildren;
  pfNode* testnode;
  int nstep;
  int frame;
  int repeat;
  float scale[10];
  char compfile[250];
  FILE *cFile;
  float ptcomp[2][125000];
  int pargc ;
  char **pargv ;
  float totalA,totalB,totalC;
  char line[2000];
  char numb[50];
  int lastframe;
} ;

// create object and set description.
// be sure to validate it!
hydrationtimeline::hydrationtimeline(dpf *app_in) :
  dtkAugment("hydrationtimeline")
{
  printf("start of constructor\n");
  setDescription(" loads files specified by DPF_HYDRATIONTIMELINE_FILES"
		 " environment variable; all objects whose centers"
		 " lie in the user-selected area are visible;"
                 " otherwise they are not visible.") ;
  app = app_in;
  validate();
  cnt = 0 ;
  numBars = 2;
  printf("pointd \n");
  boxDCSList = new pfDCS*[100];
  plotnodes = new pfNode*[100];
  printf("pointc \n");
  plotnames[0] = "plotCaO.savg";
  plotnames[1] = "plotCaOH.savg";
  plotnames[2] = "plotH2O.savg";
  plotnames[3] = "yellowbox.savg";
  plotnames[4] = "purplebox.savg";
  plotnames[5] = "pinkbox.savg";
  plotnames[6] = "aquabox.savg";
  plotnames[7] = "ltbluebox.savg";
  plotnames[8] = "ltgreenbox.savg";
  plotnodenames[0] = "plotCaO";
  plotnodenames[1] = "plotCaOH";
  plotnodenames[2] = "plotH2O";
  plotnodenames[3] = "yellowbox";
  plotnodenames[4] = "purplebox";
  plotnodenames[5] = "pinkbox";
  plotnodenames[6] = "aquabox";
  plotnodenames[7] = "ltbluebox";
  plotnodenames[8] = "ltgreenbox";
  nstep = 1;
  cFile = NULL;

  // Check if there is a shared memory segment "hydrationtimeline" 
  shm_mode = new dtkSharedMem(sizeof(int), "hydration-plot");
  if(!shm_mode || shm_mode->isInvalid()) 
    shm_mode = NULL ;

  mmode = 0;
  if (shm_mode)
    shm_mode->write(&mmode);

  shm_seq = new dtkSharedMem(sizeof(char)*100, "seqName");
  if(!shm_seq || shm_seq->isInvalid())
    shm_seq = NULL ;
} ;

// destructor
hydrationtimeline::~hydrationtimeline(void) {
  if (shm_mode)
    dtkSharedMem_destroy("hydrationtimeline");
  if (cFile != NULL)
    fclose(cFile);
}

#define MIN_BUTTONS 3

int hydrationtimeline::preFrame(void)
{
  printf("start of preFrame\n");
  display = app->display();
  if(!display) return DTK_CALLBACK_ERROR;

  // load in the colored boxes - get number or use default of 3
  char *env = getenv("DPF_HYDRATIONTIMELINE_LEVELS") ;
  if (env)
    numBars = atoi(env);
  printf("number of bars in hydrationtimeline: %d\n",numBars);

  // load in the necessary number of colored bars
  for (i = 0; i < numBars; ++i) {
    plotnodes[i] = pfdLoadFile(plotnames[i].c_str());
  }
    
  // define the background for the boxes and hang it on the scene node
  backgroundnode = pfdLoadFile("back1.savg");
  backgroundDCS = new pfDCS();
  backgroundDCS->setTrans(.2f, 1.0f, -.6f);
  backgroundDCS->setScale(.35f, .35f, .35f);
  //display->scene()->addChild(backgroundDCS);
  backgroundDCS->setName("hydrationtimeline");
  backgroundDCS->addChild(backgroundnode);

  // orient the box DCSs with respect to the background
  for (i = 0; i < numBars; ++i) {
    boxDCSList[i] = new pfDCS();
    boxDCSList[i]->setTrans(0.13, -0.1, 0.35);
    boxDCSList[i]->setScale(1.7f, 1.0f, 13.0f);
    //backgroundDCS->addChild(boxDCSList[i]);
    boxDCSList[i]->addChild(plotnodes[i]);
    boxDCSList[i]->setName(plotnodenames[i].c_str());
  }

  // bring in and orient the highlighting box
  highlightnode = pfdLoadFile("lightbluebox.savg");
  highlightDCS = new pfDCS();
  highlightDCS->setTrans(0.13, -0.1, 0.33);
  highlightDCS->setScale(0.01f, 1.0f, 0.68f);
  highlightDCS->addChild(highlightnode);
  highlightDCS->setName("highlightnode");


  // just call buttonNavControl::preFrame() once at the start.
  return DTK_REMOVE_CALLBACK;
}


int hydrationtimeline::postConfig(void) {
  printf("start of postConfig\n");
  return DTK_REMOVE_CALLBACK;
}

int hydrationtimeline::postFrame(void) {
  //printf("start of postFrame\n");
  
  // check mmode
  if (shm_mode)
    shm_mode->read(&mmode);
  
  if (mmode == 1) {
    // time to update
    
    // find the node
    if (shm_seq)
      shm_seq->read(seqnode);
    
    if (seqnode != NULL) {
      printf("print the name: %s\n",seqnode);
      
      // see if this is a seq node
      sq = getSeqNode(seqnode);
      if (sq == NULL) {
	printf("null seq\n");
	nstep = 1;
      }
      else {
	// find the timestep
	printf("good seq\n");
	// get the last frame number (196, 20, 10) to get overall frame number
	lastframe = sq->getNumChildren() - 1;
	printf("last frame: %d\n",lastframe);
	frame = sq->getFrame(&repeat);
	//printf("got the first frame number: %d\n",frame);
	
	// if this sequence file is not every frame, multiply frame number
	if (lastframe < 11)
	  frame = frame * 20;
	else if (lastframe < 22)
	  frame = frame * 10;
	//printf("got the final frame number: %d\n",frame);

	// translate the highlighting box according to their value
	// at y axis: highlightDCS->setTrans(0.13, -0.1, 0.33);
	highlightDCS->setTrans(0.13+(1.7*frame/196.0), -0.1, 0.33);
	highlightDCS->setScale(0.01f, 1.0f, 0.68f);
	//printf("moved by %f\n",0.13+(1.7*frame/196.0));
      }
      
      // reset mmode
      mmode = 0;
      shm_mode->write(&mmode);
    }
  }

  return dtkAugment::CONTINUE ; 
}    // end of postFrame

pfSequence* hydrationtimeline::getSeqNode(char* nodename) {
  // this function gets a sequence node that has been buried in a pfb file
  // when a dpf-transform file.seq file.pfb is performed, the seq file
  // is buried 7 or so group nodes under the named node
  //printf("find seq from node: %s\n",nodename);
  seq_node = (pfSequence*)display->world()->lookup(nodename,pfSequence::getClassType());

  if (seq_node == NULL) {
    seqParent = (pfGroup*)display->world()->lookup(nodename,pfGroup::getClassType());
    if (seqParent == NULL) 
      pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
	       "sgEditor:  "
	       "cannot find sequence node \"%s\"", nodename);
    else {
      // search down from this node for an seq until no more children
      numChildren = seqParent->getNumChildren();
      while (numChildren > 0 && seq_node == NULL) {
	testnode = seqParent->getChild(0);
	if (testnode->isOfType(pfSequence::getClassType())) {
	  seq_node = (pfSequence*)testnode;
	  //printf("seq found!!!\n");
	}
	else {
	  seqParent = (pfGroup*)testnode;
	  numChildren = seqParent->getNumChildren();
	}
      } // end while
    }
  }

  // else topnode was the seq node, just return it
  return (seq_node);
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new hydrationtimeline(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
