#include <stdio.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <Performer/pf/pfSequence.h>
#include <Performer/pr/pfString.h>
#include <dtk.h>

typedef double Box[2][3];

Box *getRubberbandBoxPtr ();

class hydrationtimeline : public dtkAugment {
public:
  hydrationtimeline(dpf *app) ;
  ~hydrationtimeline(void) ;
  int preFrame(void);
  int postConfig(void);
  int postFrame(void);
  
private:
  dpf* app;
  dpfDisplay *display;
  dtkSharedMem* shm_mode;
  dtkSharedMem* shm_seq;
  dtkSharedMem* shm_box;
  dtkSharedMem* shm_maxVol;

  // counter for the toggleNodes
  int cnt;
  pfDCS** plotDCSList;
  pfNode** plotnodes;
  string plotnames[10];
  string plotnodenames[10];

  pfDCS** boxplotDCSList;
  pfNode** boxplotnodes;
  string boxplotnames[10];
  string boxplotnodenames[10];

  int numBars;
  pfNode* backgroundnode;
  pfDCS* backgroundDCS;
  pfNode** backgroundnode2;
  pfDCS* backgroundDCS2;
  pfNode* highlightnode;
  pfDCS* highlightDCS;
  pfNode* highlightnode2;
  pfDCS* highlightDCS2;
  pfCoord syscoord;
  pfMatrix coordMatrix;
  int i;
  int mmode;
  char seqnode[100];
  int highlight;
  pfSequence *sq;
  pfSequence* getSeqNode(char* nodename);
  pfSequence *seq_node;
  pfGroup *seqParent;
  int numChildren;
  pfNode* testnode;
  int nstep;
  int frame;
  int repeat;
  float scale[10];
  char compfile[250];
  FILE *cFile;
  int pargc ;
  char **pargv ;
  float totalA,totalB,totalC;
  char line[2000];
  char numb[50];
  int lastframe;
  // the box we are clipping against
  Box *currentBox;
  float x1,x2,y1,y2,z1,z2,zz1,zz2;
  float dx1,dx2,dy1,dy2,dz1,dz2;
  float minMax[6];
  float maxVol;
  int currentBkgrd;
} ;

// create object and set description.
// be sure to validate it!
hydrationtimeline::hydrationtimeline(dpf *app_in) :
  dtkAugment("hydrationtimeline")
{
  //printf("start of constructor\n");
  setDescription(" loads files specified by DPF_HYDRATIONTIMELINE_FILES"
		 " environment variable; all objects whose centers"
		 " lie in the user-selected area are visible;"
                 " otherwise they are not visible.") ;
  app = app_in;
  validate();
  cnt = 0 ;
  numBars = 2;
  plotDCSList = new pfDCS*[100];
  plotnodes = new pfNode*[100];
  plotnames[0] = "plotCaO.savg";
  plotnames[1] = "plotCaOH2.savg";
  plotnames[2] = "plotH2O.savg";
  plotnames[3] = "yellowbox.savg";
  plotnames[4] = "purplebox.savg";
  plotnames[5] = "pinkbox.savg";
  plotnames[6] = "aquabox.savg";
  plotnames[7] = "ltbluebox.savg";
  plotnames[8] = "ltgreenbox.savg";
  plotnodenames[0] = "plotCaO";
  plotnodenames[1] = "plotCaOH2";
  plotnodenames[2] = "plotH2O";
  plotnodenames[3] = "yellowbox";
  plotnodenames[4] = "purplebox";
  plotnodenames[5] = "pinkbox";
  plotnodenames[6] = "aquabox";
  plotnodenames[7] = "ltbluebox";
  plotnodenames[8] = "ltgreenbox";

  boxplotDCSList = new pfDCS*[100];
  boxplotnodes = new pfNode*[100];
  boxplotnames[0] = "boxplotCaO.savg";
  boxplotnames[1] = "boxplotCaOH2.savg";
  boxplotnames[2] = "boxplotH2O.savg";
  boxplotnames[3] = "yellowbox.savg";
  boxplotnames[4] = "purplebox.savg";
  boxplotnames[5] = "pinkbox.savg";
  boxplotnames[6] = "aquabox.savg";
  boxplotnames[7] = "ltbluebox.savg";
  boxplotnames[8] = "ltgreenbox.savg";
  boxplotnodenames[0] = "boxplotCaO";
  boxplotnodenames[1] = "boxplotCaOH2";
  boxplotnodenames[2] = "boxplotH2O";
  boxplotnodenames[3] = "yellowbox";
  boxplotnodenames[4] = "purplebox";
  boxplotnodenames[5] = "pinkbox";
  boxplotnodenames[6] = "aquabox";
  boxplotnodenames[7] = "ltbluebox";
  boxplotnodenames[8] = "ltgreenbox";

  backgroundnode2 = new pfNode*[11];
  nstep = 1;
  cFile = NULL;
  currentBox = NULL;

  // Check if there is a shared memory segment "hydrationtimeline" 
  shm_mode = new dtkSharedMem(sizeof(int), "hydration-plot");
  if(!shm_mode || shm_mode->isInvalid()) 
    shm_mode = NULL ;

  mmode = 0;
  if (shm_mode)
    shm_mode->write(&mmode);

  shm_seq = new dtkSharedMem(sizeof(char)*100, "seqName");
  if(!shm_seq || shm_seq->isInvalid())
    shm_seq = NULL ;

  shm_box = new dtkSharedMem(sizeof(float)*6, "boxMinMax");
  if(!shm_box || shm_box->isInvalid())
    shm_box = NULL ;

  shm_maxVol = new dtkSharedMem(sizeof(float), "maxVolFraction");
  if(!shm_maxVol || shm_maxVol->isInvalid())
    shm_maxVol = NULL ;
  maxVol = 0.1;
  currentBkgrd = 0;
} ;

// destructor
hydrationtimeline::~hydrationtimeline(void) {
  if (shm_mode)
    dtkSharedMem_destroy("hydrationtimeline");
  if (cFile != NULL)
    fclose(cFile);
}

#define MIN_BUTTONS 3

int hydrationtimeline::preFrame(void)
{
  //printf("start of preFrame\n");
  display = app->display();
  if(!display) return DTK_CALLBACK_ERROR;

  // load in the colored boxes - get number or use default of 3
  char *env = getenv("DPF_HYDRATIONTIMELINE_LEVELS") ;
  if (env)
    numBars = atoi(env);
  //printf("number of bars in hydrationtimeline: %d\n",numBars);

  // load in the necessary number of colored bars
  for (i = 0; i < numBars; ++i) {
    plotnodes[i] = pfdLoadFile(plotnames[i].c_str());
    boxplotnodes[i] = plotnodes[i]->clone(0);
  }
    
  // define the background for the boxes and hang it on the scene node
  backgroundnode2[0] = pfdLoadFile("back0.05.savg");
  backgroundnode2[1] = pfdLoadFile("back0.1.savg");
  backgroundnode2[2] = pfdLoadFile("back0.2.savg");
  backgroundnode2[3] = pfdLoadFile("back0.3.savg");
  backgroundnode2[4] = pfdLoadFile("back0.4.savg");
  backgroundnode2[5] = pfdLoadFile("back0.5.savg");
  backgroundnode2[6] = pfdLoadFile("back0.6.savg");
  backgroundnode2[7] = pfdLoadFile("back0.7.savg");
  backgroundnode2[8] = pfdLoadFile("back0.8.savg");
  backgroundnode2[9] = pfdLoadFile("back0.9.savg");
  backgroundnode2[10] = pfdLoadFile("back1.0.savg");
  backgroundnode = pfdLoadFile("backfull.savg");
  backgroundDCS = new pfDCS();
  backgroundDCS->setTrans(.3f, 1.0f, -.2f);
  backgroundDCS->setScale(.3f, .3f, .3f);
  backgroundDCS2 = new pfDCS();
  backgroundDCS2->setTrans(.3f, 1.0f, -.9f);
  backgroundDCS2->setScale(.3f, .3f, .3f);
  backgroundDCS->setName("hydrationtimeline");
  backgroundDCS->addChild(backgroundnode);
  backgroundDCS2->setName("hydrationtimeline2");
  backgroundDCS2->addChild(backgroundnode2[2]);

  // orient the box DCSs with respect to the background
  for (i = 0; i < numBars; ++i) {
    plotDCSList[i] = new pfDCS();
    plotDCSList[i]->setTrans(0.13, -0.1, 0.35);
    plotDCSList[i]->setScale(1.7f, 1.0f, 13.0f);
    plotDCSList[i]->addChild(plotnodes[i]);
    plotDCSList[i]->setName(plotnodenames[i].c_str());

    boxplotDCSList[i] = new pfDCS();
    boxplotDCSList[i]->setTrans(0.13, -0.1, 0.35);
    boxplotDCSList[i]->setScale(1.7f, 1.0f, 13.0f);
    boxplotDCSList[i]->addChild(boxplotnodes[i]);
    boxplotDCSList[i]->setName(boxplotnodenames[i].c_str());
  }

  // bring in and orient the highlighting box
  highlightnode = pfdLoadFile("lightbluebox.savg");
  highlightDCS = new pfDCS();
  highlightDCS->setTrans(0.13, -0.1, 0.33);
  highlightDCS->setScale(0.01f, 1.0f, 0.68f);
  highlightDCS->addChild(highlightnode);
  highlightDCS->setName("highlightnode");

  highlightnode2 = highlightnode->clone(0);
  highlightDCS2 = new pfDCS();
  highlightDCS2->setTrans(0.13, -0.1, 0.33);
  highlightDCS2->setScale(0.01f, 1.0f, 0.68f);
  highlightDCS2->addChild(highlightnode2);
  highlightDCS2->setName("highlightnode2");

  // Get the pointer to the changeable box.
  currentBox = (Box *) getRubberbandBoxPtr ();

  // just call buttonNavControl::preFrame() once at the start.
  return DTK_REMOVE_CALLBACK;
}


int hydrationtimeline::postConfig(void) {
  //printf("start of postConfig\n");
  return DTK_REMOVE_CALLBACK;
}

int hydrationtimeline::postFrame(void) {
  //printf("start of postFrame\n");

  // check mmode
  if (shm_mode)
    shm_mode->read(&mmode);
  
  if (mmode == 1) {
    // time to update highlight line
    
    // find the node
    if (shm_seq)
      shm_seq->read(seqnode);
    
    if (seqnode != NULL) {
      printf("print the name: %s\n",seqnode);
      
      // see if this is a seq node
      sq = getSeqNode(seqnode);
      if (sq == NULL) {
	printf("null seq\n");
	nstep = 1;
      }
      else {
	// find the timestep
	printf("good seq\n");
	// get the last frame number (196, 20, 10) to get overall frame number
	lastframe = sq->getNumChildren() - 1;
	printf("last frame: %d\n",lastframe);
	frame = sq->getFrame(&repeat);
	//printf("got the first frame number: %d\n",frame);
	
	// if this sequence file is not every frame, multiply frame number
	if (lastframe < 11)
	  frame = frame * 20;
	else if (lastframe < 22)
	  frame = frame * 10;
	//printf("got the final frame number: %d\n",frame);

	// translate the highlighting box according to their value
	// at y axis: highlightDCS->setTrans(0.13, -0.1, 0.33);
	highlightDCS->setTrans(0.13+(1.7*frame/196.0), -0.1, 0.33);
	highlightDCS->setScale(0.01f, 1.0f, 0.68f);
	highlightDCS2->setTrans(0.13+(1.7*frame/196.0), -0.1, 0.33);
	highlightDCS2->setScale(0.01f, 1.0f, 0.68f);
	//printf("moved by %f\n",0.13+(1.7*frame/196.0));
      }
      
      // reset mmode
      mmode = 0;
      shm_mode->write(&mmode);
    }
  }
  
  if (mmode == 2) {
    //printf("mmode = 2\n");
    // remake plot lines for the components
    // reset shared memory here

    x1 = (*currentBox)[0][0];
    x2 = (*currentBox)[1][0];
    y1 = (*currentBox)[0][1];
    y2 = (*currentBox)[1][1];
    z1 = (*currentBox)[0][2];
    z2 = (*currentBox)[1][2];
    // currentBox is given in world coordinates
    
    //syscoord.xyz.set (x1,y1,z1);
    //display->worldToDpf(&syscoord);
    //dx1 = syscoord.xyz[0];
    //dy1 = syscoord.xyz[1];
    //dz1 = syscoord.xyz[2];
    //syscoord.xyz.set (x2,y2,z2);
    //display->worldToDpf(&syscoord);
    //dx2 = syscoord.xyz[0];
    //dy2 = syscoord.xyz[1];
    //dz2 = syscoord.xyz[2];
    //printf("current box: %f %f %f %f %f %f\n",x1,x2,y1,y2,z1,z2);
    //printf("worldtodpf box: %f %f %f %f %f %f\n",dx1,dx2,dy1,dy2,dz1,dz2);
    
    // now translate back to experimental coordinates
    dx1 = 50.0*(x1+.5);
    dx2 = 50.0*(x2+.5);
    float zz1,zz2;
    zz1 = z1;
    zz2 = z2;
    dz1 = -50.0*(y1-2.0);
    dz2 = -50.0*(y2-2.0);
    dy1 = 50.0*(zz1+0.5);
    dy2 = 50.0*(zz2+0.5);
    //printf("rescaled box: %f %f %f %f %f %f\n",dx1,dx2,dy1,dy2,dz1,dz2);
    // these are the coordinates in the original data to look through
    // store these in shared memory in order
    minMax[0] = dx1;
    minMax[1] = dx2;
    if (dx1 > dx2) {
      minMax[0] = dx2;
      minMax[1] = dx1;
    }
    minMax[2] = dy1;
    minMax[3] = dy2;
    if (dy1 > dy2) {
      minMax[2] = dy2;
      minMax[3] = dy1;
    }
    minMax[4] = dz1;
    minMax[5] = dz2;
    if (dz1 > dz2) {
      minMax[4] = dz2;
      minMax[5] = dz1;
    }

    if (shm_box)
      shm_box->write(minMax);
    
    // reset mmode
    mmode = 0;
    shm_mode->write(&mmode);
  }
  
  if (mmode == 3) {  
    // update the plot background

    if (shm_maxVol)
      shm_maxVol->read(&maxVol);
    //printf("maxVol is now: %f\n",maxVol);

    // remove the current background
    backgroundDCS2->removeChild(backgroundnode2[currentBkgrd]);

    // decide which one to put on
    if (maxVol > .9)
      currentBkgrd = 10;
    else if (maxVol > .8)
      currentBkgrd = 9;
    else if (maxVol > .7)
      currentBkgrd = 8;
    else if (maxVol > .6)
      currentBkgrd = 7;
    else if (maxVol > .5)
      currentBkgrd = 6;
    else if (maxVol > .4)
      currentBkgrd = 5;
    else if (maxVol > .3)
      currentBkgrd = 4;
    else if (maxVol > .2)
      currentBkgrd = 3;
    else if (maxVol > .1)
      currentBkgrd = 2;
    else if (maxVol > .05)
      currentBkgrd = 1;
    else
      currentBkgrd = 0;
    //printf("new background: %d\n",currentBkgrd);

    // put it on
    backgroundDCS2->addChild(backgroundnode2[currentBkgrd]);

    // load in new plot nodes and rescale them
    if (maxVol < 0.001)
      maxVol = 0.001;

    for (i = 0; i < numBars; ++i) {
      // delete old node
      boxplotDCSList[i]->removeChild(boxplotnodes[i]);
      pfDelete(boxplotnodes[i]);

      // read in new file
      // see if this is a good file name
      FILE *fp = fopen(boxplotnames[i].c_str(), "r");     
      if (!fp) {
	dtkMsg.add(DTKMSG_ERROR,
	  "hydration-timeline:: error opening input file %s\n",boxplotnames[i].c_str());
	return DTK_CALLBACK_ERROR;
      }
      else {
	// read the file
	boxplotnodes[i] = pfdLoadFile(boxplotnames[i].c_str());
	boxplotDCSList[i]->addChild(boxplotnodes[i]);
      }

      // original for 0.1 scale: boxplotDCSList[i]->setScale(1.7,1.0,13.0);
      boxplotDCSList[i]->setScale(1.7,1.0,13.0/(currentBkgrd+1));
    }

    // reset mmode
    mmode = 0;
    shm_mode->write(&mmode);
  }

  return dtkAugment::CONTINUE ; 
}    // end of postFrame

pfSequence* hydrationtimeline::getSeqNode(char* nodename) {
  // this function gets a sequence node that has been buried in a pfb file
  // when a dpf-transform file.seq file.pfb is performed, the seq file
  // is buried 7 or so group nodes under the named node
  //printf("find seq from node: %s\n",nodename);
  seq_node = (pfSequence*)display->world()->lookup(nodename,pfSequence::getClassType());

  if (seq_node == NULL) {
    seqParent = (pfGroup*)display->world()->lookup(nodename,pfGroup::getClassType());
    if (seqParent == NULL) 
      pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
	       "sgEditor:  "
	       "cannot find sequence node \"%s\"", nodename);
    else {
      // search down from this node for an seq until no more children
      numChildren = seqParent->getNumChildren();
      while (numChildren > 0 && seq_node == NULL) {
	testnode = seqParent->getChild(0);
	if (testnode->isOfType(pfSequence::getClassType())) {
	  seq_node = (pfSequence*)testnode;
	  //printf("seq found!!!\n");
	}
	else {
	  seqParent = (pfGroup*)testnode;
	  numChildren = seqParent->getNumChildren();
	}
      } // end while
    }
  }

  // else topnode was the seq node, just return it
  return (seq_node);
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new hydrationtimeline(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
