/*
 * pfclean.c - ".clean" pseudo-loader.
 *
 * To load a file and clean it, say:
 *	perfly <filename>.clean
 *
 * All it does is call pfdCleanTree on the node
 *
 */

#include <stdio.h>
#include <ctype.h>

#ifdef WIN32
#include <windows.h>

#define PFPFB_DLLEXPORT __declspec(dllexport)
#else
#define PFPFB_DLLEXPORT

#endif /* WIN32 */

#include <Performer/pfdu.h>
#include <Performer/pf/pfSCS.h>

char *strrnchr(char *s, int n, int c)
{
    char *p = NULL;
    int i;
    for (i = 0; i < n && s[i] != '\0'; ++i)
	if (s[i] == c)
	    p = s+i;
    return p;
}

extern "C" PFPFB_DLLEXPORT pfNode *
pfdLoadFile_clean(char *fileName)
{
  char *lastdot ;
  if ((lastdot = strrchr(fileName, '.')) == NULL) {
      pfNotify(PFNFY_WARN, PFNFY_USAGE,
	       "pfdLoadFile_clean: bad file name %s", fileName);
      return NULL;
    }
    
  char realName[PF_MAXSTRING] ;
  strncpy(realName, fileName, lastdot-fileName);
  realName[lastdot-fileName] = '\0';
  
#if 0
  printf("fileName = %p\n",fileName) ;
  printf("lastdot = %p\n",lastdot) ;
  printf("realName = \"%s\"\n",realName) ;
#endif

  pfNode *node = pfdLoadFile(realName);
  if (node == NULL)
    return NULL; /* error message printed already */
  
  pfNotify(PFNFY_NOTICE, PFNFY_USAGE,
	   "pfdLoadFile_clean: cleaning file %s", realName);
  return (pfdCleanTree(node, 0)) ;
  
}
