/*
 *  Created by;
 *    Daniel F. Johnston
 *    National Research Council of Canada
 *    2003-July-09
 *
 *  Provide without restrictions to the Open-GL Performer
 *    community in the hope that they find it useful.
 */

/*
 * pflight.C - ".light" pseudo-loader.
 *
 * To add a new light to any scene, say:
 *      perfly <filename> <r>,<g>,<b>.light
 *      (where r,g,b define the color of the new light source)
 * Also works with the ".trans" and ".rot" pseudo-loaders so you can

 *      position the new light whereever you need it
 *
 * The rgb color numbers can have decimal points (but don't
 * use exponential notation).
 */

#include <stdio.h>
#include <Performer/pf.h>
#include <Performer/pfutil.h>
#include <Performer/pfdu.h>
#include <Performer/pr/pfGeoMath.h>
#include <Performer/pf/pfLightSource.h>

char *strrnchr(char *s, int n, int c)
{
    char *p = NULL;
    int i;
    for (i = 0; i < n && s[i] != '\0'; ++i)
        if (s[i] == c)
            p = s+i;
    return p;
}

extern "C" pfNode *
pfdLoadFile_light(char *fileName)
{
    float red, green, blue;
    char params[PATH_MAX];

    char *lastdot = strrchr(fileName, '.');
    if (lastdot != NULL)
    {
        strncpy(params, fileName, lastdot-fileName);
        params[lastdot-fileName] = '\0';
        if (sscanf(params, "%f,%f,%f", &red, &green, &blue) != 3)
        {
            pfNotify(PFNFY_WARN, PFNFY_USAGE,
                    "pfdLoadFile_light: bad file name %s", fileName);
            return NULL;
        }
    }

    //pfNotify(PFNFY_INFO, PFNFY_PRINT, "Add light (%.9g,%.9g,%.9g) to
    //scene\n", red,green,blue);
    pfLightSource *lamp = new pfLightSource();
    PFASSERTALWAYS(lamp != NULL);

    // Create and configure light frustum
    pfFrustum *lightFrust = new pfFrustum;
    PFASSERTALWAYS(lightFrust != NULL);
    lightFrust->makeSimple(60.0f);
    lightFrust->setNearFar(1.0f, 100.0f);

    // set light direction
    lamp->setAttr(PFLS_PROJ_FRUST, lightFrust);
    // set light source color
    lamp->setColor(PFLT_DIFFUSE, red, green, blue );
    lamp->setVal(PFLS_INTENSITY, .5f);

    return lamp;
}
