#include <stdio.h>
#include <Performer/pfdu.h>

// print the contents of a pfdGeom to file

// HASN'T BEEN TESTED WITH INDEXED GEOMETRY OR TEXTURES

void printGeom(pfdGeom *geom, FILE *file=NULL) {

  if(!file) 
    file = stdout ;

  if (!geom)
    fprintf(file, "printGeom: NULL pfdGeom pointer\n") ;
    
  int i, j ;
  
  fprintf(file,
	  "printGeom:\n") ;

  char *flagname ;
  if (!(geom->flags & PFD_CONCAVE) && !(geom->flags & PFD_INDEXED))
    flagname = "not concave and not indexed" ;
  else if (!(geom->flags & PFD_CONCAVE) && (geom->flags & PFD_INDEXED))
    flagname = "not concave and indexed" ;
  else if ((geom->flags & PFD_CONCAVE) && !(geom->flags & PFD_INDEXED))
    flagname = "concave and not indexed" ;
  else if ((geom->flags & PFD_CONCAVE) && (geom->flags & PFD_INDEXED))
    flagname = "concave and indexed" ;
  fprintf(file,
	  "   flags = %s\n",flagname) ;

  char *bindname[] = {
    "PFGS_OFF",
    "PFGS_OVERALL",
    "PFGS_PER_PRIM",
    "PFGS_PER_VERTEX"} ;
  fprintf(file,
	  "   nbind = %s, cbind = %s\n",
	  bindname[geom->nbind], bindname[geom->cbind]) ;
  
  bool textures ;
  textures = false ;
  for (i=0; i<PF_MAX_TEXTURES; i++) {
    if ((geom->tbind)[i] != PFGS_OFF) {
      textures = true ;
      break ;
    }
  }
  if (textures) {
    fprintf(file,"   tbind:\n") ;
    for (i=0; i<PF_MAX_TEXTURES; i++) {
      fprintf(file,"      %3.3d = %d\n",i,(geom->tbind)[i]) ;
    }
  } else 
    fprintf(file,"   tbind[*] = PFGS_OFF\n") ;
  
  char *primname[] = {"PFGS_POINTS", "PFGS_LINES", 
		      "PFGS_LINESTRIPS", "PFGS_TRIS", 
		      "PFGS_QUADS", "PFGS_TRISTRIPS", 
		      "PFGS_FLAT_LINESTRIPS", "PFGS_FLAT_TRISTRIPS", 
		      "PFGS_POLYS", "PFGS_TRIFANS", 
		      "PFGS_FLAT_TRIFANS", "PFGS_NUM_PRIMS"} ;
  
  fprintf(file,
	  "   numVerts = %d, primtype = %s, pixelsize = %f\n",
	  geom->numVerts, primname[geom->primtype], geom->pixelsize) ;
  
  if (!(geom->flags & PFD_INDEXED)) { // not indexed
    for (i=0; i<geom->numVerts; i++) {
      fprintf(file,
	      "   coords[%d] = (%f, %f, %f)\n",
	      i, (geom->coords)[i][0], (geom->coords)[i][1], (geom->coords)[i][2]) ;
    }
    if (geom->nbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   norms[%d] = (%f, %f, %f)\n",
		i, (geom->norms)[i][0], (geom->norms)[i][1], (geom->norms)[i][2]) ;
      }
    else if (geom->nbind != PFGS_OFF)
      fprintf(file,
	      "   norms[0] = (%f, %f, %f)\n",
	      (geom->norms)[0][0], (geom->norms)[0][1], (geom->norms)[0][2]) ;
    if (geom->cbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   colors[%d] = (%f, %f, %f %f)\n",
		i, (geom->colors)[i][0], (geom->colors)[i][1], 
		(geom->colors)[i][2], (geom->colors)[i][3]) ;
      }
    else if (geom->cbind != PFGS_OFF) {
      fprintf(file,
	      "   colors[0] = (%f, %f, %f %f)\n",
	      ((geom->colors))[0][0], ((geom->colors))[0][1], 
	      ((geom->colors))[0][2], ((geom->colors))[0][3]) ;
    }
    if (textures) {
      for (i=0; i<geom->numVerts; i++) {
	if ((geom->tbind)[i] != PFGS_OFF)
	  for (j=0; j<PF_MAX_TEXTURES; j++) {
	    fprintf(file,
		    "   texCoords[%d][%d] = (%f, %f)\n",
		    j, i, ((geom->texCoords)[j][i])[0], ((geom->texCoords)[j][i])[1]) ;
	  }
      }
    }
  } else { // indexed
    // index attributes
    for (i=0; i<geom->numVerts; i++) {
      fprintf(file,
	      "   coordList[%d] = (%f, %f, %f)\n",
	      i, (geom->coordList)[i][0], (geom->coordList)[i][1], (geom->coordList)[i][2]) ;
    }
    if (geom->nbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   normList[%d] = (%f, %f, %f)\n",
		i, (geom->normList)[i][0], (geom->normList)[i][1], (geom->normList)[i][2]) ;
      }
    else if (geom->nbind != PFGS_OFF)
      fprintf(file,
	      "   normList[0] = (%f, %f, %f)\n",
	      (geom->normList)[0][0], (geom->normList)[0][1], (geom->normList)[0][2]) ;
    if (geom->cbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   colorList[%d] = (%f, %f, %f %f)\n",
		i, ((geom->colorList)[i])[0], (geom->colorList)[i][1], 
		(geom->colorList)[i][2], (geom->colorList)[i][3]) ;
      }
    else if (geom->cbind != PFGS_OFF)
      fprintf(file,
	      "   colorList[0] = (%f, %f, %f %f)\n",
	      (geom->colorList)[0][0], (geom->colorList)[0][1], 
	      (geom->colorList)[0][2], (geom->colorList)[0][3]) ;
    if (textures) {
      for (i=0; i<geom->numVerts; i++) {
	if ((geom->tbind)[i] != PFGS_OFF)
	  for (j=0; j<PF_MAX_TEXTURES; j++) {
	    fprintf(file,
		    "   texCoordList[%d][%d] = (%f, %f)\n",
		    j, i, ((geom->texCoordList)[j][i])[0], ((geom->texCoordList)[j][i])[1]) ;
	  }
      }
    }

    // index lists
    for (i=0; i<geom->numVerts; i++) {
      fprintf(file,
	      "   icoords[%d] = %d\n",
	      i, (geom->icoords)[i]) ;
    }
    if (geom->nbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   inorms[%d] = %d\n",
		i, (geom->inorms)[i]) ;
      }
    else if (geom->nbind != PFGS_OFF)
      fprintf(file,
	      "   inorms[0] = %d\n",
	      (geom->inorms)[0]) ;
    if (geom->cbind == PFGS_PER_VERTEX)
      for (i=0; i<geom->numVerts; i++) {
	fprintf(file,
		"   icolors[%d] = %d\n",
		i, (geom->icolors)[i]) ;
      }
    else if (geom->cbind != PFGS_OFF)
      fprintf(file,
	      "   icolors[0] = %d\n",
	      (geom->icolors)[0]) ;
    if (textures) {
      for (i=0; i<geom->numVerts; i++) {
	if ((geom->tbind)[i] != PFGS_OFF)
	  for (j=0; j<PF_MAX_TEXTURES; j++) {
	    fprintf(file,
		    "   itexCoords[%d][%d] = %d\n",
		    j, i, (geom->itexCoords)[j][i]) ;
	  }
      }
    }
    
  } //if (!(geom->flags & PFD_INDEXED))
}

