//
// Copyright 2001, Silicon Graphics, Inc.
// ALL RIGHTS RESERVED
//
// UNPUBLISHED -- Rights reserved under the copyright laws of the
// United States.   Use of a copyright notice is precautionary only
// and does not imply publication or disclosure.
//
// U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND:
// Use, duplication or disclosure by the Government is subject to
// restrictions as set forth in FAR 52.227.19(c)(2) or subparagraph
// (c)(1)(ii) of the Rights in Technical Data and Computer Software
// clause at DFARS 252.227-7013 and/or in similar or successor
// clauses in the FAR, or the DOD or NASA FAR Supplement.
// Contractor/manufacturer is Silicon Graphics, Inc.,
// 2011 N. Shoreline Blvd. Mountain View, CA 94039-7311.
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that (i) the above copyright notices and this
// permission notice appear in all copies of the software and related
// documentation, and (ii) the name of Silicon Graphics may not be
// used in any advertising or publicity relating to the software
// without the specific, prior written permission of Silicon Graphics.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
// INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
// DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
// THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
// OR PERFORMANCE OF THIS SOFTWARE.
//

//////////////////////////////////////////////////////////////////////
// IFLLoader.h
//////////////////////////////////////////////////////////////////////

#ifndef IFL_LOADER_H
#define IFL_LOADER_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include <Volumizer2/VolEnums.h>
#include "VolumeLoader.h"

class vzObject;
class vzParameterVolumeTexture;
class iflFile;

//////////////////////////////////////////////////////////////////////
// class
//////////////////////////////////////////////////////////////////////

class IFLLoader :public VolumeLoader{

    //////////////////////////////////////////////////////////////////
    // public
    //////////////////////////////////////////////////////////////////

    public:

        //////////////////////////////////////////////////////////////
        // factories
        //////////////////////////////////////////////////////////////

        static IFLLoader* open(const char fileName[]);
    
        //////////////////////////////////////////////////////////////
        // destructor
        //////////////////////////////////////////////////////////////

        ~IFLLoader();

        //////////////////////////////////////////////////////////////
        // attributes
        //////////////////////////////////////////////////////////////

        vzExternalTextureFormat getFormat() const;
        vzTextureType getType() const;
        void getSize(int size[3]) const;

        //////////////////////////////////////////////////////////////
        // actions
        //////////////////////////////////////////////////////////////

        vzParameterVolumeTexture* loadVolume(vzInternalTextureFormat
            format = VZ_DEFAULT_INTERNAL_FORMAT);

        int preloadVolume();

        bool loadBrick(void* data,
            const int position[3], const int size[3]);

	void readTheTile(unsigned char* data);

    //////////////////////////////////////////////////////////////////
    // private
    //////////////////////////////////////////////////////////////////

    private:

        //////////////////////////////////////////////////////////////
        // variables
        //////////////////////////////////////////////////////////////

        iflFile*
            m_file;

        vzExternalTextureFormat
            m_format;

        vzTextureType
            m_type;

        int
            m_size[3];

        //////////////////////////////////////////////////////////////
        // constructor
        //////////////////////////////////////////////////////////////

        IFLLoader(iflFile* file, vzExternalTextureFormat format,
            vzTextureType type, const int size[3]);

        //////////////////////////////////////////////////////////////
        // callbacks
        //////////////////////////////////////////////////////////////

        static void deleteCallback(vzObject* object, void* data);

};

//////////////////////////////////////////////////////////////////////
// IFLLoader.h
//////////////////////////////////////////////////////////////////////

#endif // IFL_LOADER_H
