// Copyright  2002 Silicon Graphics, Inc.  All Rights Reserved.
// 
// The recipient ("Recipient") of this software, including as modified
// ("Software") may reproduce, redistribute, use, and derive works from the
// Software without restriction, subject to the following conditions:
//
//  Redistribution of the Software in any form must reproduce this entire
// notice, including as modified in accordance with these provisions
// ("Notice");
//
//  Any Recipient who modifies and subsequently redistributes the Software
// shall add information to this Notice to sufficiently identify the
// Recipient's modifications;
//
//  Recipient may not use the name(s) of any previous Recipient to endorse
// or promote any products derived from the Software without prior express
// written permission from such previous Recipient.
//
// THE SOFTWARE IS PROVIDED "AS IS," WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTY OR CONDITION, INCLUDING WITHOUT LIMITATION ANY WARRANTIES OR
// CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
// PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  PATENT LICENSES, IF ANY,
// PROVIDED HEREIN DO NOT APPLY TO COMBINATIONS OF THIS PROGRAM WITH OTHER
// SOFTWARE, OR ANY OTHER PRODUCT WHATSOEVER.  IN NO EVENT WILL THE
// ORIGINATOR OR SUBSEQUENT RECIPIENT OF THE SOFTWARE BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES ARISING IN RELATION TO THE SOFTWARE, ITS USE, OR THESE
// PROVISIONS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR NEGLIGENCE OR OTHER TORT, EVEN IF
// PRE-ADVISED OF THE PROSPECT OF SUCH DAMAGES.
//
// This Software was developed at private expense; if acquired under an
// agreement with the USA government or any contractor thereto, it is
// acquired as "commercial computer software" subject to the provisions of
// this license agreement, as specified in (a) 48 CFR 12.212 of the FAR;
// or, if acquired for Department of Defense units, (b) 48 CFR 227-7202 of
// the DoD FAR Supplement; or sections succeeding thereto.
// 
// Originator:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, Mountain
// View, CA  94043.  http://www.sgi.com

//////////////////////////////////////////////////////////////////////
// MiscUtil.h
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Some utility functions used in the demo source code
//////////////////////////////////////////////////////////////////////

#ifndef UTIL_FUNCS_H
#define UTIL_FUNCS_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////
// Volumizer 2.0
#include <Volumizer2/Error.h>
#include <Volumizer2/Shape.h>
#include <Volumizer2/Appearance.h>
#include <Volumizer2/Parameter.h>
#include <Volumizer2/ParameterVolumeTexture.h>
#include <Volumizer2/ParameterLookupTable.h>

enum Platforms {
    IR,
    IMPACT,
    VPRO
};

// Simple parameter related convenience functions
int getBytesPerChannel(vzTextureType type);
int getNumChannels(vzExternalTextureFormat format);
int getBytesPerTexel(vzParameterVolumeTexture *tex);

// Take an input string and produce the resulting 
// texture format and type
vzTextureType getTextureType(const char *typeName);
vzExternalTextureFormat getDataFormat(const char *formatName);

// Return the volume texture, given the shape 
vzParameterVolumeTexture *getVolumeTexture(vzShape *shape);

// Return the texture table, given the shape
vzParameterLookupTable *getLookupTable(vzShape *shape);

// Querry the graphics hardware for the amount of texture memory available
unsigned int getTextureMemorySize(enum Platforms *platform);

// Render the faces of each bricks using GL_QUADS. Set the glPolygonMode
// to render wireframe of the bricks
void renderCube(double coords[6]);

int isPowerOfTwo (int value);
int augmentPowerOfTwo(int x);
int truncatePowerOfTwo(int value);
 
#endif
