//////////////////////////////////////////////////////////////////////
// TFChannel.h
//////////////////////////////////////////////////////////////////////

#ifndef TF_CHANNEL_H
#define TF_CHANNEL_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////
#include <iostream.h>

//////////////////////////////////////////////////////////////////////
// class
//////////////////////////////////////////////////////////////////////

class TFChannel {
  
    friend class TFManager;
    
    //////////////////////////////////////////////////////////////////
    // public
    //////////////////////////////////////////////////////////////////
    
    public:

        //////////////////////////////////////////////////////////////
        // constants
        //////////////////////////////////////////////////////////////

        enum Interpolation {
            INTERPOLATION_WINDOW_LEVEL,
            INTERPOLATION_LINEAR,
            INTERPOLATION_SPLINES,
            INTERPOLATION_GAUSSIAN,
            INTERPOLATION_PAINT,

            INTERPOLATION_COUNT
        };
    
        //////////////////////////////////////////////////////////////
        // factory
        //////////////////////////////////////////////////////////////

        TFChannel* create(Interpolation interpolation,
            int length);
        TFChannel* createFromStream(istream& stream,
            int length);

        //////////////////////////////////////////////////////////////
        // destructor
        //////////////////////////////////////////////////////////////

        ~TFChannel();

        //////////////////////////////////////////////////////////////
        // constructors
        //////////////////////////////////////////////////////////////

       TFChannel(Interpolation inter, int length,
		 int rampCenter, int rampWidth);
       TFChannel(istream& stream, int length);
        //////////////////////////////////////////////////////////////
        // attributes
        //////////////////////////////////////////////////////////////

        void setInterpolation(Interpolation interpolation);
        Interpolation getInterpolation();

        // set value directly for paint interpolation
        void setValue(int index, float value);
        const float* getValues();

        void setLength(int length);
        int  getLength() const;

        // current ramp center for linear interpolation
        void setRampCenter(int center);
        int  getRampCenter() const;
  
        // current ramp width for linear interpolation
        void setRampWidth(int width);
        int  getRampWidth() const;
 
        // mark the current channel dirty, requiring re-evaluation
        void setDirty(bool flag);
        bool isDirty() const;

        //////////////////////////////////////////////////////////////
        // actions
        //////////////////////////////////////////////////////////////

        void evaluate();

        void saveToStream(ostream& stream) const;


    //////////////////////////////////////////////////////////////////
    // private
    //////////////////////////////////////////////////////////////////

    private:

        //////////////////////////////////////////////////////////////
        // variables
        //////////////////////////////////////////////////////////////

        Interpolation
            m_interpolation;

        int
            m_length;

        float*
            m_values;

        int
            m_rampCenter,
            m_rampWidth;

        bool
            m_dirty;

	int pointCount;
	int* inpoints;
	float* invalues;

        //////////////////////////////////////////////////////////////
        // methods
        //////////////////////////////////////////////////////////////

        void generateRamps();

        void generateSplines();
        void initSpline(int x[], float y[], int n, float y2[]);
        float evaluateSpline(int xa[], float ya[], float y2a[],
            int n, int x);

        void generateGaussian();
        float evaluateGaussian(float x, float n, float mu);

        void generateLinearRamp();

        static float clamp(float value);

};

//////////////////////////////////////////////////////////////////////
// TFChannel.h - EOF
//////////////////////////////////////////////////////////////////////

#endif // TF_CHANNEL_H
