// Copyright  2002 Silicon Graphics, Inc.  All Rights Reserved.
// 
// The recipient ("Recipient") of this software, including as modified
// ("Software") may reproduce, redistribute, use, and derive works from the
// Software without restriction, subject to the following conditions:
//
//  Redistribution of the Software in any form must reproduce this entire
// notice, including as modified in accordance with these provisions
// ("Notice");
//
//  Any Recipient who modifies and subsequently redistributes the Software
// shall add information to this Notice to sufficiently identify the
// Recipient's modifications;
//
//  Recipient may not use the name(s) of any previous Recipient to endorse
// or promote any products derived from the Software without prior express
// written permission from such previous Recipient.
//
// THE SOFTWARE IS PROVIDED "AS IS," WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTY OR CONDITION, INCLUDING WITHOUT LIMITATION ANY WARRANTIES OR
// CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
// PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  PATENT LICENSES, IF ANY,
// PROVIDED HEREIN DO NOT APPLY TO COMBINATIONS OF THIS PROGRAM WITH OTHER
// SOFTWARE, OR ANY OTHER PRODUCT WHATSOEVER.  IN NO EVENT WILL THE
// ORIGINATOR OR SUBSEQUENT RECIPIENT OF THE SOFTWARE BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES ARISING IN RELATION TO THE SOFTWARE, ITS USE, OR THESE
// PROVISIONS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR NEGLIGENCE OR OTHER TORT, EVEN IF
// PRE-ADVISED OF THE PROSPECT OF SUCH DAMAGES.
//
// This Software was developed at private expense; if acquired under an
// agreement with the USA government or any contractor thereto, it is
// acquired as "commercial computer software" subject to the provisions of
// this license agreement, as specified in (a) 48 CFR 12.212 of the FAR;
// or, if acquired for Department of Defense units, (b) 48 CFR 227-7202 of
// the DoD FAR Supplement; or sections succeeding thereto.
// 
// Originator:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, Mountain
// View, CA  94043.  http://www.sgi.com



//////////////////////////////////////////////////////////////////////
// TFEditor.h
//////////////////////////////////////////////////////////////////////

#ifndef TF_EDITOR_H
#define TF_EDITOR_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

// C
#include <stdlib.h>

// X11
#include <X11/X.h>
#include <X11/keysym.h>

// Xt
#include <X11/Intrinsic.h>

// Motif
#include <Xm/Xm.h> 
#include <Xm/RepType.h> 
#include <Xm/RowColumn.h>   
#include <Xm/Frame.h>       
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/Scale.h>       
#include <Xm/ToggleB.h>     
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>
#include <Xm/FileSB.h>
#include <Xm/Protocols.h>

// OpenGL
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/GLwMDrawA.h>

// local
#include "TFManager.h"

//////////////////////////////////////////////////////////////////////
// callbacks
//////////////////////////////////////////////////////////////////////

typedef void (*TFUserFunctionPtr) (float* dataPtr, int width, 
    TFFormat format, void* data);

//////////////////////////////////////////////////////////////////////
// class
//////////////////////////////////////////////////////////////////////

class TFEditor {

    //////////////////////////////////////////////////////////////////
    // public
    //////////////////////////////////////////////////////////////////

    public:

        //////////////////////////////////////////////////////////////
        // constructor and destructor
        //////////////////////////////////////////////////////////////

        TFEditor(Widget parent, int width, TFFormat format);

        ~TFEditor();

        //////////////////////////////////////////////////////////////
        // attributes
        //////////////////////////////////////////////////////////////

        // notification callback
        void setUserFunction(TFUserFunctionPtr function, void* data);

        void getTableInfo(void*& dataPtr, int& width, TFFormat& format);

        //////////////////////////////////////////////////////////////
        // actions
        //////////////////////////////////////////////////////////////

        void popup();
        void popdown();

        bool saveToFile(const char filename[]);
        bool loadFromFile(const char fileName[]);

        void print();


    //////////////////////////////////////////////////////////////////
    // private
    //////////////////////////////////////////////////////////////////

    private:

        //////////////////////////////////////////////////////////////
        // constants
        //////////////////////////////////////////////////////////////

        static const float
            s_width,
            s_height,
            s_top,
            s_bottom,
            s_left,
            s_right;

        //////////////////////////////////////////////////////////////
        // variables
        //////////////////////////////////////////////////////////////

        Widget
            m_parent,
            m_plotArea,
            m_shell,
            m_windowLevelShell,
            m_interpolation,
            m_format,
	    m_builtIn,
            m_scaleWindow,
            m_scaleLevel,
            m_rgba,
            m_la,
            m_l;
  
        GLXContext
            m_context;

        GLint
            m_viewport[4];

        TFManager*
            m_manager;

        TFUserFunctionPtr
            m_userFunction;

        void*
            m_userData;

        TFManager::Channel
            m_currentChannel;

        int
            m_windowValue,
            m_levelValue,
            m_lastIndex;  // for paint mode

        float
            m_lastValue; // for paint mode

        bool
            m_paintBrushUp,
            m_windowLevelEnabled,
	    m_popped;
 
        //////////////////////////////////////////////////////////////
        // methods
        //////////////////////////////////////////////////////////////

        void create();
        void draw();
        void callUserFunction();

        void GL_area_button_input(XButtonEvent* event);
        void GL_area_motion_input(XMotionEvent* event);

        void motionButton1(int xpos, int ypos);
        void motionButton1Paint(int xpos, int ypos);

        void drawControlPoints(TFManager::Channel channel);
        void drawChannel(TFManager::Channel channel);

        void setCurrentChannel(TFManager::Channel channel);
        TFManager::Channel getCurrentChannel();
        TFChannel* getCurrentChannelPtr();

        // transform function to plot area
        float tf_index_2_x_plot_coord(int   index);
        float tf_value_2_y_plot_coord(float value);

        // transform plot area to function
        int   x_plot_coord_2_tf_index(float x);
        float y_plot_coord_2_tf_value(float y);

        // find widget
        Widget formatTypeToWidget(TFFormat format);
        Widget channelTypeToWidget(TFManager::Channel channel, TFFormat format);
        Widget interpolationTypeToWidget(TFChannel::Interpolation type);

        // set the motif state associated with the current interpolation type
        void setCurrentInterpolationType(TFChannel::Interpolation type);
        void setCurrentFormat(TFFormat format);

        // file selection dialog boxes
        void fileMenuSave(Widget widget);
        void fileMenuLoad(Widget widget);

        //////////////////////////////////////////////////////////////
        // callbacks
        //////////////////////////////////////////////////////////////

        void filePulldown(Widget widget, XtPointer clientData,
            XtPointer callData);
        void setTFMode(Widget widget, XtPointer clientData, 
            XtPointer callData);
        void setTFFormat(Widget widget, XtPointer clientData,
            XtPointer callData);
        void useBuiltInLUT(Widget widget, XtPointer clientData,
            XtPointer callData);
        void setCurrentChannel2(Widget widget, XtPointer clientData,
            XtPointer callData);
        void init(Widget widget, XtPointer clientData,
            XtPointer callData);
        void expose(Widget widget, XtPointer clientData,
            XtPointer callData);
        void input(Widget widget, XtPointer clientData,
            XtPointer callData);
        void resize(Widget widget, XtPointer clientData,
            XtPointer callData);
        void scaleWindow(Widget widget, XtPointer clientData,
            XtPointer callData); 
        void scaleLevel(Widget widget, XtPointer clientData,
            XtPointer callData);

        static void quitCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void filePulldownCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void loadLUTCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void saveLUTCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void setTFModeCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void setTFFormatCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void useBuiltInCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void setCurrentChannelCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void initCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void exposeCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void inputCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void resizeCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);
        static void scaleWindowCallback(Widget widget, XtPointer clientData, 
            XtPointer callData); 
        static void scaleLevelCallback(Widget widget, XtPointer clientData, 
            XtPointer callData);

};

//////////////////////////////////////////////////////////////////////
// TFEditor.h
//////////////////////////////////////////////////////////////////////

#endif // TF_EDITOR_H
