// Copyright  2002 Silicon Graphics, Inc.  All Rights Reserved.
// 
// The recipient ("Recipient") of this software, including as modified
// ("Software") may reproduce, redistribute, use, and derive works from the
// Software without restriction, subject to the following conditions:
//
//  Redistribution of the Software in any form must reproduce this entire
// notice, including as modified in accordance with these provisions
// ("Notice");
//
//  Any Recipient who modifies and subsequently redistributes the Software
// shall add information to this Notice to sufficiently identify the
// Recipient's modifications;
//
//  Recipient may not use the name(s) of any previous Recipient to endorse
// or promote any products derived from the Software without prior express
// written permission from such previous Recipient.
//
// THE SOFTWARE IS PROVIDED "AS IS," WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTY OR CONDITION, INCLUDING WITHOUT LIMITATION ANY WARRANTIES OR
// CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
// PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  PATENT LICENSES, IF ANY,
// PROVIDED HEREIN DO NOT APPLY TO COMBINATIONS OF THIS PROGRAM WITH OTHER
// SOFTWARE, OR ANY OTHER PRODUCT WHATSOEVER.  IN NO EVENT WILL THE
// ORIGINATOR OR SUBSEQUENT RECIPIENT OF THE SOFTWARE BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES ARISING IN RELATION TO THE SOFTWARE, ITS USE, OR THESE
// PROVISIONS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR NEGLIGENCE OR OTHER TORT, EVEN IF
// PRE-ADVISED OF THE PROSPECT OF SUCH DAMAGES.
//
// This Software was developed at private expense; if acquired under an
// agreement with the USA government or any contractor thereto, it is
// acquired as "commercial computer software" subject to the provisions of
// this license agreement, as specified in (a) 48 CFR 12.212 of the FAR;
// or, if acquired for Department of Defense units, (b) 48 CFR 227-7202 of
// the DoD FAR Supplement; or sections succeeding thereto.
// 
// Originator:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, Mountain
// View, CA  94043.  http://www.sgi.com


//////////////////////////////////////////////////////////////////////
// TFManager.h
//////////////////////////////////////////////////////////////////////

#ifndef TF_MANAGER_H
#define TF_MANAGER_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

// local
#include "TFChannel.h"

//////////////////////////////////////////////////////////////////////
// enums
//////////////////////////////////////////////////////////////////////

enum TFFormat {
    TF_LUMINANCE,
    TF_LUMINANCE_ALPHA,
    TF_RGBA
};

//////////////////////////////////////////////////////////////////////
// class
//////////////////////////////////////////////////////////////////////

class TFManager {
  
    //////////////////////////////////////////////////////////////////
    // public
    //////////////////////////////////////////////////////////////////

    public:

        //////////////////////////////////////////////////////////////
        // constants
        //////////////////////////////////////////////////////////////

        enum Channel {
            CHANNEL_LUMINANCE, 
            CHANNEL_RED, 
            CHANNEL_GREEN, 
            CHANNEL_BLUE, 
            CHANNEL_ALPHA, 

            CHANNEL_COUNT
        };

	enum Function {
	    GRAY_SCALE,
	    HOT_METAL,
	    SPECTRAL,
	    CONTOUR,
	    
	    BUILTIN_COUNT
	};
	
        //////////////////////////////////////////////////////////////
        // factories
        //////////////////////////////////////////////////////////////

        static TFManager* create(TFFormat format, int length);
        static TFManager* createFromStream(istream& stream);

        //////////////////////////////////////////////////////////////
        // destructor
        //////////////////////////////////////////////////////////////

        ~TFManager();
    
        //////////////////////////////////////////////////////////////
        // attributes
        //////////////////////////////////////////////////////////////

        void setFormat(TFFormat format);
        TFFormat getFormat() const;

        void setLength(int length);
        int  getLength() const;

        TFChannel* getChannel(Channel channel);
        int getNumberOfChannels() const;

        float* getData();

        //////////////////////////////////////////////////////////////
        // actions
        //////////////////////////////////////////////////////////////
	
        void saveToStream(ostream &stream) const;
	void createBuiltInLUT(Function function);
	
    //////////////////////////////////////////////////////////////////
    // protected
    //////////////////////////////////////////////////////////////////

    private:

        //////////////////////////////////////////////////////////////
        // variables
        //////////////////////////////////////////////////////////////

        TFFormat
            m_format;

        int
            m_length;

        TFChannel**
            m_channels;

        mutable bool
            m_dirty;

        mutable float*
            m_table;

        //////////////////////////////////////////////////////////////
        // constructor
        //////////////////////////////////////////////////////////////

        TFManager(TFChannel** channels, TFFormat format, int length);
    
        //////////////////////////////////////////////////////////////
        // methods
        //////////////////////////////////////////////////////////////

        static int formatToChannelCount(TFFormat format);
        static const char* formatToString(TFFormat format);
	
	void initializeChannel(Channel channel, int slot, int num, 
			       float *controlPoints);

};

//////////////////////////////////////////////////////////////////////
// TFManager.h - EOF
//////////////////////////////////////////////////////////////////////

#endif // TF_MANAGER_H

