// Copyright  2002 Silicon Graphics, Inc.  All Rights Reserved.
// 
// The recipient ("Recipient") of this software, including as modified
// ("Software") may reproduce, redistribute, use, and derive works from the
// Software without restriction, subject to the following conditions:
//
//  Redistribution of the Software in any form must reproduce this entire
// notice, including as modified in accordance with these provisions
// ("Notice");
//
//  Any Recipient who modifies and subsequently redistributes the Software
// shall add information to this Notice to sufficiently identify the
// Recipient's modifications;
//
//  Recipient may not use the name(s) of any previous Recipient to endorse
// or promote any products derived from the Software without prior express
// written permission from such previous Recipient.
//
// THE SOFTWARE IS PROVIDED "AS IS," WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTY OR CONDITION, INCLUDING WITHOUT LIMITATION ANY WARRANTIES OR
// CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
// PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  PATENT LICENSES, IF ANY,
// PROVIDED HEREIN DO NOT APPLY TO COMBINATIONS OF THIS PROGRAM WITH OTHER
// SOFTWARE, OR ANY OTHER PRODUCT WHATSOEVER.  IN NO EVENT WILL THE
// ORIGINATOR OR SUBSEQUENT RECIPIENT OF THE SOFTWARE BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES ARISING IN RELATION TO THE SOFTWARE, ITS USE, OR THESE
// PROVISIONS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR NEGLIGENCE OR OTHER TORT, EVEN IF
// PRE-ADVISED OF THE PROSPECT OF SUCH DAMAGES.
//
// This Software was developed at private expense; if acquired under an
// agreement with the USA government or any contractor thereto, it is
// acquired as "commercial computer software" subject to the provisions of
// this license agreement, as specified in (a) 48 CFR 12.212 of the FAR;
// or, if acquired for Department of Defense units, (b) 48 CFR 227-7202 of
// the DoD FAR Supplement; or sections succeeding thereto.
// 
// Originator:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, Mountain
// View, CA  94043.  http://www.sgi.com

//////////////////////////////////////////////////////////////////////
// pfVolume.h
//////////////////////////////////////////////////////////////////////

#ifndef __PF_VOLUME_H
#define __PF_VOLUME_H

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include <Performer/pf.h>
#include <Performer/pr.h>
#include <Performer/pf/pfChannel.h>
#include <Performer/pf/pfPipeWindow.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoState.h>
#include <Performer/pr/pfGeoSetCB.h>
#include <Performer/pf/pfTraverser.h>

// C++
//#include <fstream.h>

// Volumizer2
#include <Volumizer2/Version.h>
#include <Volumizer2/Error.h>
#include <Volumizer2/Shape.h>
#include <Volumizer2/Block.h>
#include <Volumizer2/ShapeSet.h>
#include <Volumizer2/Appearance.h>
#include <Volumizer2/ParameterVolumeTexture.h>
#include <Volumizer2/ParameterLookupTable.h>
#include <Volumizer2/TMLUTShader.h>
#include <Volumizer2/TMSimpleShader.h>
#include <Volumizer2/ClipRenderAction.h>

//////////////////////////////////////////////////////////////////////
// class pfVolume -
// A Performer subclass for Volumes, using Volumizer 2
//////////////////////////////////////////////////////////////////////

class pfVolume : public pfGeode {
  
    public :

       //////////////////////////////////////////////////////////////////
       // public
       //////////////////////////////////////////////////////////////////

      // Constructor and destructor
      pfVolume( void );
      ~pfVolume( void );
      
      //////////////////////////////////////////////////////////////
      // get methods
      //////////////////////////////////////////////////////////////
      
      // get the shape set
      vzShapeSet *getShapeSet() { return _shape_set; }
      
      // get the shape... returns the zero'th shape by default
      vzShape *getShape() { 
	
	// check to see if we have atleast one shape
	if(_shape_set && _shape_set->getNumShapes() > 0)
	  return _shape_set->get(0);
	
	return NULL;
      }
      
      // get the class type
      static  pfType* getClassType( void ) { return pfvzClassType; }
      pfType  *getType( void ) { return pfvzClassType; }
      
      //////////////////////////////////////////////////////////////
      // set methods
      //////////////////////////////////////////////////////////////
      
      // set the sampling rate
      void setSamplingRate(float rate[3]) {
	for(int i = 0; i < 3; i++)
	  _sampling_rate[i] = rate[i];
      }

      //////////////////////////////////////////////////////////////
      // actions
      //////////////////////////////////////////////////////////////
      int     loadFile( const char *_filename );
      static  void init( void );
      int     isExactType( pfType *t ) { return t == pfvzClassType; }
      
  private :
	
      //////////////////////////////////////////////////////////////////
      // private
      //////////////////////////////////////////////////////////////////
      vzShapeSet           *_shape_set;
    
      vzShape              *_shape;
      vzGeometry           *_geometry;
      vzAppearance         *_appearance;
  
      // The geo state shared by all the geo sets..
      // requests transparency... 
      pfGeoState    *_geo_state;
      
      // pfGeoSetCB's.. one for each pfPipeWindow
      pfGeoSetCB   **_geoset_cbs;
      pfPipeWindow **_windows;
      int            _num_windows;
      
      char           _filename [256];
      int            _initialized;
      float          _sampling_rate[3];
      
      // static members
      static pfType *pfvzClassType;
      static int     pfvzRenderActionSlot;
      static int     pfvzShapeSlot;
      
      // actions invoked by the callbacks
      void           preapp( pfPipeWindow *pw );
      void           predraw( pfPipeWindow *pw );
      void           draw( pfPipeWindow *pw );
      
      // initialization routines
      void           initialize();
      void           initializeRenderer( pfPipeWindow *pw);
      void           initializeGeoSetCB(pfPipeWindow *pipe);
      vzParameterLookupTable *initializeLookupTable(int width);
      vzParameterLookupTable *loadLookupTable(char *filename);
      int            findPipeWindow(pfPipeWindow *pw);
      
      //////////////////////////////////////////////////////////////////
      // callbacks 
      //////////////////////////////////////////////////////////////////
      
      static void    drawCB (pfGeoSet *gset, void *data);
      static int     predrawCB (pfTraverser * trav, void *data);
      static int     preappCB (pfTraverser * trav, void *data);
      static void    exceptionHandler(vzErrorSeverity severity, vzErrorType type,
				      const char *format, va_list args, void* data);
      static void cleanupLookupTable(vzObject *object, void *data);
    
      
};

//////////////////////////////////////////////////////////////////////
//  Class derived from vzClipRenderAction to override the
//  new and delete operators. This is needed since the user
//  data passed to the pfPipeWindow needs to be allocated
//  from shared memory.
//////////////////////////////////////////////////////////////////////

struct pfvzRenderAction : public vzClipRenderAction {
    
  // Constructor. Invokes the vzClipRenderAction constructor
  pfvzRenderAction() : vzClipRenderAction() {}
    
  // Virtual constructor.
  virtual ~pfvzRenderAction() {}
  
  // MACRO to declare the new and delete operators.
  PFSTRUCT_DECLARE;
};

#endif
