//////////////////////////////////////////////////////////////////////////
//
// measurement1D DSO - select 2 points and measure the distance between
// them
//
//////////////////////////////////////////////////////////////////////////

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <Performer/pf/pfText.h>
#include <Performer/pr/pfString.h>
#include <Performer/pr/pfFont.h>

//
// Indicates if the user is in the process of interacting with the box.
//
int interactionIsInProgress = 0;

#define MIN_BUTTONS 3
#define PI (3.14159265358979f)
#define defineMode (1)
#define editMode (2)
#define deleteMode (3)
#define resetMode (4)
#define saveMode (5)
#define restoreMode (6)
#define selectMode (7)
#define unselectMode (8)

class measurement1D : public dtkAugment {

public:

  // Here are the methods needed by dtkDSO_loader
  measurement1D (dpf *app) ;
  int preConfig (void);
  int postConfig (void);
  int preFrame (void);
  int postFrame (void);
  ~measurement1D ();

private:

  dpf* app;
  dpfDisplay *display;

  // we get the wand position directly from dtk
  dtkInLocator *wand;

  dtkInButton *button;
  dtkDequeuer *dequeuer;
  dtkRecord_event *e;
  dtkRecord_event *lastEvent ;
  int whichButton;
  int buttonState;
  bool button1On; // indicates if button 1 is being pressed
  bool button1LastFrame; // indicates old position of button
  bool button2On; // indicates if button 1 is being pressed
  bool button2LastFrame; // indicates old position of button
  //  Shared memory segment to put in measurement1D mode
  dtkSharedMem* shm_measure;
  //  Shared memory segment used to request the wand pointer
  dtkSharedMem* shm2;
  unsigned char mmode;
  pfDCS *boxdcs;
  pfDCS *arrowdcs;
  float wandPos[6];
  pfMatrix wandWorld;
  pfCoord wandCoord;
  pfVec3 arrowcheck;
  pfVec3 check;
  pfVec3 upVector;
  float xs, ys, zs;
  pfNode* arrownode;
  float arrow_angle;
  pfNode* clonenode;
  int clonecount;
  pfNode* blueboxnode;
  pfNode* redboxnode;
  float pt1[3];
  float pt2[3];
  float pt3[3];
  int which;
  pfMatrix coordMatrix;
  pfCoord syscoord;
  float wandData[6];
  dtkMatrix *matrix;
  int count, oldcount;
  float distance;
  char dist[81];
  char ptx[20];
  char pty[20];
  char ptz[20];
  char ptall[60];
  pfNode* unitLine;
  pfNode** newLine;
  int numLines;
  int numArrows;
  pfCoord lineCoord;
  void makeLine();
  float findHeading();
  float findPitch();
  float theta;
  double ratio;
  float scale;
  char* line;
  pfText* txt;
  pfText* positionText;
  pfString *posString;
  pfFont *helveticaBoldOblique ;
  pfFont *helveticaNarrow; 
  pfString *str; 
  pfString *newString;
  pfGroup* createTextBox(char* itSays);
  pfString* createNewString(char* itSays);
  pfDCS* label_dcs ;
  char* whatTextSays;
  pfGroup *textnode;
  pfGroup *positionnode;
  pfDCS *positiondcs;
  pfMatrix mat;
  int waitcnt;
  int framecnt;
  const pfBox *box;
  pfString* strng;
  unsigned char turnit;
  int arrowOn;
  FILE *distFile;
  FILE *restoreFile;
  string savefilename;
  string restorefilename;
  string commentstring;
  char* nameOfArrow;
  char* nameOfBox;
  char *env;
  char *env2;
  char *env3;
  // keep arrays of which arrows are with which measurement1D lines
  int* arrow1;
  int* arrow2;
  // keep an array of the arrow, line, distance marker dcss
  pfDCS** arrowdcsList;
  pfDCS** distdcsList;
  pfDCS** linedcsList;
  // need to save the endpoints so can recreate the measurements
  float **savepts;
  int i,ii;
  char *restoreline;
  int pargc;
  char **pargv;
  void makeArrow();
  void makeArrowFromPoints(float* pt);
  void makeDistanceText();
  bool overAnArrow();
  bool gotAnArrow;
  int selectedArrow;
  int otherArrow;
  int arr1,arr2;
  bool blueboxOn;
  int nummeasure;
  int arrow1or2;
  void findSelectedMeasure();
  bool moveArrow;
  bool firstPress, secondPress;
} ;

// Constructor
//
// Create object and set description.  
// Be sure to validate it!
//
measurement1D::measurement1D (dpf *app_in) :
  dtkAugment ("measurement1D")
{
  setDescription (
    "measurement1D DSO: allows the user to measure distance between points in space.\n");
  //printf("constructor point to point\n");
  app = app_in;

  // All of the rest of this really gets set in preConfig and preFrame

  display = NULL;
  wand = NULL;
  boxdcs = NULL;
  arrowdcs = NULL;
  arrowdcsList = new pfDCS*[100];
  distdcsList = new pfDCS*[100];
  linedcsList = new pfDCS*[100];
  newLine = new pfNode*[100];
  arrow1 = new int[101];
  arrow2 = new int[101];
  savepts = new float*[3];
  savepts[0] = new float[200];
  savepts[1] = new float[200];
  savepts[2] = new float[200];
  numLines = 0;
  numArrows = 0;
  button = NULL;
  dequeuer = NULL;
  clonecount = 0;
  waitcnt = 0;
  framecnt = 0;
  arrowOn = 0;
  button1On = false;
  button2On = false;
  distFile = NULL;
  restoreFile = NULL;
  restoreline = new char[250];
  selectedArrow = -1;
  otherArrow = -1;
  gotAnArrow = false;
  blueboxOn = false;
  moveArrow = false;
  firstPress = false;
  secondPress =false;
  commentstring = "# This is an output file generated by the measurement1D DSO\n# Each line contains xyz for 2 points (6 numbers)\n";

  // define shared memory for measurement1D mode
  shm_measure = new dtkSharedMem(sizeof(unsigned char), "measureMode");
  if(!shm_measure || shm_measure->isInvalid())
    shm_measure = NULL ;
  else
    shm_measure->flush();
  
  mmode = 0;
  if (shm_measure)
    shm_measure->write(&mmode);

  shm2 = new dtkSharedMem(sizeof(unsigned char), "pointerOn");
  if(!shm2 || shm2->isInvalid()) {
      shm2 = NULL;
  }
  else
    shm2->flush();

  // Be sure to validate it!
  validate ();
} // end of constructor


measurement1D::~measurement1D ()
{
  if (distFile != NULL)
    fclose(distFile);
  if (restoreFile != NULL)
    fclose(restoreFile);
}  // end of destructor

//
// preConfig is called only once by dpf, after pfInit() and 
//  dpfDisplay:preConfig() but before pfConfig()
//
int measurement1D::preConfig (void) {

  return dtkAugment::CONTINUE ; 
}   // end of preConfig


//
// postConfig is called after pfConfig
//
int measurement1D::postConfig (void) {
  //printf("postConfig point to point\n");
  //
  // Get access to wand data stream
  //
  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand || wand->isInvalid())
    {
    wand = NULL ;
    dtkMsg.add(DTKMSG_ERROR,
                 "measurement1D::postConfig() failed:"
                 "Unable to get valid wand pointer.\n");
    return dtkAugment::ERROR_;
    }

  // open the output file to store the points and distances
  
  // see if user specified an output file name
  char* fmarker = getenv("Measurement1DOutputFile");
  if (fmarker)
    savefilename = fmarker;
  else
    savefilename = "save_measurements";
   
  // see if user specified a restore file name
  fmarker = getenv("Measurement1DRestoreFile");
  if (fmarker)
    restorefilename = fmarker;
  else
    restorefilename = "restore_measurements";

  return dtkAugment::REMOVE_CALLBACK ;
}  // end of postConfig

int measurement1D::preFrame (void)
{
  //printf("preFrame point to point\n");
  display = app->display ();
  if (!display) {
    dtkMsg.add(DTKMSG_ERROR,
	       "measurement1D::preFrame() failed:"
	       "Unable to get display pointer.\n");
    return dtkAugment::ERROR_;
  }

  button = (dtkInButton *) app->get("buttons", DTKINBUTTON_TYPE);
  if(!button) return DTK_CALLBACK_ERROR;

   if(button->number() < MIN_BUTTONS)
     {
       dtkMsg.add(DTKMSG_ERROR,
		  "buttonNavControl::preConfig() failed:"
		  " the dtkInButton named \"%s\" has "
		  "just %d buttons"
		  " which is less than the %d needed.\n",
		  button->getName(), button->number(),
		  MIN_BUTTONS);
      return DTK_CALLBACK_ERROR;
    }
  button->queue();

  dequeuer = new dtkDequeuer(app->record());
  if(!dequeuer || dequeuer->isInvalid()) return DTK_CALLBACK_ERROR;

  helveticaBoldOblique = 
    pfdLoadFont_type1("Helvetica-BoldOblique", PFDFONT_EXTRUDED);
  helveticaNarrow = 
    pfdLoadFont_type1("Helvetica-Narrow", PFDFONT_EXTRUDED);

  if (helveticaBoldOblique && helveticaNarrow) {
    line = "Text Box";
    textnode = createTextBox(line);

    // create a text box for the wand position
    positionnode = new pfGroup();
    positionText = new pfText();
    positionnode->addChild(positionText);
    posString = createNewString("test");
    positionText->addString(posString);
  }

  newString = createNewString("replacement string");
  positionText->replaceString(posString,newString);
  positiondcs = new pfDCS();
  positiondcs->addChild(positionnode);
  positiondcs->setTrans(0.0, 0.0, 0.0);

  // load in the arrow and put it on the wand dcs
  env = getenv("arrowObjectName");

  if (env) {
    nameOfArrow = strdup(env) ;
  }
  else {
    nameOfArrow = "arrow.savg";
  }

  arrownode = pfdLoadFile(nameOfArrow);

  env2 = getenv("arrowangle");
  arrow_angle = 0.0;
  if (env2)
    arrow_angle = atof(env2);
  //printf("print the arrow angle: %f\n",arrow_angle);
  
  // fix direction for top of arrow, no angle = (0, .3, 0)
  arrow_angle = arrow_angle * PI/180;
  xs = -0.3 * sin(arrow_angle);
  ys =  0.3 * cos(arrow_angle);
  zs =  0.0;
  //printf("new xs,ys,zs: %f %f %f\n",xs,ys,zs);

  unitLine = pfdLoadFile("unitLine.savg");

  env = NULL;
  env = getenv("blueObjectName");
  if (env)
    nameOfArrow = strdup(env);
  else
    nameOfArrow = "blueBox.savg";
  blueboxnode = pfdLoadFile(nameOfArrow);

  env = NULL;
  env = getenv("redObjectName");
  if (env)
    nameOfArrow = strdup(env);
  else
    nameOfArrow = "redBox.savg";
  redboxnode = pfdLoadFile(nameOfArrow);

  // define a dcs in front of the wand for the arrow and box
  arrowdcs = new pfDCS();      
  display->world()->addChild(arrowdcs);
  boxdcs = new pfDCS();      
  display->world()->addChild(boxdcs);

  return dtkAugment::REMOVE_CALLBACK;
}    // end of preFrame


//
// We call postFrame every frame to get wand position and adjust the box
//
int measurement1D::postFrame (void) {
  //printf("postframe: measurement1D  ");

  if (shm_measure) {
    shm_measure->read(&mmode);
  }

  if (shm2) {
    shm2->read(&turnit);
    // if measure mode is on, turn the wandPointer off
    if (mmode == defineMode) {
      turnit = 2;
      shm2->write(&turnit);
    }
  }

  // first read the button (whichbutton) on the mouse/wand
  e = dequeuer->getNextEvent(button);

  dtkRecord_event *lastEvent ;
  
  // if we have data, toss all but the last one
  if ((oldcount+1) != count && e != NULL) {
    while (e) {
      lastEvent = e ;
      e = dequeuer->getNextEvent(button);
    }
    e = lastEvent ;
  }
  
  buttonState = button->read(&whichButton, e);

  //--------------------------------------------------
  // button1On; indicates if button 1 is being pressed
  // button1LastFrame; indicates old position of button 1
  //--------------------------------------------------
    
  // read the wand position
  if (wand)
    wand->read (wandData);
  wandWorld.makeEuler(wandData[3],wandData[4],wandData[5]);
  
  // define the wand data in world coordinates
  wandCoord.xyz.set(wandData[0], wandData[1], wandData[2]);
  wandCoord.hpr.set(wandData[3], wandData[4], wandData[5]);
  display->dpfToWorld(&wandCoord);
  wandWorld.makeEuler(wandCoord.hpr[0],wandCoord.hpr[1],wandCoord.hpr[2]);

  upVector.set(0.f, 0.5f, 0.f);
  check.xformVec(upVector, wandWorld);

  upVector.set(xs, ys, zs);
  arrowcheck.xformVec(upVector, wandWorld);
  
  wandPos[0] = wandData[0];
  wandPos[1] = wandData[1];
  wandPos[2] = wandData[2];
  wandPos[3] = wandData[3];
  wandPos[4] = wandData[4];
  wandPos[5] = wandData[5];
  
  // update the arrowdcs
  if (arrownode != NULL) {
    syscoord.xyz[0] = wandCoord.xyz[0] + check[0];
    syscoord.xyz[1] = wandCoord.xyz[1] + check[1];
    syscoord.xyz[2] = wandCoord.xyz[2] + check[2];
    syscoord.hpr[0] = wandCoord.hpr[0];
    syscoord.hpr[1] = wandCoord.hpr[1];
    syscoord.hpr[2] = wandCoord.hpr[2];
    coordMatrix.makeCoord(&syscoord) ;
    arrowdcs->setMat(coordMatrix);
  }

  // see if you need to put the arrow on or take it off 
  // (and the position textbox)
  if (arrowOn == 1 && mmode == 0) {
    arrowdcs->removeChild(arrownode);
    //display->scene()->removeChild(positiondcs);
    arrowdcs->removeChild(positiondcs);
    arrowOn = 0;
  }
  if (arrowOn == 0 && mmode == 1) {
    arrowdcs->addChild(arrownode);
    //display->scene()->addChild(positiondcs);
    arrowdcs->addChild(positiondcs);
    arrowOn = 1;
  }

  if (whichButton == 1) {
    // first see if button 1 is currently pressed
    if (button->read(1,e)) {
      if (!button1LastFrame) {
	// switch button1ON: if off, turn it on; if on, turn it off
	if (button1On) {
	  button1On = false;
	}
	else {
	  button1On = true;
	}
      }
      button1LastFrame = true;
    }
    else {
      button1LastFrame = false;
    }
  }

  if (whichButton == 2) {
    // first see if button 1 is currently pressed
    if (button->read(2,e)) {
      if (!button2LastFrame) {
	// switch button2ON: if off, turn it on; if on, turn it off
	if (button2On) {
	  button2On = false;
	}
	else {
	  button2On = true;
	}
      }
      button2LastFrame = true;
    }
    else {
      button2LastFrame = false;
    }
  }

  // move the boxdcs if in select mode
  if (mmode == selectMode)
    boxdcs->setMat(coordMatrix);

  if (mmode == defineMode) {
    // put up position of arrow in the corner
    // replace every 20 frames
    if (framecnt > 20) {
      pt3[0] = wandCoord.xyz[0] + check[0] + arrowcheck[0];
      pt3[1] = wandCoord.xyz[1] + check[1] + arrowcheck[1];
      pt3[2] = wandCoord.xyz[2] + check[2] + arrowcheck[2];
      sprintf(ptx, "%f", pt3[0]);
      sprintf(pty, "%f", pt3[1]);
      sprintf(ptz, "%f", pt3[2]);
      strcpy(ptall,ptx);
      strcat(ptall, ",");
      strcat(ptall,pty);
      strcat(ptall, ",");
      strcat(ptall,ptz);

      newString->setString(ptall);
      positionText->replaceString(posString,newString);
      framecnt = 0;
    }
    else
      framecnt = framecnt + 1;

    // mmode needs to be set for anything to happen in postFrame
    // wait a few frame after it is set to pick anything
    waitcnt = waitcnt + 1;

    // if in measurement1D mode, turn off wandPointer
    turnit = 2;
    if (shm2)
      shm2->write(&turnit);

    if (button2On && waitcnt > 50) {
      
      makeArrow();
      button2On = false;
      
      // store the point at the end of the new arrow
      if (clonecount == 0) {
	pt1[0] = wandCoord.xyz[0] + check[0] + arrowcheck[0];
	pt1[1] = wandCoord.xyz[1] + check[1] + arrowcheck[1];
	pt1[2] = wandCoord.xyz[2] + check[2] + arrowcheck[2];
	savepts[0][numArrows-1] = pt1[0];
	savepts[1][numArrows-1] = pt1[1];
	savepts[2][numArrows-1] = pt1[2];
	clonecount += 1;
      }
      else {
	pt2[0] = wandCoord.xyz[0] + check[0] + arrowcheck[0];
	pt2[1] = wandCoord.xyz[1] + check[1] + arrowcheck[1];
	pt2[2] = wandCoord.xyz[2] + check[2] + arrowcheck[2];
	savepts[0][numArrows-1] = pt2[0];
	savepts[1][numArrows-1] = pt2[1];
	savepts[2][numArrows-1] = pt2[2];
	clonecount = 0;
	
	makeDistanceText();

	// put a thin line between the ends of the arrows
	makeLine();

	// turn off measure mode
	mmode = 0;
	waitcnt = -1;
	if (shm_measure)
	  shm_measure->write(&mmode);

	if (numLines >= 100) {
	  // increase array sizes
	  newLine = (pfNode **)realloc(newLine, sizeof(pfNode*)*numLines);
	  distdcsList = (pfDCS **)realloc(distdcsList, sizeof(pfDCS*)*numLines);
	  linedcsList = (pfDCS **)realloc(linedcsList, sizeof(pfDCS*)*numLines);
	  arrowdcsList = (pfDCS **) realloc(arrowdcsList,sizeof(pfDCS*)*numLines);
	  arrow1 = (int *) realloc(arrow1, sizeof(int)*(numLines+1));
	  arrow2 = (int *) realloc(arrow2, sizeof(int)*(numLines+1));
	  savepts[0] = (float *) realloc(savepts[0], sizeof(float)*2*numLines);
	  savepts[1] = (float *) realloc(savepts[1], sizeof(float)*2*numLines);
	  savepts[2] = (float *) realloc(savepts[2], sizeof(float)*2*numLines);
	}
      }
    }
  }

  else if (mmode == editMode) {
    //printf("edit mode\n");

    if (selectedArrow > -1) {
      //printf("got and arrow: %d\n",selectedArrow);

      if (button2On || moveArrow) {
	moveArrow = true;
	// update the arrowdcs, linedcs, and distance dcs
	syscoord.xyz[0] = wandCoord.xyz[0] + check[0];
	syscoord.xyz[1] = wandCoord.xyz[1] + check[1];
	syscoord.xyz[2] = wandCoord.xyz[2] + check[2];
	syscoord.hpr[0] = wandCoord.hpr[0];
	syscoord.hpr[1] = wandCoord.hpr[1];
	syscoord.hpr[2] = wandCoord.hpr[2];
	coordMatrix.makeCoord(&syscoord);
	arrowdcsList[selectedArrow]->setMat(coordMatrix);
      }

      if (button2On) {
	if (firstPress == false) {
	  firstPress = true;
	  button2On = false;
	}
	else {
	  secondPress = true;
	  button2On = false;
	}
      }

      // do stuff to the arrowdcs
      if (firstPress && !secondPress) {
	boxdcs->setMat(coordMatrix);
	
	// update savepts array
	savepts[0][selectedArrow] = wandCoord.xyz[0] + check[0] + arrowcheck[0];
	savepts[1][selectedArrow] = wandCoord.xyz[1] + check[1] + arrowcheck[1];
	savepts[2][selectedArrow] = wandCoord.xyz[2] + check[2] + arrowcheck[2];
	// update the measurement - resets nummeasure
	findSelectedMeasure();
	//printf("found the measure: %d and arrow: %d\n",nummeasure,arrow1or2);

	// update the distance text
	pt1[0] = savepts[0][arrow1[nummeasure]];
	pt1[1] = savepts[1][arrow1[nummeasure]];
	pt1[2] = savepts[2][arrow1[nummeasure]];
	pt2[0] = savepts[0][arrow2[nummeasure]];
	pt2[1] = savepts[1][arrow2[nummeasure]];
	pt2[2] = savepts[2][arrow2[nummeasure]];
	distance = sqrt((pt1[0]-pt2[0])*(pt1[0]-pt2[0]) +
			(pt1[1]-pt2[1])*(pt1[1]-pt2[1]) +
			(pt1[2]-pt2[2])*(pt1[2]-pt2[2]));
	sprintf(dist, "%f", distance);

	// replace the distance text and move it
	syscoord.xyz[0] = pt2[0] - .1;
	syscoord.xyz[1] = pt2[1] - .3;
	syscoord.xyz[2] = pt2[2];
	syscoord.hpr[0] = 0.0;
	syscoord.hpr[1] = 0.0;
	syscoord.hpr[2] = 0.0;
	coordMatrix.makeCoord(&syscoord) ;
	distdcsList[nummeasure]->setMat(coordMatrix);

	// redefine the dcs matrix based on the 2 points from input
	lineCoord.xyz[0] = pt1[0];
	lineCoord.xyz[1] = pt1[1];
	lineCoord.xyz[2] = pt1[2];

	// define the hpr
	// move the points so that p1 is at the origin
	pt2[0] = pt2[0] - pt1[0];
	pt2[1] = pt2[1] - pt1[1];
	pt2[2] = pt2[2] - pt1[2];
	
	// pitch first
	lineCoord.hpr[1] = findPitch();
	lineCoord.hpr[0] = findHeading();
	lineCoord.hpr[2] = 0.0;
	scale = sqrt(pt2[0]*pt2[0] + pt2[1]*pt2[1] + pt2[2]*pt2[2]);
	linedcsList[nummeasure]->setCoord(&lineCoord);
	linedcsList[nummeasure]->setScale(0.2, 0.2, scale);
      }
      else if (secondPress) {
	// update the distance text with the last distance
	newString = createNewString(dist);
	// find pfText and replace its string
	for (i=0; i<distdcsList[nummeasure]->getNumChildren(); ++i) {
	  if (distdcsList[nummeasure]->getChild(i)->isOfType(pfGroup::getClassType())) {
	    positionnode = (pfGroup*)distdcsList[nummeasure]->getChild(i);

	    // now find the pfText hung on the pfGroup
	    for (ii=0; ii<positionnode->getNumChildren(); ++ii) {
	      if (positionnode->getChild(i)->isOfType(pfText::getClassType())) {
		positionText = (pfText*)positionnode->getChild(i);
		posString = positionText->getString(0);
		positionText->replaceString(posString,newString);
		break;
	      }
	    }
	  }
	}
	selectedArrow = -1;
	moveArrow = false;
	otherArrow = -1;
	button2On = false;
	firstPress = false;
	secondPress = false;
	if (blueboxOn) {
	  boxdcs->removeChild(blueboxnode);
	  boxdcs->removeChild(redboxnode);
	  blueboxOn = false;
	  mmode = 0;
	  if (shm_measure) {
	    shm_measure->write(&mmode);
	  }
	}
      }
    }
  }

  else if (mmode == deleteMode) {
    //printf("delete mode\n");

    // delete the selected measurement1D or the last one if none selected
    if (selectedArrow > -1) {
      // find the meaurement line number and the other arrow number
      findSelectedMeasure();

      if (nummeasure > -1) {
	// delete the line and distance marker
	display->world()->removeChild(linedcsList[nummeasure]);
	display->world()->removeChild(distdcsList[nummeasure]);
	for (i = nummeasure; i < numLines-1; ++i ) {
	  linedcsList[i] = linedcsList[i+1];
	  distdcsList[i] = distdcsList[i+1];
	}
	numLines = numLines - 1;

	// delete its arrows
	display->world()->removeChild(arrowdcsList[selectedArrow]);
	display->world()->removeChild(arrowdcsList[otherArrow]);

	if (selectedArrow < otherArrow) {
	  for (i = selectedArrow; i < numArrows-2; ++i ) {
	    arrowdcsList[i] = arrowdcsList[i+2];
	  }
	}
	else {
	  for (i = otherArrow; i < numArrows-2; ++i ) {
	    arrowdcsList[i] = arrowdcsList[i+2];
	  }
	}
	numArrows = numArrows - 2;
      }

      if (blueboxOn) {
	boxdcs->removeChild(blueboxnode);
	boxdcs->removeChild(redboxnode);
	blueboxOn = false;
	mmode = 0;
	if (shm_measure) {
	  shm_measure->write(&mmode);
	}
      }
    }
    else if (numLines > 0) {
      // take off the last line
      display->world()->removeChild(linedcsList[numLines-1]);

      // take off the arrows connected to it
      if (arrow1[numLines-1] > -1) {
	display->world()->removeChild(arrowdcsList[arrow1[numLines-1]]);
	numArrows = numArrows - 1;
      }
      if (arrow2[numLines-1] > -1) {
	display->world()->removeChild(arrowdcsList[arrow2[numLines-1]]);
	numArrows = numArrows - 1;
      }

      // take off the distance text
      display->world()->removeChild(distdcsList[numLines-1]);
      numLines = numLines - 1;
    }

    mmode = 0;
    if (shm_measure) {
      shm_measure->write(&mmode);
    }
  }

  else if (mmode == resetMode) {
    //printf("reset mode\n");

    // reset all lines and text boxes
    for (i = 0; i < numLines; ++i) {
      display->world()->removeChild(linedcsList[i]);
      display->world()->removeChild(distdcsList[i]);
      arrow1[i] = -1;
      arrow2[i] = -1;
    }

    // reset all arrows
    for (i = 0; i < numArrows; ++i) {
      display->world()->removeChild(arrowdcsList[i]);
    }
    
    numLines = 0;
    numArrows = 0;

    mmode = 0;
    if (shm_measure) {
      shm_measure->write(&mmode);
    }
  }

  else if (mmode == saveMode) {
    //printf("save mode\n");
 
    distFile = fopen(savefilename.c_str(), "w");
    if (!distFile) {
      pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
	       "  measurement1D: unable to open %s for writing", savefilename.c_str());
      distFile = NULL;
      return DTK_CALLBACK_ERROR;
    }
    //printf("name of savg file: %s\n",savefilename.c_str());

    // start the savg file out with a comment line
    fprintf(distFile, commentstring.c_str());
    // save the current pts to a file
    for (i = 0; i < numLines; ++i) {
      // find arrow numbers and print xyzs for each arrow
      arr1 = arrow1[i];
      arr2 = arrow2[i];
      fprintf(distFile, "%f %f %f %f %f %f\n",savepts[0][arr1],savepts[1][arr1],savepts[2][arr1],savepts[0][arr2],savepts[1][arr2],savepts[2][arr2]);
    }

    mmode = 0;
    if (shm_measure) {
      shm_measure->write(&mmode);
    }
  }

  else if (mmode == restoreMode) {
    //printf("restore mode\n");

    // open the appropriate file
     
    restoreFile = fopen(restorefilename.c_str(), "r");
    if (!restoreFile) {
      pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
	       "  measurement1D: unable to open %s for reading", restorefilename.c_str());
      restoreFile = NULL;
      return DTK_CALLBACK_ERROR;
    }

    // read sets of points, create arrows and lines
    while ((fgets(restoreline,250,restoreFile)!=NULL)) {
      //printf("got the next line: %s\n", restoreline);
      dpf_charsToArgcArgv(restoreline, &pargc, &pargv) ;

      // check for blank or comment
      if (restoreline == NULL || strlen(restoreline) == 0 || restoreline[0] == '#') {
	// do nothing
      }
      else if (pargc > 5) {
	pt1[0] = atof(pargv[0]);
	pt1[1] = atof(pargv[1]); 
	pt1[2] = atof(pargv[2]);
	pt2[0] = atof(pargv[3]);
	pt2[1] = atof(pargv[4]);
	pt2[2] = atof(pargv[5]);

	// make the arrows
	makeArrowFromPoints(pt1);
	savepts[0][numArrows-1] = pt1[0];
	savepts[1][numArrows-1] = pt1[1];
	savepts[2][numArrows-1] = pt1[2];
	makeArrowFromPoints(pt2);
	savepts[0][numArrows-1] = pt2[0];
	savepts[1][numArrows-1] = pt2[1];
	savepts[2][numArrows-1] = pt2[2];
	
	// make the text
	makeDistanceText();

	// make the line
	makeLine();
      }
    }

    mmode = 0;
    if (shm_measure) {
      shm_measure->write(&mmode);
    }
  }

  else if (mmode == selectMode) {
    //printf("select an arrow mode\n");

    // add the selection box onto the wand
    if (!blueboxOn){
      boxdcs->addChild(blueboxnode);
      blueboxOn = true;
    }

    // see if the blue box is over an arrow
    if (button2On) {
      if (overAnArrow()) {
	boxdcs->addChild(redboxnode);
	button2On = false;
	mmode = 0;
	if (shm_measure) {
	  shm_measure->write(&mmode);
	}
      }
    }
  }
 
  else if (mmode == unselectMode) {
    //printf("unselect a selected arrow\n");

    // add the selection box onto the wand
    if (blueboxOn){
      boxdcs->removeChild(blueboxnode);
      boxdcs->removeChild(redboxnode);
      blueboxOn = false;
    }

    selectedArrow = -1;
    mmode = 0;
    if (shm_measure) {
      shm_measure->write(&mmode);
    }
  }
   
  return dtkAugment::CONTINUE ; 
}    // end of postFrame

pfGroup* measurement1D::createTextBox(char* itSays) {
  textnode = new pfGroup ;
  txt= new pfText() ;
  textnode->addChild(txt);

  strng = new pfString();
  strng->setFont(helveticaBoldOblique);
  strng->setMode(PFSTR_JUSTIFY, PFSTR_CENTER);
  strng->setColor(0.0f, .3f, 1.0f, 1.0f);
  strng->setString(itSays);
  box = strng->getBBox() ;
  scale = 0.2f/(box->max[0]-box->min[0]) ;
  mat.makeScale(scale, scale, scale) ;
  mat.postTrans(mat, 0.1f, 0.31f, 0.01f);
  strng->setMat(mat);
  txt->addString(strng);
  return textnode;
}

pfString* measurement1D::createNewString(char* itSays) {
  strng = new pfString();
  strng->setFont(helveticaBoldOblique);
  strng->setMode(PFSTR_JUSTIFY, PFSTR_CENTER);
  strng->setColor(0.0f, .3f, 1.0f, 1.0f);
  strng->setString(itSays);
  box = strng->getBBox() ;
  scale = 0.3f/(box->max[0]-box->min[0]) ;
  mat.makeScale(scale, scale, scale) ;
  mat.postTrans(mat, 0.f, 0.5f, 0.f);
  strng->setMat(mat);
  return strng;
}

void measurement1D::makeLine() {
  // find hpr values and make a transformation matrix
  // copy the unit line again
  newLine[numLines] = unitLine->clone(0);
  linedcsList[numLines] = new pfDCS();
  display->world()->addChild(linedcsList[numLines]);
  linedcsList[numLines]->addChild(newLine[numLines]);

  // define the dcs matrix based on the 2 points from input
  lineCoord.xyz[0] = pt1[0];
  lineCoord.xyz[1] = pt1[1];
  lineCoord.xyz[2] = pt1[2];

  // define the hpr
  // move the points so that p1 is at the origin
  pt2[0] = pt2[0] - pt1[0];
  pt2[1] = pt2[1] - pt1[1];
  pt2[2] = pt2[2] - pt1[2];

  // pitch first
  lineCoord.hpr[1] = findPitch();
  lineCoord.hpr[0] = findHeading();
  lineCoord.hpr[2] = 0.0;
  scale = sqrt(pt2[0]*pt2[0] + pt2[1]*pt2[1] + pt2[2]*pt2[2]);
  //printf("new lineCoord: %f %f %f %f %f %f\n",lineCoord.xyz[0],lineCoord.xyz[1],lineCoord.xyz[2],lineCoord.hpr[0],lineCoord.hpr[1],lineCoord.hpr[2]);
  linedcsList[numLines]->setCoord(&lineCoord);
  
  // set the scale to extend the unit line, but keep the x,y
  // dimensions the same
  // load the line on the dcs and the dcs in the scene
  linedcsList[numLines]->setScale(0.2, 0.2, scale);

  // keep track of which arrows go with this line
  arrow1[numLines] = numArrows - 2;
  arrow2[numLines] = numArrows - 1;
  //printf("set arrow 1 and 2 to %d %d\n",numArrows - 2,numArrows - 1);
  ++numLines;
  arrow1[numLines] = - 1;
  arrow2[numLines] = - 1;
}

float measurement1D::findPitch() {
  if (abs(pt2[2]) < 1.0e-6) {
    theta = 90.0;
  }
  else {
    ratio = sqrt(pt2[0]*pt2[0] + pt2[1]*pt2[1])/pt2[2];
    theta = (180.0/PI)*atan(ratio);
    if (pt2[2] < 0.0)
      theta = 180.0 - abs(theta);
  }

  return theta;
}

float measurement1D::findHeading() {
  // if the z value is zero: return 90
  if (abs(pt2[1]) < 1.0e-6) {
    // either +90 or -90
    theta = 90.0;
    if (pt2[0] < 0.0)
      theta = -90.0;
  }
  else {
    ratio = (pt2[0]/pt2[1]);
    theta = (180.0/PI)*atan(ratio);
    if (pt2[1] > 0.0)
      theta = 180.0 - theta;
    else
      theta = -theta;
  }

  return theta;
}

int isBlankorCommentLine(char *pline) {
  if(pline == NULL || strlen(pline) == 0 || pline[0] == '#')
    return 1;
  return 0;
}

void measurement1D::makeArrow() {
  // clone the arrow
  clonenode = arrownode->clone(0);
  arrowdcsList[numArrows] = new pfDCS();
  syscoord.xyz[0] = wandCoord.xyz[0] + check[0];
  syscoord.xyz[1] = wandCoord.xyz[1] + check[1];
  syscoord.xyz[2] = wandCoord.xyz[2] + check[2];
  syscoord.hpr[0] = wandCoord.hpr[0];
  syscoord.hpr[1] = wandCoord.hpr[1];
  syscoord.hpr[2] = wandCoord.hpr[2];
  coordMatrix.makeCoord(&syscoord);
  arrowdcsList[numArrows]->setMat(coordMatrix);
  display->world()->addChild(arrowdcsList[numArrows]);
  arrowdcsList[numArrows]->addChild(clonenode);
  ++numArrows;
}

void measurement1D::makeArrowFromPoints(float* pt) {
  // clone the arrow
  clonenode = arrownode->clone(0);
  arrowdcsList[numArrows] = new pfDCS();
  syscoord.xyz[0] = pt[0] - xs;
  syscoord.xyz[1] = pt[1] - ys;
  syscoord.xyz[2] = pt[2] - zs;
  syscoord.hpr[0] = 0.0;
  syscoord.hpr[1] = 0.0;
  syscoord.hpr[2] = 0.0;
  coordMatrix.makeCoord(&syscoord);
  arrowdcsList[numArrows]->setMat(coordMatrix);
  display->world()->addChild(arrowdcsList[numArrows]);
  arrowdcsList[numArrows]->addChild(clonenode);
  ++numArrows;
}

void measurement1D::makeDistanceText() {	
  // compute the distance
  distance = sqrt((pt1[0]-pt2[0])*(pt1[0]-pt2[0]) +
		  (pt1[1]-pt2[1])*(pt1[1]-pt2[1]) +
		  (pt1[2]-pt2[2])*(pt1[2]-pt2[2]));
  //printf("new distance: %f\n",distance);
  sprintf(dist, "%f", distance);
	
  // create a new text string to hold this distance
  textnode = createTextBox(dist);
  distdcsList[numLines] = new pfDCS();
  // position of text box is: 0.1, 0.31, 0.01
  syscoord.xyz[0] = pt2[0] - .1;
  syscoord.xyz[1] = pt2[1] - .3;
  syscoord.xyz[2] = pt2[2];
  syscoord.hpr[0] = 0.0;
  syscoord.hpr[1] = 0.0;
  syscoord.hpr[2] = 0.0;
  coordMatrix.makeCoord(&syscoord) ;
  distdcsList[numLines]->setMat(coordMatrix);
  display->world()->addChild(distdcsList[numLines]);
  distdcsList[numLines]->addChild(textnode);
}

bool measurement1D::overAnArrow() {
  gotAnArrow = false;
  
  // get current position of the blue box and store in pt1
  pt1[0] = wandCoord.xyz[0] + check[0] + arrowcheck[0];
  pt1[1] = wandCoord.xyz[1] + check[1] + arrowcheck[1];
  pt1[2] = wandCoord.xyz[2] + check[2] + arrowcheck[2];

  for (i = 0; i < numArrows; ++i) {
    // get the point
    pt2[0] = savepts[0][i];
    pt2[1] = savepts[1][i];
    pt2[2] = savepts[2][i];

    // find the distance to the blue box
    distance = sqrt((pt1[0]-pt2[0])*(pt1[0]-pt2[0]) +
		  (pt1[1]-pt2[1])*(pt1[1]-pt2[1]) +
		  (pt1[2]-pt2[2])*(pt1[2]-pt2[2]));

    if (distance < .05) {
      //printf("found the correct arrow\n");
      gotAnArrow = true;
      selectedArrow = i;
      break;
    }
  }

  return gotAnArrow;
}

void measurement1D::findSelectedMeasure() {
  // set nummeasure to the correct line number
  nummeasure = -1;
  arrow1or2 = 0;
  for (i = 0; i < numLines; ++i) {
    if (arrow1[i] == selectedArrow) {
      nummeasure = i;
      arrow1or2 = 1;
      break;
    }
    if (arrow2[i] == selectedArrow) {
      nummeasure = i;
      arrow1or2 = 2;
      break;
    }
  }

  //printf("selected arrow is: %d nummeasure: %d\n",selectedArrow,nummeasure);
  // if found, find the other arrow
  //printf("arrow1 and 2 of nummeasure: %d %d\n",arrow1[nummeasure],arrow2[nummeasure]); 
  if (arrow1[nummeasure] == selectedArrow)
    otherArrow = arrow2[nummeasure];
  else
    otherArrow = arrow1[nummeasure];
  //printf("other arrow: %d\n",otherArrow);
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new measurement1D(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
