// a modification of savgDisplay- a 4x3 aspect ratio, so will look better as a
// movie- kelso, 1/05

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

#define WIDTH (640)
#define HEIGHT (480)

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  dpfDisplay *display = app->display();
  if(!display) return DTKDSO_LOAD_ERROR;

  // This is set up for the NIST RAVE, where one DPF unit = 42"
  display->meters(1.0688);

  // one pipe
  if((display->pipes(1))) return DTKDSO_LOAD_ERROR;

  dpfPipe *pipe = display->first() ;
  
  pipe->pwins(1);
  dpfPwin *pwin = pipe->first();

  pwin->setOriginSize(300, 200, WIDTH, HEIGHT) ;
  pwin->name("movieDisplay") ;

  // one screen in the one window
  pwin->screens(1);

  dpfScreen *screen = pwin->first();
  screen->simpleFrustum(1);

  // modify which axis causes the screen to scale- PFFRUST_CALC_VERT is default
  screen->aspect(PFFRUST_CALC_HORIZ) ;

  // 90 degree field of view so the corners of CAVE sim cube is lined
  // up with the viewing frusta edges at the initial view position.
  screen->fov(90.0f);

  return DTKDSO_LOAD_UNLOAD;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  return DTKDSO_UNLOAD_CONTINUE;
}



