// sets the pivot to the default if the current navigation's name is in the
// colon-seperated list of navigation names specified by the environment
// variable $NAV_NAME_PIVOT_MOD.  If this variable isn't set, a default list
// is used.

#include <sys/types.h>
#include <stdlib.h>
#include <dtk.h>
#include <dtk/dtkDSO_loader.h>

#include <string>
using namespace std ;

#define DEFPIVOTNAV "desktopFlyNav:desktopHoverNav:wandJoystickNav:wandJoystickPitchNav:wandJoystickRollNav"

class navDefaultPivotMod : public dtkAugment {
public:
  navDefaultPivotMod(dtkManager *);
  int postFrame(void);
  
private:
  dtkNavList *navList ;
  dtkManager *mgr ;
  string defNavNames ;
  dtkNav *oldNav ;
  dtkCoord *thisPivot ;
};


navDefaultPivotMod::navDefaultPivotMod(dtkManager *m) :
  dtkAugment("navDefaultPivotMod") {
  mgr = m ;
  navList = NULL ;
  oldNav = NULL ;
  setDescription("sets the pivot to the default if the current navigation's "
		 "name is in the colon-seperated list of navigation names "
		 "specified by the environment variable $NAV_NAME_PIVOT_MOD.  "
		 "If this variable isn't set, a default list is used.") ;
 
  if (!getenv("NAV_NAME_PIVOT_MOD"))
    defNavNames = DEFPIVOTNAV ;
  else
    defNavNames = getenv("NAV_NAME_PIVOT_MOD") ;
  
  dtkMsg.add(DTKMSG_INFO, "navDefaultPivotMod: default pivot will be used for the following navigations: %s\n",defNavNames.c_str()) ;

  // add colons at beginning and end of list to make search easier
  defNavNames.insert(0,":") ;
  defNavNames += ":" ;

  // dtkAugment::dtkAugment() will not validate the object
  validate() ;
}

int navDefaultPivotMod::postFrame(void) {
  // this stuff with "n" is so all the DSOs get a chance to run their
  // postFrame before we start keeping track of the pivot point
  static int n = 0 ;
  n++ ;
  if (n<3) 
     return CONTINUE ;
  n=3 ;

  // now that things are stable, let's go!

  // geta pointer to the navlist- only need to do this once
  if (!navList) {
    navList = (dtkNavList *) mgr->check("dtkNavList", DTKNAVLIST_TYPE) ;
    if (!navList) {
      navList = new dtkNavList ;
      if (mgr->add(navList)) {
	delete navList ;
	navList = NULL ;
	return ERROR_ ;
      }
    }
  }
  
  dtkNav *thisNav = navList->current() ;

  // has the navigation changed since last time?  if so, get pivot pointers
  if (thisNav != oldNav) {
    oldNav = thisNav ;

    string thisNavName = thisNav->getName() ;
    //printf("current navigation is %s\n",thisNavName.c_str()) ;

    // get the current pivot- may or may not be default
    dtkCoord *curPivot = thisNav->pivot() ;
    // set pivot to default
    thisNav->pivot(NULL) ;
    // get pivot back out
    dtkCoord *defPivot = thisNav->pivot() ;

    thisNavName.insert(0,":") ;
    thisNavName += ":" ;
    if (defNavNames.find(thisNavName) == string::npos) {
      dtkMsg.add(DTKMSG_INFO, 
		 "navDefaultPivotMod: %s using current pivot\n", 
		 thisNavName.c_str()) ;
      thisPivot = curPivot ;
    } else {
      dtkMsg.add(DTKMSG_INFO, 
		 "navDefaultPivotMod: %s using default pivot\n", 
		 thisNavName.c_str()) ;
      thisPivot = curPivot ;
      thisPivot = defPivot ;
    }

  }

  thisNav->pivot(thisPivot) ;

  return CONTINUE ;
}



/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  return new navDefaultPivotMod(manager);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
