// loads the gnomon at the position/orientation of the navigation's pivot

#include <sys/types.h>
#include <stdlib.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

#define SHM "SHM_NAV_WORLD_PIVOT"

class navPivotGnomon : public dtkAugment {
public:
  navPivotGnomon(dpf *);
  int postFrame(void);
  
private:
  
  dpf *app ;
  dpfDisplay *display ;
  pfDCS *dcs ;
  dtkNavList *navList ;
};


navPivotGnomon::navPivotGnomon(dpf *app_in) :
  dtkAugment("navPivotGnomon") {
  app = app_in ;
  dcs = NULL ;
  display = app->display() ;
  navList = NULL ;
  setDescription("place a gnomon at the navigation's pivot") ;

  dcs = NULL ;

  // dtkAugment::dtkAugment() will not validate the object
  validate() ;
}

int navPivotGnomon::postFrame(void) {
  // this should get done just once- first pass
  if (!dcs) {
    dcs = new pfDCS() ;
    display->scene()->addChild(dcs) ;
    display->addGhostNode(dcs) ;
    dcs->addChild(pfdLoadFile("gnomon.pfb")) ;
  }

  // same for this
  if (!navList) {
    navList = (dtkNavList *) app->check("dtkNavList", DTKNAVLIST_TYPE) ;
    if (!navList) {
      navList = new dtkNavList ;
      if (app->add(navList)) {
	delete navList ;
	navList = NULL ;
	return ERROR_ ;
      }
    }
  }

  pfCoord foo = dpf_dtkCoord_to_pfCoord(navList->current()->pivot()) ;
  dcs->setCoord(&foo) ;

  return CONTINUE ;
}



/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new navPivotGnomon(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
