// noClipWorldNode - a copy of the world node on which objects will
// not be effected by plane clipping
//
// Authors:
//     Adele Peskin
//      Scientific Applications & Visualization Group
//           SAVG web page: http://math.nist.gov/mcsd/savg/
//      Mathematical & Computational Science Division
//           MCSD web page: http://math.nist.gov/mcsd/
//      National Institute of Standards and Technology
//           NIST web page: http://www.nist.gov/
//
//
// For information about this DSO, 
// contact Adele Peskin, peskin@boulder.nist.gov
//
//
// This code is a modification to the example generic dtkAugment DSO provided
// with the DIVERSE distribution. The following disclaimer applies to the NIST
// modifications.
// 
// "This software was developed at the National Institute of Standards
// and Technology by employees of the Federal Government in the course
// of their official duties. Pursuant to title 17 Section 105 of the
// United States Code this software is not subject to copyright
// protection and is in the public domain.  noClipWorldNode is an experimental 
// system.  NIST assumes no responsibility whatsoever for its use by 
// other parties, and makes no guarantees, expressed or implied, about 
// its quality, reliability, or any other characteristic.
// 
// We would appreciate acknowledgement if the software is used."

#include <stdio.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <Performer/pf.h>
#include <Performer/pf/pfSCS.h>

// example noClipWorldNode dtkAugment DSO
class noClipWorldNode : public dtkAugment {
public:
  noClipWorldNode(dpf *app) ;
  ~noClipWorldNode(void) ;
  int preFrame(void);
  int postConfig(void);
  int postFrame(void);
  
private:
  dpf* app;
  dpfDisplay *display;
  pfDCS* nav;
  pfDCS* noClipWorld;
  pfMatrix matether;
} ;

#define NUMBER_ALLOWED 5

// create object and set description.
// be sure to validate it!
noClipWorldNode::noClipWorldNode(dpf *app_in) :
  dtkAugment("noClipWorldNode")
{
  setDescription(" creates a DCS that acts like the world node, "
		 " except that nodes under it remain visible "
		 " with plane clipping. "
		 " This version was compiled on 09/23/04.") ;
  app = app_in;
  validate();
} ;

// destructor
noClipWorldNode::~noClipWorldNode(void) {
}

#define MIN_BUTTONS 3

int noClipWorldNode::preFrame(void) {
  display = app->display();

  if(!display) return DTK_CALLBACK_ERROR;

  // just call buttonNavControl::preFrame() once at the start.
  return DTK_REMOVE_CALLBACK;
}

// called only once by dpf, after pfConfig() and dpfDisplay:postConfig()
// pfConfig() 
int noClipWorldNode::postConfig(void) {
  // first get the display and make and set the group node
  display = app->display();
  noClipWorld = new pfDCS();
  noClipWorld->setName("noClip");
  display->nav()->addChild(noClipWorld);

  return DTK_REMOVE_CALLBACK ;
}

// called every display loop, as a pfFrame callback, after pfDraw()
// this return value causes it to be called once, and the object removed
int noClipWorldNode::postFrame(void) {

  // update the nav matrix - replicate the dpfToWorld routine
  display->ether()->getMat(matether);
  noClipWorld->setMat(matether);
  return DTK_CONTINUE ;
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new noClipWorldNode(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
