/*
 * 
 *       Scientific Applications & Visualization Group
 *            SAVG web page: http://math.nist.gov/mcsd/savg/
 *       Mathematical & Computational Science Division
 *            MCSD web page: http://math.nist.gov/mcsd/
 *       National Institute of Standards and Technology
 *            NIST web page: http://www.nist.gov/
 * 
 *  
 *  This software was developed at the National Institute of Standards
 *  and Technology by employees of the Federal Government in the course
 *  of their official duties. Pursuant to title 17 Section 105 of the
 *  United States Code this software is not subject to copyright
 *  protection and is in the public domain.  isolate is an experimental 
 *  system.  NIST assumes no responsibility whatsoever for its use by 
 *  other parties, and makes no guarantees, expressed or implied, about 
 *  its quality, reliability, or any other characteristic.
 *  
 *  We would appreciate acknowledgement if the software is used.
 * 
 */


/************************************************
 * savg-ruler   			        *
 * Created by: Adele Peskin     		*
 * Created on: 5-18-05  			*
 * Current Version: 1.00			*
 *						*
 * Description:	                                *
 *    savg-ruler generates a 10 x 1 unit ruler  *
 * by default. The user can say how many units  *
 * are to be on the ruler, and how many tick    *
 * marks.                                       *
 ************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* Prints information on usage*/
void usage();

int main(argc, argv)
int argc;
char *argv[];
{
 // COMMAND LINE PROCESSING

  int i=1;
  int units = 10;
  double dmark = 0.1;

  while(i<argc) {
    if(strcmp(argv[i],"-u")==0) {
      if (argc > (i+1)) {
	units = atoi(argv[++i]);
	i+=2;
      }
      else
	usage();
    }
    else
      usage();
  }

  // BODY

  // front face
  printf("polygons\n");
  printf(" 0 0  0 1 1 0 1 0 1 0\n");
  printf("%d 0  0 1 1 0 1 0 1 0\n",units);
  printf("%d 0  1 1 1 0 1 0 1 0\n",units);
  printf(" 0 0  1 1 1 0 1 0 1 0\n");

  // make big tick marks
  for (i = 0; i <= units; ++i) {
    printf("lines\n");
    printf("%d -0.21   0 0 0 0 1\n",i);
    printf("%d -0.21 0.3 0 0 0 1\n",i);
    printf("lines\n");
    printf("%d  0.01   0 0 0 0 1\n",i);
    printf("%d  0.01 0.3 0 0 0 1\n",i);
  }

  // make litle tick marks
  dmark = 0.0;
  for (i = 0; i <= units*10; ++i) {
    printf("lines\n");
    printf("%f -0.21    0 0 0 0 1\n",dmark);
    printf("%f -0.21 0.15 0 0 0 1\n",dmark);
    printf("lines\n");
    printf("%f  0.01    0 0 0 0 1\n",dmark);
    printf("%f  0.01 0.15 0 0 0 1\n",dmark);
    dmark += 0.1;
  }

  // back face
  printf("polygons\n");
  printf("%d -0.2  0 1 1 0 1 0 -1 0\n",units);
  printf(" 0 -0.2  0 1 1 0 1 0 -1 0\n");
  printf(" 0 -0.2  1 1 1 0 1 0 -1 0\n");
  printf("%d -0.2  1 1 1 0 1 0 -1 0\n",units);

  // top face
  printf("polygons\n");
  printf(" 0    0  1 1 1 0 1 0  0 1\n");
  printf("%d    0  1 1 1 0 1 0  0 1\n",units);
  printf("%d -0.2  1 1 1 0 1 0  0 1\n",units);
  printf(" 0 -0.2  1 1 1 0 1 0  0 1\n");

  // bottom face
  printf("polygons\n");
  printf(" 0    0  0 1 1 0 1 0  0 -1\n");
  printf("%d    0  0 1 1 0 1 0  0 -1\n",units);
  printf("%d -0.2  0 1 1 0 1 0  0 -1\n",units);
  printf(" 0 -0.2  0 1 1 0 1 0  0 -1\n");

  // +x face
  printf("polygons\n");
  printf("%d    0  0 1 1 0 1 1 0 0\n",units);
  printf("%d    0  1 1 1 0 1 1 0 0\n",units);
  printf("%d -0.2  1 1 1 0 1 1 0 0\n",units);
  printf("%d -0.2  0 1 1 0 1 1 0 0\n",units);

  // -x face
  printf("polygons\n");
  printf(" 0    0  1 1 1 0 1 -1 0 0\n");
  printf(" 0    0  0 1 1 0 1 -1 0 0\n");
  printf(" 0 -0.2  0 1 1 0 1 -1 0 0\n");
  printf(" 0 -0.2  1 1 1 0 1 -1 0 0\n");

  // numbers on one side

  // 1
  printf("lines\n");
  printf("1 -0.21 0.32 0 0 0 1\n");
  printf("1 -0.21 0.5  0 0 0 1\n");
  printf("lines\n");
  printf("0.97 -0.21 0.32 0 0 0 1\n");
  printf("1.03 -0.21 0.32 0 0 0 1\n");
  printf("lines\n");
  printf(".97 -0.21 0.48 0 0 0 1\n");
  printf("1   -0.21 0.5  0 0 0 1\n");

  // 2
  printf("lines\n");
  printf("2.03 -0.21 0.32 0 0 0 1\n");
  printf("1.97 -0.21 0.32 0 0 0 1\n");
  printf("2.03 -0.21 0.42 0 0 0 1\n");
  printf("2.03 -0.21 0.45 0 0 0 1\n");
  printf("2.0  -0.21 0.5  0 0 0 1\n");
  printf("1.97 -0.21 0.45 0 0 0 1\n");
  printf("1.97 -0.21 0.42 0 0 0 1\n");

  // 3
  printf("lines\n");
  printf("2.97 -0.21 0.32 0 0 0 1\n");
  printf("3.03 -0.21 0.32 0 0 0 1\n");
  printf("3.03 -0.21 0.4  0 0 0 1\n");
  printf("3.   -0.21 0.4  0 0 0 1\n");
  printf("3.03 -0.21 0.5  0 0 0 1\n");
  printf("2.97 -0.21 0.5  0 0 0 1\n"); 

  // 4
  printf("lines\n");
  printf("4   -0.21 0.32 0 0 0 1\n");
  printf("4   -0.21 0.45 0 0 0 1\n");
  printf("lines\n");
  printf("4.03 -0.21 0.4 0 0 0 1\n");
  printf("3.95 -0.21 0.4 0 0 0 1\n");
  printf("3.97 -0.21 0.5 0 0 0 1\n");

  // 5
  printf("lines\n");
  printf("4.97 -0.21 0.32 0 0 0 1\n");
  printf("5.03 -0.21 0.32 0 0 0 1\n");
  printf("5.03 -0.21 0.44 0 0 0 1\n");
  printf("4.98 -0.21 0.44 0 0 0 1\n");
  printf("4.98 -0.21 0.5  0 0 0 1\n");
  printf("5.03 -0.21 0.5  0 0 0 1\n");

  // 6
  printf("lines\n");
  printf("5.97 -0.21 0.5  0 0 0 1\n");
  printf("5.97 -0.21 0.32 0 0 0 1\n");
  printf("6.03 -0.21 0.32 0 0 0 1\n");
  printf("6.03 -0.21 0.4  0 0 0 1\n");
  printf("5.97 -0.21 0.4  0 0 0 1\n");

  // 7
  printf("lines\n");
  printf("7.   -0.21 0.32 0 0 0 1\n");
  printf("7.03 -0.21 0.5  0 0 0 1\n");
  printf("6.97 -0.21 0.5  0 0 0 1\n");

  // 8
  printf("lines\n");
  printf("7.97 -0.21 0.32 0 0 0 1\n");
  printf("8.03 -0.21 0.32 0 0 0 1\n");
  printf("8.03 -0.21 0.42 0 0 0 1\n");
  printf("7.97 -0.21 0.42 0 0 0 1\n");
  printf("7.97 -0.21 0.32 0 0 0 1\n");
  printf("lines\n");
  printf("8.02 -0.21 0.42 0 0 0 1\n");
  printf("8.02 -0.21 0.5  0 0 0 1\n");
  printf("7.98 -0.21 0.5  0 0 0 1\n");
  printf("7.98 -0.21 0.42 0 0 0 1\n");

  // 9
  printf("lines\n");
  printf("9.01 -0.21 0.32 0 0 0 1\n");
  printf("9.01 -0.21 0.5  0 0 0 1\n");
  printf("8.97 -0.21 0.5  0 0 0 1\n");
  printf("8.97 -0.21 0.42 0 0 0 1\n");
  printf("9.01 -0.21 0.42 0 0 0 1\n");


  // numbers on the other side

  // 1
  printf("lines\n");
  printf("9  0.01 0.32 0 0 0 1\n");
  printf("9  0.01 0.5  0 0 0 1\n");
  printf("lines\n");
  printf("8.97  0.01 0.32 0 0 0 1\n");
  printf("9.03  0.01 0.32 0 0 0 1\n");
  printf("lines\n");
  printf("9.03 0.01 0.48 0 0 0 1\n");
  printf("9    0.01 0.5  0 0 0 1\n");

  // 2
  printf("lines\n");
  printf("7.97  0.01 0.32 0 0 0 1\n");
  printf("8.03  0.01 0.32 0 0 0 1\n");
  printf("7.97  0.01 0.42 0 0 0 1\n");
  printf("7.97  0.01 0.45 0 0 0 1\n");
  printf("8.0   0.01 0.5  0 0 0 1\n");
  printf("8.03  0.01 0.45 0 0 0 1\n");
  printf("8.03  0.01 0.42 0 0 0 1\n");

  // 3
  printf("lines\n");
  printf("7.03  0.01 0.32 0 0 0 1\n");
  printf("6.97  0.01 0.32 0 0 0 1\n");
  printf("6.97  0.01 0.4  0 0 0 1\n");
  printf("7     0.01 0.4  0 0 0 1\n");
  printf("6.97  0.01 0.5  0 0 0 1\n");
  printf("7.03  0.01 0.5  0 0 0 1\n"); 

  // 4
  printf("lines\n");
  printf("6    0.01 0.32 0 0 0 1\n");
  printf("6    0.01 0.45 0 0 0 1\n");
  printf("lines\n");
  printf("5.97  0.01 0.4 0 0 0 1\n");
  printf("6.05  0.01 0.4 0 0 0 1\n");
  printf("6.03  0.01 0.5 0 0 0 1\n");

  // 5
  printf("lines\n");
  printf("5.03  0.01 0.32 0 0 0 1\n");
  printf("4.97  0.01 0.32 0 0 0 1\n");
  printf("4.97  0.01 0.44 0 0 0 1\n");
  printf("5.02  0.01 0.44 0 0 0 1\n");
  printf("5.02  0.01 0.5  0 0 0 1\n");
  printf("4.97  0.01 0.5  0 0 0 1\n");

  // 6
  printf("lines\n");
  printf("4.03  0.01 0.5  0 0 0 1\n");
  printf("4.03  0.01 0.32 0 0 0 1\n");
  printf("3.97  0.01 0.32 0 0 0 1\n");
  printf("3.97  0.01 0.4  0 0 0 1\n");
  printf("4.03  0.01 0.4  0 0 0 1\n");

  // 7
  printf("lines\n");
  printf("3     0.01 0.32 0 0 0 1\n");
  printf("2.97  0.01 0.5  0 0 0 1\n");
  printf("3.03  0.01 0.5  0 0 0 1\n");

  // 8
  printf("lines\n");
  printf("2.03  0.01 0.32 0 0 0 1\n");
  printf("1.97  0.01 0.32 0 0 0 1\n");
  printf("1.97  0.01 0.42 0 0 0 1\n");
  printf("2.03  0.01 0.42 0 0 0 1\n");
  printf("2.03  0.01 0.32 0 0 0 1\n");
  printf("lines\n");
  printf("1.98  0.01 0.42 0 0 0 1\n");
  printf("1.98  0.01 0.5  0 0 0 1\n");
  printf("2.02  0.01 0.5  0 0 0 1\n");
  printf("2.02  0.01 0.42 0 0 0 1\n");

  // 9
  printf("lines\n");
  printf("0.99  0.01 0.32 0 0 0 1\n");
  printf("0.99  0.01 0.5  0 0 0 1\n");
  printf("1.03  0.01 0.5  0 0 0 1\n");
  printf("1.03  0.01 0.42 0 0 0 1\n");
  printf("0.99  0.01 0.42 0 0 0 1\n");
}

/* Prints information on usage*/
void usage()
{
  printf("usage: savg-ruler [-u units]\n");
  exit(1);
}




