/*
 *       Scientific Applications & Visualization Group
 *            SAVG web page: http://math.nist.gov/mcsd/savg/
 *       Mathematical & Computational Science Division
 *            MCSD web page: http://math.nist.gov/mcsd/
 *       National Institute of Standards and Technology
 *            NIST web page: http://www.nist.gov/
 * 
 *  
 *  This software was developed at the National Institute of Standards
 *  and Technology by employees of the Federal Government in the course
 *  of their official duties. Pursuant to title 17 Section 105 of the
 *  United States Code this software is not subject to copyright
 *  protection and is in the public domain.  isolate is an experimental 
 *  system.  NIST assumes no responsibility whatsoever for its use by 
 *  other parties, and makes no guarantees, expressed or implied, about 
 *  its quality, reliability, or any other characteristic.
 *  
 *  We would appreciate acknowledgement if the software is used.
 * 
 */


/************************************************
 * savg-shrinkage         			*
 * Created by: Adele Peskin		        *
 * Created on: 3-24-05				*
 * Current Version: 1.00			*
 *						*
 * Description:					*
 *    savg-shrinkage is used to indicate that   *   
 * each filled polygon or tristrip should be    *
 * shrunk towards its center by scale.  The     *
 * smaller the value of scale, the smaller      *
 * the objects.                                 *
 ************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "feed.h"
#include "savgLineEvaluator.c"

/* Prints information on usage*/
void usage();

int main(argc, argv)
int argc;
char** argv;
{
/*
 * INITIALIZATION
 */
  float shrink = 1.0;
  char line[STRING_LENGTH]; 
  int pargc = 0;
  char **pargv  = NULL;

/*
 * COMMAND LINE PROCESSING
 */
  // there should be exactly 1 parameter, wire or filled
  if (argc != 2) {
    usage();
    exit(EXIT_FAILURE);
  }

  shrink = atof(argv[1]);
/*
 * BODY
 */

  printf("shrinkage %f\n",shrink);
 
  while(getLine(line)) {
    // print every line unless it is another striplength line

    // if it is a comment just print it
    if (containsComment(line))
      printf("%s\n",line);

    else {
      // parse and check each line
      dpf_charsToArgcArgv(line, &pargc, &pargv);
      
      if (pargc > 0) {
	if (strcasecmp("shrinkage",pargv[0])!=0)
	  printf("%s\n",line);
      }
      else
	printf("\n");
    }
  }
   
  return 0;
}

/* Prints information on usage*/
void usage()
{
	fprintf(stderr,
	"ERROR: Incorrect command line arguments\n"
	"\n"
	"USAGE:\n"
	"\tsavg-shrinkage  shrink\n");
}

