/*
 *       Scientific Applications & Visualization Group
 *            SAVG web page: http://math.nist.gov/mcsd/savg/
 *       Mathematical & Computational Science Division
 *            MCSD web page: http://math.nist.gov/mcsd/
 *       National Institute of Standards and Technology
 *            NIST web page: http://www.nist.gov/
 * 
 *  
 *  This software was developed at the National Institute of Standards
 *  and Technology by employees of the Federal Government in the course
 *  of their official duties. Pursuant to title 17 Section 105 of the
 *  United States Code this software is not subject to copyright
 *  protection and is in the public domain.  isolate is an experimental 
 *  system.  NIST assumes no responsibility whatsoever for its use by 
 *  other parties, and makes no guarantees, expressed or implied, about 
 *  its quality, reliability, or any other characteristic.
 *  
 *  We would appreciate acknowledgement if the software is used.
 * 
 */


/************************************************
 * savg-text-style          			*
 * Created by: Adele Peskin		        *
 * Created on: 6-30-05				*
 * Current Version: 1.00			*
 *						*
 * Description:					*
 *    savg-text-style changes the font style    *
 * for all text in a file.                      *
 ************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "feed.h"
#include "savgLineEvaluator.c"

/* Prints information on usage*/
void usage();

int main(argc, argv)
int argc;
char** argv;
{
/*
 * INITIALIZATION
 */
  char line[STRING_LENGTH];
  int style = 0;
  int pargc = 0;
  char **pargv  = NULL;

/*
 * COMMAND LINE PROCESSING
 */
  // make sure there is one argument
  if (argc != 2 ) {
    usage();
    exit(EXIT_FAILURE);
  }

  if (strcasecmp(argv[1],"TEXTURED")==0)
    style = 1;
  else if (strcasecmp(argv[1],"OUTLINE")==0)
    style = 2;
  else if (strcasecmp(argv[1],"MESH")==0)
    style = 3;
  else if (strcasecmp(argv[1],"EXTRUDED")==0)
    style = 4;
  else {
    usage();
    exit(EXIT_FAILURE);
  }

/*
 * BODY
 */

  // print the text style line at the top
  if (style == 1)
    printf("TEXT STYLE TEXTURED\n");
  if (style == 2)
    printf("TEXT STYLE OUTLINE\n");
  if (style == 3)
    printf("TEXT STYLE MESH\n");
  if (style == 4)
    printf("TEXT STYLE EXTRUDED\n");
 
  // remove any other text style lines
  while(getLine(line)) {

    // if it is a comment just print it
    if (containsComment(line))
      printf("%s\n",line);

    else {
      // parse and check each line
      dpf_charsToArgcArgv(line, &pargc, &pargv);

      if (pargc == 1)
	printf("%s\n",line);
      else if (pargc > 1 && strcasecmp("STYLE",pargv[1])!=0)
	  printf("%s\n",line);
      else
	printf("\n");     
    }
  }
  
  return 0;
}

/* Prints information on usage*/
void usage()
{
	fprintf(stderr,
	"ERROR: Incorrect command line arguments\n"
	"\n"
	"USAGE:\n"
	"\tsavg-text-style TEXTURED/OUTLINE/MESH/EXTRUDED\n");
}
