// a simple display for savgfly & NIST G'burg RAVE
// kelso, 12/04

// added envvars for overriding defaults- kelso 3/05
// HEV_SAVG_DISPLAY_FOV sets screen->fov()
// HEV_SAVG_DISPLAY_METERS sets display->meters()
// HEV_SAVG_DISPLAY_ORIGIN_X sets pwin->setOriginSize()
// HEV_SAVG_DISPLAY_ORIGIN_Y sets pwin->setOriginSize()
// HEV_SAVG_DISPLAY_SIZE_X sets pwin->setOriginSize()
// HEV_SAVG_DISPLAY_SIZE_Y sets pwin->setOriginSize()
// HEV_SAVG_DISPLAY_NAME sets pwin->name()
// HEV_SAVG_DISPLAY_ASPECT sets pwin->aspect()- set to vertical if set to "v" or "V"

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  dpfDisplay *display = app->display();
  if(!display) return DTKDSO_LOAD_ERROR;

  float f ;
  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_METERS"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting meters to %f\n",f) ;
    display->meters(f);
  } else
    // This is set up for the NIST RAVE, where one DPF unit = 42"
    display->meters(1.0688);

  // one pipe
  if((display->pipes(1))) return DTKDSO_LOAD_ERROR;

  dpfPipe *pipe = display->first() ;
  
  pipe->pwins(1);
  dpfPwin *pwin = pipe->first();

  int xo, yo, xs, ys ;
  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_ORIGIN_X"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting x origin to %d\n",(int) f) ;
    xo= (int) f ;
  } else
    xo=300 ;

  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_ORIGIN_Y"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting y origin to %d\n",(int) f) ;
    yo= (int) f ;
  } else
    yo=200 ;

  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_SIZE_X"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting x size to %d\n",(int) f) ;
    xs= (int) f ;
  } else
    xs=700 ;

  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_SIZE_Y"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting y size to %d\n",(int) f) ;
    ys= (int) f ;
  } else
    ys=700 ;

  pwin->setOriginSize(xo, yo, xs, ys) ;

  if ((getenv("HEV_SAVG_DISPLAY_NAME"))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting name to \"%s\"\n",getenv("HEV_SAVG_DISPLAY_NAME")) ;
    pwin->name(getenv("HEV_SAVG_DISPLAY_NAME")) ;
  } else
    pwin->name("savgDisplay") ;

  // one screen in the one window
  pwin->screens(1);

  dpfScreen *screen = pwin->first();
  screen->simpleFrustum(1);
  
  if ((getenv("HEV_SAVG_DISPLAY_ASPECT")) && 
      (!strncasecmp(getenv("HEV_SAVG_DISPLAY_ASPECT"), "v", 1))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting aspect to \"vertical\"\n") ;
    screen->aspect(PFFRUST_CALC_VERT) ;
  } else
    // modify which axis causes the screen to scale- PFFRUST_CALC_VERT is default
    screen->aspect(PFFRUST_CALC_HORIZ) ;

  if (!(dpf_string_to_float(getenv("HEV_SAVG_DISPLAY_FOV"),&f))) {
    dtkMsg.add(DTKMSG_INFO,
	       "savgDisplay: setting fov to %f\n",f) ;
    screen->fov(f);
  } else
    screen->fov(60.0f);

  return DTKDSO_LOAD_UNLOAD;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  return DTKDSO_UNLOAD_CONTINUE;
}



