// sgEditor - 
// A DIVERSE DSO which loads and unloads files, and then put them
// in a scenegraph or takes them out of a scenegraph
//
//
// Authors:
//     Adele Peskin
//      Scientific Applications & Visualization Group
//           SAVG web page: http://math.nist.gov/mcsd/savg/
//      Mathematical & Computational Science Division
//           MCSD web page: http://math.nist.gov/mcsd/
//      National Institute of Standards and Technology
//           NIST web page: http://www.nist.gov/
//    
//    Steve Satterfield provided 
//    the idea for the program
//
//
// For information about this DSO, 
// contact Adele Peskin, peskin@boulder.nist.gov
//
//
// This code is a modification to the example generic dtkAugment DSO provided
// with the DIVERSE distribution. The following disclaimer applies to the NIST
// modifications.
// 
// "This software was developed at the National Institute of Standards
// and Technology by employees of the Federal Government in the course
// of their official duties. Pursuant to title 17 Section 105 of the
// United States Code this software is not subject to copyright
// protection and is in the public domain.  xxxxxxx is an experimental 
// system.  NIST assumes no responsibility whatsoever for its use by 
// other parties, and makes no guarantees, expressed or implied, about 
// its quality, reliability, or any other characteristic.
// 
// We would appreciate acknowledgement if the software is used."

#include <stdio.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "testSuffix.C"

class sgEditor : public dtkAugment {
public:
  sgEditor(dpf *app) ;
  ~sgEditor(void) ;
  int postConfig(void);
  int preFrame(void);
  int postFrame(void);
  
private:
  dpf* app;
  dpfDisplay *display;
  pfGroup* rootGroup;
  //  Shared memory segment to put the call
  dtkSharedMem* shm_call;
  dtkSharedMem* shmwire;
  dtkInLocator *wand ;
  dtkInLocator *head ;

  int cnt;
  // keep track of current objectPtrs, filenames as you go
  char* filename;
  char** dcsnames;
  char objectname[50];
  pfNode* node;
  pfNode* node2;
  dpfToggle* tog;
  unsigned char toggleChar;
  unsigned char selected;
  pfGroup* parentgroup;
  pfGroup *newgroup;
  char* file_a;
  char* file_b; 
  char* commndA; 
  char* commndB;
  char parameter[100];
  void setNodeName();
  void addToDisplay();
  void addWanddcs();
  void addHeaddcs();
  void addHeadnohprdcs();
  void addToggleNode();
  float wandData[6];
  float wandPos[6];
  float headData[6];
  float headPos[6];
  float headPos2[6];
  char *line;
  int num;
  int i,ii;
  int len;
  int len2;
  int numevents;
  bool foundNode;
  pfDCS *wanddcs;
  pfDCS *headdcs;
  pfDCS *headnohprdcs;
  pfDCS* newdcs;
  pfDCS* rightDCS;
  pfSCS* newscs;
  pfSCS* rightSCS;
  pfMatrix coordMatrix;
  pfCoord syscoord;
  pfVec3  scale ;
  pfDCS* dcs;
  pfSCS* scs;
  int pcnt;
  pfNode* findNodeWithLookup(char* name);
  bool isit;
  char* nameOfRubberband;
  float params[6];
  pfBox box;
  int pargc ;
  char **pargv ;
  char *env;
  char *world;
  char command[10];
} ;


// create object and set description.
// be sure to validate it!
sgEditor::sgEditor(dpf *app_in) :
  dtkAugment("sgEditor")
{
  setDescription("loads and unloads files specified on a command line."
		 " This version was compiled on 6/10/04.") ;
  app = app_in;
  validate();
  cnt = 0 ;
  filename = NULL;
  wanddcs = NULL;
  headdcs = NULL;
  headnohprdcs = NULL;
  tog = NULL;
  line = new char[1000];
  selected = 2;
  toggleChar = 1;
  pargv = NULL ;
  world = new char[6];
  world = "world";

  // shared memory to hold an input line
  shm_call = new dtkSharedMem(sizeof(char)*1000, "fileChars");
  if(!shm_call || shm_call->isInvalid())
      shm_call = NULL ;
  else
    shm_call->flush();

  shmwire = new dtkSharedMem(sizeof(float)*9, "rubberband");
  if(!shmwire || shmwire->isInvalid()) {
      shmwire = NULL;
  }

  // initialize the wireframe data
  params[0] = 0.0;
  params[1] = 0.0;
  params[2] = 0.0;
  params[3] = 0.0;
  params[4] = 0.0;
  params[5] = 0.0;
  if (shmwire)
    shmwire->write(params);

  wandPos[0] = wandPos[1] = wandPos[2] = wandPos[3] = 
    wandPos[4] = wandPos[5] = 0.0f ;
  headPos[0] = headPos[1] = headPos[2] = headPos[3] = 
    headPos[4] = headPos[5] = 0.0f ;
  headPos2[0] = headPos2[1] = headPos2[2] = headPos2[3] = 
    headPos2[4] = headPos2[5] = 0.0f ;
} ;

// destructor
sgEditor::~sgEditor(void) {
  dtkSharedMem_destroy("fileChars");
}

int sgEditor::postConfig(void) {
  display = app->display();
  rootGroup = new pfGroup;
  display->world()->addChild(rootGroup);

  // if the user is using rubberband, there might be an
  // environment variable for the rubberband object name
  
  // see if the user wants to use a different boundingBox object
  env = getenv("RubberbandName");

  if (env) {
    nameOfRubberband = strdup(env) ;
  }
  else {
    nameOfRubberband = "box.savg";
  }

  char *old_path = (char *) pfGetFilePath();
  char *path = NULL;
  if(old_path)
    {
      // copy old_path from static memory to the heap.
      old_path = dtk_strdup(old_path);
      path = (char *) dtk_malloc(strlen(old_path) +
				 strlen("%s:.:"DPF_DATA_DIR));
      sprintf(path,"%s:.:"DPF_DATA_DIR, old_path);
      pfFilePath(path);
    }
  else // old_path == NULL
    pfFilePath(".:"DPF_DATA_DIR);

  return DTK_REMOVE_CALLBACK ; 
}
int sgEditor::preFrame(void)
{
  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand || wand->isInvalid())
    wand = NULL ;

  head = (dtkInLocator *) app->get("head", DTKINLOCATOR_TYPE);
  if(!head || head->isInvalid())
    head = NULL ;

  return DTK_REMOVE_CALLBACK;
}

// called every display loop, as a pfFrame callback, after pfDraw()
// this return value causes it to only be called once, and the object removed
int sgEditor::postFrame(void) {
  num = 0;
  numevents = 0;
  // read the wand direction 
  if (wand)
      wand->read(wandData);

  // read the headd direction 
  if (head)
      head->read(headData);

  wandPos[0] = wandData[0];
  wandPos[1] = wandData[1];
  wandPos[2] = wandData[2];
  wandPos[3] = wandData[3];
  wandPos[4] = wandData[4];
  wandPos[5] = wandData[5];

  headPos[0] = headData[0];
  headPos[1] = headData[1];
  headPos[2] = headData[2];
  headPos[3] = headData[3];
  headPos[4] = headData[4];
  headPos[5] = headData[5];

  headPos2[0] = headData[0];
  headPos2[1] = headData[1];
  headPos2[2] = headData[2];
  headPos2[3] = 0.0;
  headPos2[4] = 0.0;
  headPos2[5] = 0.0;

  if (shm_call) {
    num = 0;
    while (num < 100 && numevents == 0) {
      numevents = shm_call->qread(line);
      ++num;
      if (numevents != 0) {
	// parse the line if there is an event
	dpf_charsToArgcArgv(line, &pargc, &pargv) ;
	//printf("read the next line in sgEditor: %s\n",line);

	if (pargc >= 2) {
	  strcpy(command,pargv[0]);
	  strcpy(objectname,pargv[1]);
	  if (pargc==3) {
	    strcpy(parameter, pargv[2]);
	  }

	  // load
	  if (strcasecmp("load", pargv[0]) == 0) {
	    //printf("load - read file for- this node in sgEditor\n");
	    // for a load you need a filename
	    if (pargc == 3) {
	      filename = parameter;
	      
	      // see if this is a good file name
	      FILE *fp = fopen(filename, "r");     
	      if (!fp) {
		dtkMsg.add(DTKMSG_ERROR,
			   "sgEditor:: error opening input file %s\n",filename);
		return DTK_CALLBACK_ERROR;
	      }
	      else {
		// read the file
		testSuffix(filename);
		node = pfdLoadFile(filename);
		// name the node
		node->setName(objectname);
	      }
	    }
	  }
	  
	  // unload
	  if (strcasecmp("unload", pargv[0]) == 0) {
	    // find the object in the lookup table
	    node = findNodeWithLookup(objectname);
	    
	    if (node != NULL) {
	      // remove from any parent
	      // find the parent
	      pcnt = node->getNumParents();
	      if (pcnt > 0) {
		// remove node
		parentgroup = node->getParent(0);
		
		if (node != NULL && parentgroup != NULL) 
		  parentgroup->removeChild(node);
	      }
	      pfDelete(node);
	    }
	  }
	  
	  // clone
	  if (strcasecmp("clone", pargv[0]) == 0) {
	    //printf("clone a node that is already loaded\n");
	    if (pargc == 3) {
	      // make sure the requested node is really present
	      node = findNodeWithLookup(objectname);

	      if (node != NULL) {
		// if it is, clone it and rename it
		node2 = node->clone(0);
		node2->setName(parameter);
	      }
	      else {
		dtkMsg.add(DTKMSG_ERROR,
			   "sgEditor:: no node found to clone: %s\n",objectname);
		return DTK_CALLBACK_ERROR;
	      }
	    }
	  }
	  
	  // on
	  else if (strcasecmp("on", pargv[0]) == 0) {
	    //printf("put a node on a named node in sgEditor\n");
	    
	    // see if there is a node by this name
	    node = findNodeWithLookup(objectname);
	    if (node != NULL) {
	      if (pargc == 3)
		setNodeName();
	      else
		strcpy(parameter, world);
	      addToDisplay();
	    }
	    else {
		dtkMsg.add(DTKMSG_ERROR,
			   "sgEditor:: Cannot locate the following node for the ON command: %s\n",objectname);
	    }
	  }
	  
	  // off
	  else if (strcasecmp("off", pargv[0]) == 0) {
	    //printf("turn off this node: %s \n",objectname);
	    node = findNodeWithLookup(objectname);

	    // find the parent
	    pcnt = node->getNumParents();
	    parentgroup = node->getParent(0);
	    
	    if (node != NULL && parentgroup != NULL) 
	      parentgroup->removeChild(node);
	    else {
		dtkMsg.add(DTKMSG_ERROR,
			   "sgEditor:: Cannot locate the parent node for node: %s\n",objectname);
	    }
	  }
	  
	  // group
	  if (strcasecmp("group", pargv[0]) == 0) {
	    // define a new group node
	    node = new pfGroup();
	    
	    // give it a name
	    node->setName(objectname);
	  }
	  
	  // dcs
	  if (strcasecmp("dcs", pargv[0]) == 0 ||
	      strcasecmp("scs", pargv[0]) == 0 ||
	      strcasecmp("updatedcs", pargv[0]) == 0) {
	    //printf("make a dcs or scs: number of parameters: %d\n",pargc);
	    // initialize to identity
	    syscoord.xyz = pfVec3(0.f, 0.f, 0.f) ;
	    syscoord.hpr = pfVec3(0.f, 0.f, 0.f) ;
	    scale = pfVec3(1.f, 1.f, 1.f) ;
	    if (pargc > 4) {
	      // get xyz values
	      syscoord.xyz[0] = atof(pargv[2]);
	      syscoord.xyz[1] = atof(pargv[3]);
	      syscoord.xyz[2] = atof(pargv[4]);
	    }
	    if (pargc > 7) {
	      syscoord.hpr[0] = atof(pargv[5]);
	      syscoord.hpr[1] = atof(pargv[6]);
	      syscoord.hpr[2] = atof(pargv[7]);
	    }
	    if (pargc == 9) {
	      scale[0] = atof(pargv[8]);
	      scale[1] = scale[0];
	      scale[2] = scale[0];
	    }
	    else if (pargc > 10) {
	      // get scaling values
	      scale[0] = atof(pargv[8]);
	      scale[1] = atof(pargv[9]);
	      scale[2] = atof(pargv[10]);
	    }
	    coordMatrix.makeCoord(&syscoord) ;
	    coordMatrix.preScale(scale[0], scale[1], scale[2], coordMatrix) ;
	    if (strcasecmp("dcs", pargv[0]) == 0) {
	      dcs = new pfDCS();
	      dcs->setMat(coordMatrix);
	      dcs->setName(objectname);
	    }
	    else if (strcasecmp("scs", pargv[0]) == 0) {
	      scs = new pfSCS(coordMatrix);
	      scs->setName(objectname);
	    }
	    else if (strcasecmp("updatedcs", pargv[0]) == 0) {
	    // find the dcs by the objectname
	    dcs = NULL;
	    dcs = (pfDCS*)display->world()->lookup(objectname,pfDCS::getClassType());
	    if (dcs != NULL)
	      dcs->setMat(coordMatrix);
	    }
	  }
 
	  // rubberband
	  if (strcasecmp("rubberband", pargv[0]) == 0) {
	    // attach the rubberband object onto the required dcs
	    //printf("put up the rubberband\n");

	    // find the dcs by the objectname
	    dcs = NULL;
	    dcs = (pfDCS*)display->world()->lookup(objectname,pfDCS::getClassType());
	    if (dcs != NULL) {
	      // read in the rubberband object
	      pfNode* wireObject = pfdLoadFile(nameOfRubberband);
	      wireObject->setName("rubberband");

	      // put the rubberband object on the dcs
	      if (wireObject != NULL) {
		dcs->addChild(wireObject);

		// store the rubberband information in shared memory
		if (shmwire) {
		  // find bounding sphere
		  box = dpf_getBBox(wireObject);
		  params[0] = box.min[0];
		  params[1] = box.min[1];
		  params[2] = box.min[2];
		  params[3] = box.max[0];
		  params[4] = box.max[1];
		  params[5] = box.max[2];

		  shmwire->write(params);
		}
	      }
	    }
	  }
	} // pargc >= 2
      } // numevents
    } // while num < 100
  } // if shm_call

  num = 0;
  numevents = 0;
  return DTK_CONTINUE ;
}

pfNode* sgEditor::findNodeWithLookup(char* name) {
  // look in all possible places
  node = NULL;
  node = display->world()->lookup(name,pfNode::getClassType());
  if (node == NULL)
    node = display->world()->lookup(name,pfGroup::getClassType());
  if (node == NULL)
    node = display->world()->lookup(name,pfDCS::getClassType());
  if (node == NULL)
    node = display->world()->lookup(name,pfSCS::getClassType());
  return node;
}

void sgEditor::setNodeName() {
   if (strcasecmp(parameter,"wand")==0) {
     if (wanddcs == NULL) {
       addWanddcs();
     }
   }
   else if (strcasecmp(parameter,"head")==0) {
     if (headdcs == NULL)
       addHeaddcs();
   }
   else if (strcasecmp(parameter,"head_no_hpr")==0) {
     if (headnohprdcs == NULL)
       addHeadnohprdcs();
   }
   else if (strcasecmp(parameter,"hidden_node")==0) {
     if (tog == NULL)
       addToggleNode();
   }
}

void sgEditor::addWanddcs() {
  // first see if there is a wand dcs from another DSO
  wanddcs = (pfDCS*) display->world()->lookup("savg_wand",pfDCS::getClassType());
  if (wanddcs == NULL) {
    // define the dcs
    wanddcs = new dpfDCS(wandPos, /* position */
			 1.0f, /* overall scale */
			 /* file  scale xyz position xyz rotation hpr */
			 NULL, DPF_IDENTITY,
			 NULL);
    
    wanddcs->setName("savg_wand");  
    display->scene()->addChild(wanddcs);
  }
}

void sgEditor::addHeaddcs() {
  // first see if there is a head dcs from another DSO
  headdcs = (pfDCS*) display->world()->lookup("savg_head",pfDCS::getClassType());
  if (headdcs == NULL) {
    // define the head dcs
    headdcs = new dpfDCS(headPos, /* position */
			 1.0f, /* overall scale */
			 /* file  scale xyz position xyz rotation hpr */
			 NULL, DPF_IDENTITY,
			 NULL);
    
    headdcs->setName("savg_head");
    display->scene()->addChild(headdcs);
  }
}

void sgEditor::addHeadnohprdcs() {
  // first see if there is a headnohpr dcs from another DSO
  headnohprdcs = (pfDCS*) display->world()->lookup("savg_headnohpr",pfDCS::getClassType());
  if (headnohprdcs == NULL) {
    // define the head (no hpr) dcs
    headnohprdcs = new dpfDCS(headPos2, /* position */
			      1.0f, /* overall scale */
			      /* file  scale xyz position xyz rotation hpr */
			      NULL, DPF_IDENTITY,
			      NULL);
    
    headnohprdcs->setName("savg_headnohpr"); 
    display->scene()->addChild(headnohprdcs);
  }
}

void sgEditor::addToggleNode() {
  // define the hidden node toggle
  // set up toggle
  // 1 = hide, 2 = show
  toggleChar = 1;
  tog = new dpfToggle(&toggleChar, 1);
  tog->setName("hidden_node");
  display->scene()->addChild(tog);
}

void sgEditor::addToDisplay() {
  if (strcasecmp(parameter,"world")==0)
    display->world()->addChild(node);
  else if (strcasecmp(parameter,"scene")==0)
    display->scene()->addChild(node);
  else if (strcasecmp(parameter,"ether")==0)
    display->ether()->addChild(node);
  else if (strcasecmp(parameter,"nav")==0)
    display->nav()->addChild(node);
  else if (strcasecmp(parameter,"wand")==0)
    wanddcs->addChild(node);
  else if (strcasecmp(parameter,"head")==0)
    headdcs->addChild(node);
  else if (strcasecmp(parameter,"head_no_hpr")==0)
    headnohprdcs->addChild(node);
  else if (strcasecmp(parameter,"hidden_node")==0)
    tog->addChild(node);
  else {
    // find the right dcs or scs or group
    // search for a dcs
    newdcs = NULL;
    newdcs = (pfDCS*)display->world()->lookup(parameter,pfDCS::getClassType());
    if (newdcs == NULL) {
      // search for an scs
      newscs = NULL;
	newscs = (pfSCS*)display->world()->lookup(parameter,pfSCS::getClassType());
      if (newscs == NULL) {
	// search for a group
	newgroup = NULL;
	//printf("looking for a pfGroup in sgEditor\n");
	newgroup = (pfGroup*)display->world()->lookup(parameter,pfGroup::getClassType());
	if (newgroup == NULL) {
		dtkMsg.add(DTKMSG_ERROR,
			   "sgEditor:: Cannot locate the group node: %s\n",parameter);
	}
	else {
	  newgroup->addChild(node);
	}
      }
      else
	newscs->addChild(node);
    }
    else
      newdcs->addChild(node);
  }
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new sgEditor(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
