// uses the wand to set the response- not really a navigation at all!

#define DEAD_ZONE (0.05f)
#define SHM_NAV_NAME "shmNavControl"
#define SHM_TEXT_NAME "shmHeadsUpText" 

#define MAX_RESPONSE (1000.f)
#define MIN_RESPONSE (1.f/MAX_RESPONSE) 

// how long does the text remain visible?
#define TIMEOUT (2.f)

#include <sys/types.h>
#include <stdlib.h>
#include <math.h>
#include <dtk.h>
#include <dtk/dtkDSO_loader.h>

class joystickResponseNav : public dtkNav {
public:
  joystickResponseNav(dtkManager *) ;
  
  int postConfig(void) ;
  int preFrame(void) ;
  int postFrame(void) ;
  
private:
  dtkManager *mgr ;
  dtkSharedMem *shmNav ;
  dtkSharedMem *shmText ;
  dtkTime t;
  bool visible ;
  dtkInValuator *joystick ;
  char *line ;
} ;


joystickResponseNav::joystickResponseNav(dtkManager *m) :
  dtkNav(m, "joystickResponseNav") {
  mgr = m ;
  shmNav = NULL ;
  shmText = NULL ;
  visible = false ;

  // try to load shmNavControl if it is not already loaded
  if (!mgr->check("shmNavControl") && (mgr->load("shmNavControl"))) 
    return ;

  // try to load shmHeadsUpText if it is not already loaded
  if (!mgr->check("shmHeadsUpText") && (mgr->load("shmHeadsUpText"))) 
    return ;

  setDescription("set navigation response using Y value of joystick\n") ;

  // dtkAugment::dtkAugment() will not validate the object
  validate() ;
}

int joystickResponseNav::postConfig(void) {
  joystick = (dtkInValuator *) mgr->get("joystick", DTKINVALUATOR_TYPE) ;

  if(joystick->number() != 2) {
    dtkMsg.add(DTKMSG_ERROR,
	       "joystickResponseNav::postConfig: "
	       "the dtkInValuator named \"joystick\" has %d values,"
	       "2 needed.",joystick->number()) ;
    return ERROR_ ;
  }
  
  return dtkNav::postConfig() ;
}

int joystickResponseNav::preFrame(void) {

  // needed??
  t.reset(0.0,1);

  // presumptuously unload buttonNavControl
  mgr->remove("buttonNavControl") ;
  
  // try to connect to shared memory created by shmNavControl
  shmNav =  new dtkSharedMem(SHM_NAV_NAME) ;
  if (!shmNav)
    dtkMsg.add(DTKMSG_ERROR,
	       "joystickResponseNav::preFrame() failed:"
	       " couldn't open shared memory segment %s\n",SHM_NAV_NAME) ; 

  // try to connect to shared memory created by shmHeadsUpText
  shmText =  new dtkSharedMem(SHM_TEXT_NAME) ;
  if (!shmText)
    dtkMsg.add(DTKMSG_ERROR,
	       "buttonNavSwitch::preFrame() failed:"
	       " couldn't open shared memory segment %s\n",SHM_TEXT_NAME) ; 

  line = (char *) malloc(shmText->getSize()) ;

  return REMOVE_CALLBACK ;
}

int joystickResponseNav::postFrame(void) {

  if ((float)t.get()>TIMEOUT && visible) {
    shmText->write("") ;
    visible = false ;
  }

  float joy_xy[2] ;

  // poll current joystick value
  joystick->read(joy_xy) ;
  
  if (fabs(joy_xy[1])>DEAD_ZONE) {
    
    float newResponse = _response ;

    float dy = (joy_xy[1] > 0.0f) ? 
      ((joy_xy[1] - DEAD_ZONE) / (1.0f - DEAD_ZONE)):
      ((joy_xy[1] + DEAD_ZONE) / (1.0f - DEAD_ZONE)) ;

    // clamp between -.1 and .1
    dy *= .1f ;

    // make it a scale around 1
    dy += 1.f ;

    // flatten out higher values
    dy = sqrt(dy) ;
    dy = sqrt(dy) ;

    newResponse *= dy ;

    if (newResponse<MIN_RESPONSE)
      newResponse = MIN_RESPONSE ;
    if (newResponse>MAX_RESPONSE)
      newResponse = MAX_RESPONSE ;

    _response = newResponse ;
    sprintf(line,"%f",_response) ;
    shmText->write(line) ;
    visible = true ;
    t.reset(0.0,1);
    
  }
  return dtkNav::postFrame() ;
}

/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  return new joystickResponseNav(manager) ;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment ;
  return DTKDSO_UNLOAD_CONTINUE ;
}
