// DSO to write a lilne of text in head's up windows
// text is read from shared memory

// total number or rows on entire screen
#define ROWS (42)
#define COLS (81)
// where we're going to start- at the bottom
#define START (1)

#define SHM_TEXT_NAME "shmHeadsUpText" 
#define SHM_TEXT_SIZE (81)

#include <strings.h>

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class shmHeadsUpText : public dpfHeadsUpText {
public:
  shmHeadsUpText(dpf *app);
  ~shmHeadsUpText(void) ;
  int preConfig(void) ;
  int preFrame(void) ;
  int postFrame(void) ;   

private:
  dpf *app ;
  dpfDisplay *d ;
  dtkSharedMem *shm ;
  char line[SHM_TEXT_SIZE] ;
  u_int64_t count ;
};


shmHeadsUpText::shmHeadsUpText(dpf *app_in):
  dpfHeadsUpText(app_in, "shmHeadsUpText") {
  app = app_in ;
  d = app->display();
  setDescription("write text from shared memory in headsUp window") ;
  shm = NULL ;
  
  // see if shared memory already exists
  dtkSharedMem *old_shm = new dtkSharedMem(SHM_TEXT_NAME, 0) ;

  // it does, but is of wrong size
  if (old_shm && old_shm->isValid() && (old_shm->getSize()!=SHM_TEXT_SIZE)) {
    dtkMsg.add(DTKMSG_ERROR,"shmNavControl: shared "
	       "memory \"%s\" already exists, but wrong size.\n", SHM_TEXT_NAME) ;
    return ;
  }

  // it doesn't, so create a new one
  if ((!old_shm) || 
      (old_shm && old_shm->isInvalid())) { // have to create a new segment
    shm = new dtkSharedMem(SHM_TEXT_SIZE, SHM_TEXT_NAME) ;
    if(!shm || shm->isInvalid()) {
      dtkMsg.add(DTKMSG_ERROR,"shmHeadsUpText: unable to get valid shared "
		 "memory \"%s\".\n", SHM_TEXT_NAME) ;
      return ;
    }
  } else { // reuse old segment
    shm = old_shm ;
  }
  
  if (shm->queue())  {
    dtkMsg.add(DTKMSG_ERROR,"shmHeadsUpText: unable to set queued read\n") ;
    return ;
  }

  if (shm->writeCount(1)) {
    dtkMsg.add(DTKMSG_ERROR,"shmHeadsUpText: unable to set write count\n") ;
    return ;
  }
  
  // this seems to fix a problem of missing the first read for a new segment
  if (shm->write("")) {
    dtkMsg.add(DTKMSG_ERROR,"shmHeadsUpText: unable to write to shared "
	       "memory \"%s\".\n", SHM_TEXT_NAME) ;
    return ;
  }
  
  if (shm->read(line)) {
    dtkMsg.add(DTKMSG_ERROR,"shmHeadsUpText: unable to read shared "
	       "memory \"%s\".\n", SHM_TEXT_NAME) ;
    return ;
  }
  count = shm->getWriteCount() ;

}

shmHeadsUpText::~shmHeadsUpText(void) {
}


int shmHeadsUpText::preConfig(void) {

  if (d->pipes() <1) {
    dtkMsg.add(DTKMSG_ERROR,
	       "shmHeadsUpText::preConfig- no dpfPipes exist\n") ;
  } else {
    spew.shift() ;
    dpfPipe *p ;
    int pi ;
    dpfPwin *w ;
    int wi ;
    dpfScreen *s ;
    int si ;
    int n = 0 ;

    // look for screns with HPR of zero
    pi = 0 ;
    for (p=d->first(); p; p=p->next()) {
      wi = 0 ;
      for (w=p->first(); w; w=w->next()) {
	si = 0 ;
	for (s=w->first(); s; s=s->next()) {
	  pfCoord offset ;
	  offset=s->offset() ;
	  pfVec3 zero = pfVec3(0.0f, 0.0f, 0.0f) ;
	  if (offset.hpr == zero) {
	    spew.print("adding headsUp callback to "
		       "pipe(%d)->pwin(%d)->screen(%d)\n",
		       pi,wi,si) ;
	    s->addHeadsUp(this) ;
	    n++ ;
	  }
	  si++ ;
	}
	wi++ ;
      }
      pi++ ;
    }
    
    // if none are found, use first screen
    if (n==0) {
      s = d->first()->first()->first() ;
      if (s) {
	spew.print("adding headsUp callback to first screen\n") ;
	s->addHeadsUp(this) ;
	n++ ;
      }
    }

    spew.unshift() ;
  }

  if (init(ROWS, COLS)) {

    dtkMsg.add(DTKMSG_ERROR,
	       "shmHeadsUpText::preConfig- unable to init text\n") ;
    return dtkAugment::REMOVE_CALLBACK ;
  }

  return dpfHeadsUp::preConfig();
}

int shmHeadsUpText::preFrame(void) {

  // set area of screen to be used
  screenPos(0, START, 0, COLS-1) ;
  visible(0) ;

  return dtkAugment::REMOVE_CALLBACK;
}


int shmHeadsUpText::postFrame(void) {
  
  bzero(line, SHM_TEXT_SIZE) ;
  if (shm->read(line)) 
    return ERROR_ ;
  
  u_int64_t currentCount = shm->getWriteCount() ;
  
  if (currentCount == count) 
    return CONTINUE ;
  
  count = currentCount ;
  
  //printf("line = \"%s\"\n",line) ;
  if (strlen(line)==0)
    visible(0) ;
  else {
    clear() ;
    strcpy(text(START,1),line) ;
    flush() ;
    visible(1) ;
  }
  
  return dtkAugment::CONTINUE ;
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new shmHeadsUpText(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


