/* Just reads queued buttons and prints their value when changed */

#include <dtk.h>
#include <dtk/dtkDSO_loader.h>

class buttonRead : public dtkAugment {
public:
  buttonRead(dtkManager *) ;
  ~buttonRead(void) ;

  int preFrame(void) ;
  int postFrame(void) ;

private:

  dtkManager *mgr ;
  dtkInButton *button ;
  dtkDequeuer *dequeuer ;
  int buttons ;
} ;


buttonRead::buttonRead(dtkManager *m) :
  dtkAugment("buttonRead") {
  setDescription("reads buttons, queued, and reports changes") ;
  mgr = m ;
  dequeuer = NULL ;
  button = NULL ;

  // dtkAugment::dtkAugment() will not validate the object
  validate() ;
}


buttonRead::~buttonRead(void) {
  if (dequeuer) {
    delete dequeuer ;
    dequeuer = NULL ;
  }
}

int buttonRead::preFrame(void) {

  dequeuer = new dtkDequeuer(mgr->record()) ;
  if (!dequeuer || dequeuer->isInvalid()) 
    return ERROR_ ;
  
  button = (dtkInButton *) mgr->get("buttons", DTKINBUTTON_TYPE) ;
  if (!button) 
    return ERROR_ ;
  
  buttons = button->number() ;

  dtkMsg.add(DTKMSG_INFO, "buttonRead: "
	     "button device contains %d buttons\n", buttons) ;

  button->queue() ;
  
  // just call buttonRead::preFrame() once at the start.
  return REMOVE_CALLBACK ;
}


int buttonRead::postFrame(void) {

  char *state[2] = {"released", "pressed" } ;

  dtkRecord_event *e ;
  while ((e = dequeuer->getNextEvent(button))) {
    int whichButton ;
    int buttonState = button->read(&whichButton, e) ;

    dtkMsg.add(DTKMSG_INFO, "buttonRead: "
	       "button %d %s\n", whichButton,state[buttonState]) ;
  }
  return CONTINUE ;
}


/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  return new buttonRead(manager) ;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment ;
  return DTKDSO_UNLOAD_CONTINUE ;
}


