/* example of using slb: reads shared memory as set by slb 
   just uses the same envars as set for slb
*/

#include <stdlib.h>
#include <dtk.h>
#include <dtk/dtkDSO_loader.h>

class slbQueue : public dtkAugment {
public:
  slbQueue(dtkManager *) ;

  int preFrame(void) ;
  int postFrame(void) ;

private:

  dtkManager *mgr ;
  char *shmName[3] ;
  dtkSharedMem *shm[3] ;
  char value[3] ;
} ;


slbQueue::slbQueue(dtkManager *m) :
  dtkAugment("slbQueue") {
  setDescription("reads buttons, queued, and reports changes") ;
  mgr = m ;

  shmName[0] = "Button 0" ;
  shmName[1] = "Button 1" ;
  shmName[2] = "Button 2" ;

  // dtkAugment::dtkAugment() will not validate the object
  validate() ;
}


int slbQueue::preFrame(void) {
  
  for (int i=0; i<3; i++) {
    shm[i] =  new dtkSharedMem(shmName[i]) ;
    if (!shm[i] || shm[i]->isInvalid()) {
      dtkMsg.add(DTKMSG_ERROR,
		 "slbRead::preFrame() failed:"
		 " couldn't open shared memory segment\"%s\"\n",
		 shmName[i]) ; 
      return ERROR_ ;
    }
    shm[i]->queue() ;
    shm[i]->flush() ;
  }
  // just call slbQueue::preFrame() once at the start.
  return REMOVE_CALLBACK ;
}


int slbQueue::postFrame(void) {
  
  char *state[2] = {"released", "pressed" } ;

  for (int i=0; i<3; i++) {
    if (shm[i] && shm[i]->qread(&value[i])) 
      dtkMsg.add(DTKMSG_INFO,
		 "slbRead: %s %s\n",
		 shmName[i],state[(int)value[i]]) ;
  }
  return CONTINUE ;
}


/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  return new slbQueue(manager) ;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment ;
  return DTKDSO_UNLOAD_CONTINUE ;
}


