/* 

   This DIVERSE DSO turns on pfChannel::drawStats

   Stats are drawn on the channels of the dpfDisplay::masterScreen

   The environment variable DPF_STATS is set to the name of a one byte
   shared memory file.  If DPF_STATS isn't set, the shared memory segment
   named "stats" is used.  If "stats" doesn't exist, stats are always drawn.
   If stats exists and is zero, no stats are drawn.  If it is non-zero,
   stats are drawn.

   from the command ilne, set a zero value with:
     dtk-zeroShm stats
   if the segment already exists, or if it doesn't
     dtk-writeShm stats 1 1 %d 0

   and to a non-zero value with
     dtk-writeShm stats 1 1 %d 1

   John Kelso, NIST, kelso@nist.gov, 11/04

*/

#include <Performer/pf/pfCullProgram.h>

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class stats : public dtkAugment {
public:

  stats(dpf *app) ;
  ~stats(void) ;
  int preConfig(void);
  int postConfig(void);
  int preFrame(void);
  int postFrame(void);

private:
  dpfDisplay *display ;
  dtkSharedMem *shm ;
  unsigned char shmData ;
} ;

stats::stats(dpf *app) :
  dtkAugment("stats") {
  display = app->display();
  setDescription("turns on pfChanne::drawStats") ;
  validate();
} ;

stats::~stats(void) {
}

int stats::preConfig(void) {
  return dtkAugment::REMOVE_CALLBACK ;
}

void setCull(void) {
}

int stats::postConfig(void) {
  char *shmName ;
  shmName = getenv("DPF_STATS") ;
  if (!shmName)
    shmName = "stats" ;

  shm = NULL ;
  shm =  new dtkSharedMem(1, shmName, DTK_CONNECT) ;

  if (!shm || shm->isInvalid()) {
    shmData = 1 ;
    shm = NULL ;
    dtkMsg.add(DTKMSG_NOTICE,
	       "stats::postConfig() failed:"
	       " couldn't open shared memory segment\"%s\", stats always on.\n",
	       shmName) ; 
  } 

  return dtkAugment::REMOVE_CALLBACK ; 
}

int stats::preFrame(void) {
  return dtkAugment::REMOVE_CALLBACK ; 
}

int stats::postFrame(void) {
  
  if (shm) 
    shm->read(&shmData) ;
  //printf("%d           \r",(int)shmData) ;
  //fflush(stdout) ;
  if (shmData) {
    display->masterScreen()->pfchan(0)->drawStats() ;
  }
  //return dtkAugment::REMOVE_CALLBACK ; 
  return dtkAugment::CONTINUE ; 
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new stats(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
