#include <stdlib.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

static dtkInXKeyboard *keyboard = NULL ;
static bool testKeyCode(int code, char *name) {
  if (!keyboard->getKeyCode(code)) {
    dtkMsg.add(DTKMSG_ERROR,
	       "testKeyCodes: no X Keycap for %s\n",
	       name); 
    return false ;
  } else {
    return true ;
  }
}

class testKeyCodes : public dtkAugment
{
public:

  testKeyCodes(dpf *app);
  int postConfig(void);   

private:


  dpf *app;
};


testKeyCodes::testKeyCodes(dpf *app_in) :
dtkAugment("testKeyCodes")
{
  // dtkAugment starts out invalid.

  app = app_in;
  setDescription("tests X keycodes") ;

  validate(); // dtkAugment starts out invalid.
}

int testKeyCodes::postConfig(void)
{
  if(!(keyboard = static_cast<dtkInXKeyboard *>
       (app->getNext(DTKINXKEYBOARD_TYPE))))
    return dtkAugment::ERROR_;

  keyboard->queue();


  //////////////// get X key codes from key syms ///////////////////////

  // these are fatal if not found
  testKeyCode(XK_Alt_L, "XK_Alt_L") ;
  testKeyCode(XK_Alt_L, "XK_Alt_L") ;
  testKeyCode(XK_Alt_R, "XK_Alt_R") ;
  //testKeyCode(XK_Mode_switch, "XK_Mode_switch") ;
  testKeyCode(XK_Control_L, "XK_Control_L") ;
  testKeyCode(XK_Control_R, "XK_Control_R") ;
  testKeyCode(XK_Shift_L, "XK_Shift_L") ;
  testKeyCode(XK_Shift_R, "XK_Shift_R") ;
  testKeyCode(XK_space, "XK_space") ;
  testKeyCode(XK_less, "XK_less") ;
  testKeyCode(XK_greater, "XK_greater") ;
  testKeyCode(XK_R, "XK_R") ;
  testKeyCode(XK_Home, "XK_Home") ;
  testKeyCode(XK_Left, "XK_Left") ;
  testKeyCode(XK_Up, "XK_Up") ;
  testKeyCode(XK_Right, "XK_Right") ;
  testKeyCode(XK_Down, "XK_Down") ;
  testKeyCode(XK_W, "XK_W") ;
  testKeyCode(XK_H, "XK_H") ;
  testKeyCode(XK_J, "XK_J") ;

  #if 0
  // if these aren't found, it tries laptop mode, not fatal
  testKeyCode(XK_KP_4, "XK_KP_4") ;
  testKeyCode(XK_KP_Left, "XK_KP_Left") ;
  testKeyCode(XK_KP_6, "XK_KP_6") ;
  testKeyCode(XK_KP_Right, "XK_KP_Right") ;
  testKeyCode(XK_KP_2, "XK_KP_2") ;
  testKeyCode(XK_KP_Down, "XK_KP_Down") ;
  testKeyCode(XK_KP_8, "XK_KP_8") ;
  testKeyCode(XK_KP_Up, "XK_KP_Up") ;
  testKeyCode(XK_KP_5, "XK_KP_5") ;
  testKeyCode(XK_KP_Begin, "XK_KP_Begin") ;
  testKeyCode(XK_KP_1, "XK_KP_1") ;
  testKeyCode(XK_KP_End, "XK_KP_End") ;
  testKeyCode(XK_KP_Add, "XK_KP_Add") ;
  testKeyCode(XK_KP_Subtract, "XK_KP_Subtract") ;
#endif

  // these are keypad mode keys, and fatal if not found
  testKeyCode(XK_I, "XK_I") ;
  testKeyCode(XK_P, "XK_P") ;
  testKeyCode(XK_L, "XK_L") ;
  testKeyCode(XK_K, "XK_K") ;
  testKeyCode(XK_9, "XK_9") ;
  testKeyCode(XK_0, "XK_0") ;
  testKeyCode(XK_O, "XK_O") ;
  testKeyCode(XK_plus, "XK_plus") ;
  testKeyCode(XK_minus, "XK_minus") ;
  
  return dtkAugment::CONTINUE;
}

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new testKeyCodes(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


