// this DSO toggles the files visageScreens.savg and vistaScreen.savg

// repeated keypresses display:
// 0: neither file displayed
// 1: visageScreens.savg displayed
// 2: vistaScreen.savg displayed
// 3: both visageScreens.savg and  vistaScreen.savg displayed

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class toggleWalls : public dtkAugment {
public:
  
  toggleWalls(dpf *app) ;
  ~toggleWalls(void) ;
  int preFrame(void) ;   
  int postFrame(void) ;   
  
private:
  dpf *app ;
  unsigned int mode ;
  pfNode *visage ;
  pfNode *vista ;
  dtkInXKeyboard *kb;
  KeyCode xkc_n;
  KeyCode xkc_s;
  dtkDequeuer *dq;
  bool shiftKeyPressed;
  pfScene *scene ;
  pfGroup *group ;
} ; 


toggleWalls::toggleWalls(dpf *app_in) : dtkAugment("toggleWalls") {
  app = app_in ;
  setDescription("toggles the files visageScreens.savg and vistaScreen.savg") ;
  kb = NULL;
  dq = NULL ;
  mode = 0 ;
  shiftKeyPressed = false ;
  // dtkAugment starts out invalid.
  validate() ; 
}


toggleWalls::~toggleWalls(void) {
  if(dq) {
    delete dq;
    dq = NULL;
  }
  invalidate() ;
}

int toggleWalls::preFrame(void) {

  kb = (dtkInXKeyboard *) app->get("xkeyboard", DTKINXKEYBOARD_TYPE);
  if(!kb) return dtkAugment::ERROR_ ;
  
  kb->queue();
 
  if(!(xkc_s = kb->getKeyCode(XK_S))) {
    dtkMsg.add(DTKMSG_ERROR,"ToggleWalls::preFrame() failed"
	       " to get x key code for XK_S.\n");
    return dtkAugment::ERROR_ ;
  }
  if(!(xkc_n = kb->getKeyCode(XK_N))) {
    dtkMsg.add(DTKMSG_ERROR,"ToggleWalls::preFrame() failed"
	       " to get x key code for XK_N.\n");
    return dtkAugment::ERROR_ ;
  }

  dq = new dtkDequeuer(app->record());
  if(!dq || dq->isInvalid()) return  dtkAugment::ERROR_;

  scene = app->display()->scene() ;
  if (!scene) {
    return dtkAugment::ERROR_;  
  }

  group = new pfGroup ;
  app->display()->addGhostNode(group) ;
  scene->addChild(group) ;

  visage = pfdLoadFile("visageScreens.savg") ;
  vista = pfdLoadFile("vistaScreen.savg") ;

  if (!vista || !visage) {
    dtkMsg.add(DTKMSG_ERROR,"ToggleWalls::preFrame() failed"
	       " to load wall files\n");
    return dtkAugment::ERROR_ ;
  }

  return dtkAugment::REMOVE_CALLBACK;
}


int toggleWalls::postFrame(void) {

  dtkRecord_event *event;

  while ((event = dq->getNextEvent(kb))) {
    KeyCode keyCode;
    
    int pressed = kb->read(&keyCode, event);
    
    if (keyCode == xkc_s) {
      if (pressed)
	shiftKeyPressed = true ;
      else
	shiftKeyPressed = false ;
    } else if (keyCode == xkc_n && shiftKeyPressed && pressed) {
      mode = (++mode)%4 ;
      if (mode == 0) {
	if (group->getNumChildren() != 0) {
	  group->removeChild(visage) ;
	  group->removeChild(vista) ;
	}
	dtkMsg.add(DTKMSG_INFO,"ToggleWalls: visage and vista walls turned off\n") ;
      } else if (mode == 1) {
	group->addChild(visage) ;
	dtkMsg.add(DTKMSG_INFO,"ToggleWalls: visage walls turned on\n") ;
      } else if (mode == 2) {
	group->removeChild(visage) ;
	group->addChild(vista) ;
	dtkMsg.add(DTKMSG_INFO,"ToggleWalls: vista wall turned on\n") ;
      } else if (mode == 3) {
	group->addChild(visage) ;
	dtkMsg.add(DTKMSG_INFO,"ToggleWalls: visage and vista walls turned on\n") ;
      }
    }
  }
  return dtkAugment::CONTINUE ;
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new toggleWalls(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


