#include <math.h>
#include <values.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

//static float getFloat(char *prompt, float now) ;

#define DPF_INCHES (84.f/2.f)

// DSO to set default graphics state for the NIST Boulder Power Wall

// The Power Wall has a single screen 105" feet on a side, with the floor
// 1.5" off the ground.  The screen has an aspect ration of 5:4, same as the
// projector, which is 1280/1024, so the vertical display height is 84"

// one DIVERSE unit is 84"/2, or 42*0.0254 = 1.0688

// the screen is -1 to 1 in Y, -1.25 to 1.25 in Y

// the entire pipe/window/screen maps to the screen

/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  dpfDisplay *display = app->display();
  if(display == NULL) return DTKDSO_LOAD_ERROR;

  display->meters(DPF_INCHES*.0254f) ;

  dpfPipe *pipe ;
  dpfPwin *pwin ;
  dpfScreen *screen ;

  pfCoord offset ;

  printf("\n\n\nVISTA DISPLAY\n\n\n") ;

  // one pipe
  if (display->pipes(1)) return DTKDSO_LOAD_ERROR;

  pipe = display->first() ;

  // onewindow in the pipe
  pipe->pwins(1) ;

  // Rave window, all of the screen
  pwin = pipe->first() ;

  pwin->fullScreen(1) ;
  pwin->noBorder(1) ;
  pwin->stereo(1) ;

  // set this to one to make cursor invisible!!??  8^)
  pwin->cursor(0) ;

  pwin->screens(1) ;

  screen = pwin->first() ;

  screen->simpleFrustum(0) ;

  offset.xyz = pfVec3(0.0f, 0.0f, 0.0f) ;
  offset.hpr = pfVec3(0.0f, 0.0f, 0.0f) ;
  screen->width(2.0f*1.25f) ;
  screen->height(2.0f) ;
  screen->offset(offset) ;
  screen->center(pfVec3(0.0f, 
			1.0f, 
			0.f+(1.5f/DPF_INCHES))) ;
  return DTKDSO_LOAD_UNLOAD;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  return DTKDSO_UNLOAD_CONTINUE;
}


