//
// DIVERSE graphics interface -- Performer ---- version 2.2.0
// released: Fri Oct 18 09:27:29 EDT 2002
// The DIVERSE HOMPAGE is at: www.diverse.vt.edu
// 
// Copyright (C) 2000, 2001, 2002  John Kelso, Lance Arsenault and Ron Kriz
// 
// This software, DIVERSE graphics interface -- Performer (DGIPF),
// is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License (LGPL) as
// published by the Free Software Foundation; either version 2.1 of
// the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this software, in the top level source directory in
// a file named "COPYING"; if not, see it at:
// http://www.gnu.org/copyleft/lesser.html or write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//
// This loads a pointer at the position and orientation of the wand
// and is based on the original wandAxis DSO

#include <stdlib.h>
#include <math.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class wandPointer : public dtkAugment
{
public:

  wandPointer(dpf *app) ;
  int postConfig(void) ;   
  int postFrame(void) ;   

private:

  dpf *app ;
  dtkInLocator *wand;
  float wandPos[6] ;
  //  Shared memory segment used to request the wand pointer from another program
  dtkSharedMem* shm;
  dpfToggle* tog;
  unsigned char toggleChar;
  unsigned char selected;
  char* nameOfWand;
} ; 


wandPointer::wandPointer(dpf *app_in) : dtkAugment("wandPointer")
{
  app = app_in ;
  setDescription("adds a axis object at positon and direction of the wand") ;
  // dtkAugment starts out invalid.
  wandPos[0] = wandPos[1] = wandPos[2] = wandPos[3] = 
    wandPos[4] = wandPos[5] = 0.0f ;

  selected = 2;
  toggleChar = 1;
  shm = new dtkSharedMem(sizeof(unsigned char), "pointerOn");
  if(!shm || shm->isInvalid()) {
      shm = NULL;
  }
  else {
    shm->flush();
    shm->write(&selected);
  }

  validate() ;
}


int wandPointer::postConfig(void)
{
  // see if the user wants to use a different wand object
  char *env = getenv("wandName");

  if (env) {
    nameOfWand = strdup(env) ;
  }
  else {
    nameOfWand = "wandArrow.pfb";
  }

  printf("name of the wand object  %s\n", nameOfWand);

  dpfDisplay *display = app->display();
  if(!display) 
    return DTK_CALLBACK_ERROR ;
  if (!display->scene()) 
    return DTK_CALLBACK_ERROR ;

  char *path = (char *) pfGetFilePath() ;
  if(path)
    // copy path from static memory to the heap.
    path = dtk_strdup(path);
  pfFilePath(".:"DPF_DATA_DIR);
  //cout << "see where wandpointer fails: " << endl;
  pfGroup *dcs =
    new dpfDCS(wandPos, /* position */
	       1.0f, /* overall scale */
	       /* file       scale xyz     position xyz  rotation hpr */
	       nameOfWand, DPF_IDENTITY,
	       NULL);
  if(path)
    {
      pfFilePath(path);
      free(path);
    }
  //display->scene()->addChild(dcs) ;

  wand = (dtkInLocator *) app->get("wand", DTKINLOCATOR_TYPE);
  if(!wand) return DTK_CALLBACK_ERROR ;

  // set up toggle
  //toggleChar = 2;
  // start wand off in the turned off position
  toggleChar = 1;
  tog = new dpfToggle(&toggleChar, 1);
  tog->addChild(dcs);
  display->scene()->addChild(tog) ;
  //cout << "end of postconfig" << endl;
  return DTK_REMOVE_CALLBACK;
}

int wandPointer::postFrame(void)
{
  //cout << "start of postFrame" << endl;
  wand->read(wandPos);
  //cout << "read the wand" << endl;

  // read the shared memory to see if a sound is necessary
  if (shm)
    shm->read(&selected);

  if (selected == 2) {
    // hide the wand
    //cout << "hide the wand" << endl; 
    toggleChar = 1;
    }
  else if (selected == 1) {
    // show the wand
    //cout << "show the wand" << endl; 
    toggleChar = 2;
  }

  return DTK_CONTINUE ;
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new wandPointer(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}


