/*
 *       Scientific Applications & Visualization Group
 *            SAVG web page: http://math.nist.gov/mcsd/savg/
 *       Mathematical & Computational Science Division
 *            MCSD web page: http://math.nist.gov/mcsd/
 *       National Institute of Standards and Technology
 *            NIST web page: http://www.nist.gov/
 * 
 *  
 *  This software was developed at the National Institute of Standards
 *  and Technology by employees of the Federal Government in the course
 *  of their official duties. Pursuant to title 17 Section 105 of the
 *  United States Code this software is not subject to copyright
 *  protection and is in the public domain.  isolate is an experimental 
 *  system.  NIST assumes no responsibility whatsoever for its use by 
 *  other parties, and makes no guarantees, expressed or implied, about 
 *  its quality, reliability, or any other characteristic.
 *  
 *  We would appreciate acknowledgement if the software is used.
 * 
 */


/************************************************
 * savg-style           			*
 * Created by: Adele Peskin		        *
 * Created on: 3-24-05				*
 * Current Version: 1.00			*
 *						*
 * Description:					*
 *    savg-style changes the style between      *
 * filled and wireframe                         *
 ************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "feed.h"
#include "savgLineEvaluator.c"

/* Prints information on usage*/
void usage();

int main(argc, argv)
int argc;
char** argv;
{
/*
 * INITIALIZATION
 */
  char line[STRING_LENGTH];
  int style = 0; 
  int pargc = 0;
  char **pargv  = NULL;

/*
 * COMMAND LINE PROCESSING
 */

  // there should be exactly 1 parameter, wire or filled
  if (argc != 2) {
    usage();
    exit(EXIT_FAILURE);
  }

  if(strcasecmp(argv[1], "wire") == 0)
    style = 1;
  else if (strcmp(argv[1], "filled") == 0)
    style = 2;
  else {
    usage();
    exit(EXIT_FAILURE);
  }

/*
 * BODY
 */

  // print all of the new data lines
  if (style == 1)
    printf("style wireframe\n");
  else
    printf("style filled\n");
 
  while(getLine(line)) {
    // print every line unless it is another striplength line

    // if it is a comment just print it
    if (containsComment(line))
      printf("%s\n",line);

    else {
      // parse and check each line
      dpf_charsToArgcArgv(line, &pargc, &pargv);
      
      if (pargc > 0) {
	if (strcasecmp("style",pargv[0])!=0)
	  printf("%s\n",line);
      }
      else
	printf("\n");
    }
  }
   
  return 0;
}

/* Prints information on usage*/
void usage()
{
	fprintf(stderr,
	"ERROR: Incorrect command line arguments\n"
	"\n"
	"USAGE:\n"
	"\tsavg-style wire | savg-style filled\n");
}
